@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,18 +24,54 @@
 #include <config.h>
 #endif
 
-#include "NonCopyable.h"
+/*
+This class gives any SAP (Service Access Point) the semantics of auto_ptr.
+*/
 
 template<typename T>
 class AutoClosingSAP : public T
 {
-	DECLARE_NON_COPYABLE(AutoClosingSAP)
 public:
 	AutoClosingSAP() {}
 	
-	AutoClosingSAP(T const& sap) : T(sap) {}
+	explicit AutoClosingSAP(T& sap);
+	
+	AutoClosingSAP(AutoClosingSAP& other);
 	
 	~AutoClosingSAP() { this->close(); }
+	
+	AutoClosingSAP& operator=(AutoClosingSAP& rhs);
+	
+	void release() { static_cast<T&>(*this) = T(); }
 };
 
+
+template<typename T>
+AutoClosingSAP<T>::AutoClosingSAP(T& sap)
+:	T(sap)
+{
+	sap = T();
+}
+
+template<typename T>
+AutoClosingSAP<T>::AutoClosingSAP(AutoClosingSAP& other)
+{
+	T& to = *this;
+	T& from = other;
+	to = from;
+	from = T();
+}
+
+template<typename T>
+AutoClosingSAP<T>&
+AutoClosingSAP<T>::operator=(AutoClosingSAP& rhs)
+{
+	T& to = *this;
+	T& from = rhs;
+	to.close();
+	to = from;
+	from = T();
+	return *this;
+}
+
 #endif
