@@ -221,7 +221,8 @@ SplittableBuffer::appendDestructive(SplittableBuffer& buf)
 		// IntrusivePtr is also just a pointer, so we are not copying
 		// any atomic counters or anything like that.
 		
-		(char*&)dst_range->spanEnd += src_used;
+		char* dst_span_end = (char*)dst_range->spanEnd + src_used;
+		dst_range->spanEnd = (BString*)dst_span_end;
 		src_range->spanBegin = src_range->spanEnd;
 		src_range->prev->next = src_range->next;
 		src_range->next->prev = src_range->prev;
