@@ -63,29 +63,42 @@ public:
 		BString const& script, char const* filename, int lineno);
 	
 	void setListener(Listener& listener) {
-		m_listenerList.clear();
-		m_listenerList.push_back(listener);
+		m_listenerLink.setObserver(&listener);
 	}
 	
 	void removeListener() {
-		m_listenerList.clear();
+		m_listenerLink.setObserver(0);
 	}
 	
 	Listener* getListener() {
-		return m_listenerList.empty() ? 0 : &m_listenerList.front();
+		if (m_listenerSuspenderLink.getObserver()) {
+			return 0;
+		}
+		return m_listenerLink.getObserver();
+	}
+	
+	void setListenerSuspender(ListenerSuspender& suspender) {
+		m_listenerSuspenderLink.setObserver(&suspender);
+	}
+	
+	void removeListenerSuspender() {
+		m_listenerSuspenderLink.setObserver(0);
+	}
+	
+	ListenerSuspender* getListenerSuspender() {
+		return m_listenerSuspenderLink.getObserver();
 	}
 	
 	void setPageOpenListener(PageOpenListener& listener) {
-		m_poListenerList.clear();
-		m_poListenerList.push_back(listener);
+		m_poListenerLink.setObserver(&listener);
 	}
 	
 	void removePageOpenListener() {
-		m_poListenerList.clear();
+		m_poListenerLink.setObserver(0);
 	}
 	
 	PageOpenListener* getPageOpenListener() {
-		return m_poListenerList.empty() ? 0 : &m_poListenerList.front();
+		return m_poListenerLink.getObserver();
 	}
 private:
 	enum { STACK_SIZE = 15*1024 };
@@ -174,8 +187,9 @@ private:
 	static JSClass m_sDivClass;
 	static JSClass m_sMimetypeClass;
 	static JSClass m_sPluginClass;
-	IntrusiveList<Listener> m_listenerList;
-	IntrusiveList<PageOpenListener> m_poListenerList;
+	SingleObserverLink<Listener> m_listenerLink;
+	SingleObserverLink<ListenerSuspender> m_listenerSuspenderLink;
+	SingleObserverLink<PageOpenListener> m_poListenerLink;
 	JSContext* m_pContext;
 	JSObject* m_pWindowObj;
 	JSObject* m_pDocumentObj;
@@ -1042,6 +1056,22 @@ JsEnvironment::removeListener()
 	}
 }
 
+void
+JsEnvironment::setListenerSuspender(ListenerSuspender& suspender)
+{
+	if (m_ptrContext.get()) {
+		m_ptrContext->setListenerSuspender(suspender);
+	}
+}
+
+void
+JsEnvironment::removeListenerSuspender()
+{
+	if (m_ptrContext.get()) {
+		m_ptrContext->removeListenerSuspender();
+	}
+}
+
 void
 JsEnvironment::setPageOpenListener(PageOpenListener& listener)
 {
