@@ -25,7 +25,7 @@
 #endif
 
 #include "NonCopyable.h"
-#include "IntrusiveList.h"
+#include "Observer.h"
 #include <stddef.h>
 #include <memory>
 #include <string>
@@ -43,6 +43,7 @@ class JsEnvironment
 	DECLARE_NON_COPYABLE(JsEnvironment)
 public:
 	class Listener;
+	class ListenerSuspender;
 	class PageOpenListener;
 	
 	JsEnvironment(URI const& page_url);
@@ -61,6 +62,10 @@ public:
 	
 	void removeListener();
 	
+	void setListenerSuspender(ListenerSuspender& suspender);
+	
+	void removeListenerSuspender();
+	
 	void setPageOpenListener(PageOpenListener& listener);
 	
 	void removePageOpenListener();
@@ -74,9 +79,8 @@ private:
 };
 
 
-class JsEnvironment::Listener : private IntrusiveListNode
+class JsEnvironment::Listener : public ObserverBase
 {
-	friend class IntrusiveList<Listener>;
 public:
 	virtual ~Listener() {}
 	
@@ -87,9 +91,14 @@ public:
 	virtual void processInnerHtmlAssignment(char const* data) = 0;
 };
 
-class JsEnvironment::PageOpenListener : private IntrusiveListNode
+
+class JsEnvironment::ListenerSuspender : public ObserverBase
+{
+};
+
+
+class JsEnvironment::PageOpenListener : public ObserverBase
 {
-	friend class IntrusiveList<PageOpenListener>;
 public:
 	virtual ~PageOpenListener() {}
 	
