@@ -22,13 +22,12 @@
 #endif
 
 #include "prthread.h"
-#include "types.h"
-#include <ace/OS_NS_Thread.h>
+#include <errno.h>
 
 PRThread* PR_GetCurrentThread()
 {
-	ACE_thread_t id = ACE_OS::thr_self();
-	return reinterpret_cast<PRThread*>((uintptr_t(id) + 1) << 1);
 	// libjs expects thread ids not to be 0
-	// but always have the least significant bit clear
+	// but always have the least significant bit clear.
+	// errno fits perfectly here.
+	return reinterpret_cast<PRThread*>(&errno);
 }
