@@ -24,11 +24,12 @@
 #include <config.h>
 #endif
 
+#include "Observer.h"
 #include "AutoClosingSAP.h"
 
 class ACE_SOCK_Stream;
 
-class AsyncConnectorListener
+class AsyncConnectorListener : public ObserverBase
 {
 public:
 	enum FailureCode {
@@ -41,9 +42,9 @@ public:
 	
 	virtual ~AsyncConnectorListener() {}
 	
-	virtual void connectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm) = 0;
+	virtual void onConnectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm) = 0;
 	
-	virtual void connectionFailed(FailureCode fcode, int errnum) = 0;
+	virtual void onConnectionFailed(FailureCode fcode, int errnum) = 0;
 };
 
 #endif
