@@ -66,7 +66,8 @@ class Client::ResponseHandler :
 public:
 	ResponseHandler(
 		ClientWeakPtr const& client_ptr, ConstRequestPtr const& request,
-		RequestTag const& request_tag, long id, bool conn_close);
+		RequestTag const& request_tag, long id,
+		HttpStreamWriter::Connection conn_persistence);
 	
 	virtual ~ResponseHandler();
 private:
@@ -86,10 +87,10 @@ private:
 	ConstRequestPtr const m_ptrRequest;
 	RequestTag m_requestTag;
 	long const m_id;
-	bool m_isConnClose;
 	bool m_isEOF;
 	uintmax_t m_producedDataSize;
 	HttpStreamWriter m_streamWriter;
+	HttpStreamWriter::Connection m_connPersistence;
 };
 
 
@@ -222,19 +223,26 @@ Client::processInputData(bool eof)
 		SplittableBuffer body_data;
 		bool body_eof = false;
 		bool const empty_input = m_inputData.empty();
-		bool const message_finishes = processNewData(m_inputData, eof, body_data, body_eof);
-		if (message_finishes) {
+		Status const status = processNewData(
+			m_inputData, eof, body_data, body_eof
+		);
+		
+		if (status == MESSAGE_ENDS) {
 			DBG_INCOMING_REQUEST_END();
 		}
+
 		if (!body_data.empty() || body_eof) {
 			ScopedSuspender suspender(*this);
+			
+			// This may result in a recursive invocation
+			// of reactor event loop.
 			m_ptrRequestHandler->appendBodyData(body_data, body_eof);
 		}
-		if (message_finishes) {
+		
+		if (status == MESSAGE_ENDS) {
 			m_ptrRequestHandler.reset(0);
 			m_ptrRequestURI.reset(0);
-		}
-		if (isErrorState()) {
+		} else if (status == ERROR_STATE) {
 			DBG_INCOMING_REQUEST_ERROR();
 			//DEBUGLOG("ERROR parsing request");
 			IntrusivePtr<ErrorDescriptor> edesc(errorDescriptor());
@@ -269,7 +277,8 @@ Client::processInputData(bool eof)
 				IntrusivePtr<ResponseHandler> response_handler(
 					new ResponseHandler(
 						m_ptrSelf, request, request_tag,
-						++m_responseIdCounter, true /* conn_close */
+						++m_responseIdCounter,
+						HttpStreamWriter::CONN_CLOSE
 					)
 				);
 				ScopedSuspender suspender(*this);
@@ -280,6 +289,9 @@ Client::processInputData(bool eof)
 					)
 				);
 				SplittableBuffer body_data;
+				
+				// This may result in a recursive invocation
+				// of reactor event loop.
 				request_handler->appendBodyData(body_data, true);
 			}
 			
@@ -288,9 +300,9 @@ Client::processInputData(bool eof)
 			m_inputData.clear();
 			break;
 		}
-		if (!message_finishes) {
-			// This means more input data is needed.
-			// Example: the input ends in the middle of a header.
+		
+		if (status == NEED_MORE_DATA) {
+			// Can happen for example if input ends in the middle of a header.
 			break;
 		}
 	}
@@ -468,7 +480,9 @@ Client::eventGotMetadata(auto_ptr<HttpRequestMetadata> metadata, bool is_persist
 	IntrusivePtr<ResponseHandler> response_handler(
 		new ResponseHandler(
 			m_ptrSelf, request, request_tag,
-			++m_responseIdCounter, !is_persistent
+			++m_responseIdCounter,
+			is_persistent ? HttpStreamWriter::CONN_KEEP_ALIVE
+			: HttpStreamWriter::CONN_CLOSE
 		)
 	);
 	ScopedSuspender suspender(*this);
@@ -531,15 +545,16 @@ Client::handleWrite(ACE_HANDLE)
 
 Client::ResponseHandler::ResponseHandler(
 	ClientWeakPtr const& client_ptr, ConstRequestPtr const& request,
-	RequestTag const& request_tag, long id, bool conn_close)
+	RequestTag const& request_tag, long id,
+	HttpStreamWriter::Connection conn_persistence)
 :	m_ptrClient(client_ptr),
 	m_ptrRequest(request),
 	m_requestTag(request_tag),
 	m_id(id),
-	m_isConnClose(conn_close),
 	m_isEOF(false),
 	m_producedDataSize(0),
-	m_streamWriter(*this)
+	m_streamWriter(*this),
+	m_connPersistence(conn_persistence)
 {
 }
 
@@ -552,7 +567,8 @@ Client::ResponseHandler::processProvisionalResponse(std::auto_ptr<HttpResponseMe
 {
 	if (m_ptrRequest->requestLine().getHttpVersion() >= HttpVersion::HTTP_1_1) {
 		m_streamWriter.startResponse(
-			*metadata, m_ptrRequest->requestLine().getHttpVersion(), false
+			*metadata, m_ptrRequest->requestLine(),
+			HttpStreamWriter::CONN_KEEP_ALIVE
 		);
 		SplittableBuffer body_data;
 		m_streamWriter.appendBodyData(body_data, true);
@@ -566,9 +582,10 @@ Client::ResponseHandler::processResponseMetadata(std::auto_ptr<HttpResponseMetad
 		client->setResponseInProgress(true);
 	}
 	m_streamWriter.startResponse(
-		*metadata, m_ptrRequest->requestLine().getHttpVersion(), m_isConnClose
+		*metadata, m_ptrRequest->requestLine(),
+		m_connPersistence
 	);
-	m_isConnClose |= m_streamWriter.isConnCloseSet();
+	m_connPersistence = m_streamWriter.getConnPersistence();
 
 #ifdef DEBUG
 	typedef AbstractDebugAgent ADA;
@@ -627,7 +644,7 @@ Client::ResponseHandler::processError(IntrusivePtr<ErrorDescriptor> const& edesc
 		m_streamWriter.reset();
 		if (Client* client = m_ptrClient.get()) {
 			if (client->isReadingForbidden()) {
-				m_isConnClose = true;
+				m_connPersistence = HttpStreamWriter::CONN_CLOSE;
 			}
 		}
 		auto_ptr<CraftedResponse> response = edesc->responseGenerator()();
@@ -642,10 +659,10 @@ Client::ResponseHandler::processError(IntrusivePtr<ErrorDescriptor> const& edesc
 			response->metadata().setBodyStatus(HttpResponseMetadata::BODY_FORBIDDEN);
 			response->body().clear();
 		}
+		
 		m_streamWriter.startResponse(
-			response->metadata(),
-			m_ptrRequest->requestLine().getHttpVersion(),
-			m_isConnClose
+			response->metadata(), m_ptrRequest->requestLine(),
+			m_connPersistence
 		);
 		DBG_ERROR_OUTGOING_RESPONSE_BEGIN(response->metadata());
 		m_streamWriter.appendBodyData(response->body(), true);
@@ -665,7 +682,10 @@ Client::ResponseHandler::processNewData(SplittableBuffer& data, bool eof)
 	m_producedDataSize += data.size();
 	
 	if (Client* client = m_ptrClient.get()) {
-		bool stream_eof = eof && (m_isConnClose || client->isReadingForbidden());
+		bool stream_eof = eof && (
+			m_connPersistence == HttpStreamWriter::CONN_CLOSE
+			|| client->isReadingForbidden()
+		);
 		client->processResponseData(data, stream_eof, m_id);
 		if (eof) {
 			client->setResponseInProgress(false);
