@@ -204,7 +204,7 @@ Debug::httpMessageBegin(
 	}
 	
 	ostringstream strm;
-	metadata.requestLine().toStream(strm, false);
+	metadata.requestLine().toStream(strm);
 	metadata.headers().toStream(strm);
 	
 	httpMessageBegin(type, metadata.getClientRequestId(), strm.str());
