@@ -45,7 +45,9 @@ namespace ErrorCodes
 		WRONG_RESPONSE_STATUS, // } when fetching external scripts
 		SCRIPT_FETCH_NOFILTER, //
 		REQUEST_CANCEL_TO_SAVE_TRAFFIC, // used by response analyzers
-		REMOTE_PROXY_CONFIG_PROBLEM
+		REMOTE_PROXY_CONFIG_PROBLEM,
+		SOCKS_PROXY_ERROR,
+		TUNNEL_PORT_NOT_ALLOWED
 	};
 }
 
