@@ -32,6 +32,7 @@
 #include "StringUtils.h"
 #include "HttpStatusLine.h"
 #include "ArraySize.h"
+#include "HtmlEscaper.h"
 #include <ace/config-lite.h>
 #include <ace/OS_NS_netdb.h>
 #include <ace/OS_NS_unistd.h>
@@ -233,6 +234,25 @@ ErrorResponse::errRemoteProxyConfig(string url, string problem)
 	return response;
 }
 
+auto_ptr<CraftedResponse>
+ErrorResponse::errSocksProxyError(
+	std::string url, std::string host, int port, std::string problem)
+{
+	static char const* templ[] =
+#	include "ERR_SOCKS_PROXY_ERROR.inc"
+	MacroMap m;
+	prepareCommonMacros(m);
+	m[BString("@URL@")] = url;
+	m[BString("@HOST@")] = host;
+	m[BString("@PORT@")] = StringUtils::fromNumber(port);
+	m[BString("@PROBLEM@")] = problem;
+	auto_ptr<CraftedResponse> response(
+		new CraftedResponse(HttpStatusLine::SC_SERVICE_UNAVAILABLE)
+	);
+	prepareContent(*response, templ, m);
+	return response;
+}
+
 auto_ptr<CraftedResponse>
 ErrorResponse::errDummyError()
 {
@@ -262,14 +282,17 @@ ErrorResponse::prepareContent(
 			MacroMap::const_iterator fit =
 				macros.find(BString(empty, str, str + len));
 			if (fit != macros.end()) {
-				strm << fit->second;
+				strm << HtmlEscaper::escape(fit->second);
 			}
 		} else {
 			strm.write(str, len);
 		}
 	}
 
-	response.metadata().headers().setHeader(BString("Content-Type"), BString("text/html"));
+	HttpHeadersCollection& headers = response.metadata().headers();
+	headers.setHeader(BString("Content-Type"), BString("text/html"));
+	headers.setHeader(BString("Cache-Control"), BString("no-cache, must-revalidate"));
+	headers.setHeader(BString("X-BFilter-Content"), BString("generated"));
 	strm.swapData(response.body());
 }
 
