@@ -44,23 +44,35 @@ public:
 	static std::auto_ptr<CraftedResponse> errHostNameResolution(
 		std::string url, std::string host);
 	
-	static std::auto_ptr<CraftedResponse> errServerTimeout(std::string url);
+	static std::auto_ptr<CraftedResponse> errServerTimeout(
+		std::string url);
 	
-	static std::auto_ptr<CraftedResponse> errParsingRequest(std::string url, std::string problem);
+	static std::auto_ptr<CraftedResponse> errParsingRequest(
+		std::string url, std::string problem);
 	
-	static std::auto_ptr<CraftedResponse> errParsingResponse(std::string url, std::string problem);
+	static std::auto_ptr<CraftedResponse> errParsingResponse(
+		std::string url, std::string problem);
 	
-	static std::auto_ptr<CraftedResponse> errUnsupportedProtocol(std::string url, std::string protocol);
+	static std::auto_ptr<CraftedResponse> errUnsupportedProtocol(
+		std::string url, std::string protocol);
 	
-	static std::auto_ptr<CraftedResponse> errUnsupportedMethod(std::string url, std::string method);
+	static std::auto_ptr<CraftedResponse> errUnsupportedMethod(
+		std::string url, std::string method);
 	
-	static std::auto_ptr<CraftedResponse> errExpectationFailed(std::string url, std::string expectation);
+	static std::auto_ptr<CraftedResponse> errExpectationFailed(
+		std::string url, std::string expectation);
 	
-	static std::auto_ptr<CraftedResponse> errGenericError(std::string url, std::string error, int status_code);
+	static std::auto_ptr<CraftedResponse> errGenericError(
+		std::string url, std::string error, int status_code);
 	
-	static std::auto_ptr<CraftedResponse> errUrlForbidden(std::string url);
+	static std::auto_ptr<CraftedResponse> errUrlForbidden(
+		std::string url);
 	
-	static std::auto_ptr<CraftedResponse> errRemoteProxyConfig(std::string url, std::string problem);
+	static std::auto_ptr<CraftedResponse> errRemoteProxyConfig(
+		std::string url, std::string problem);
+	
+	static std::auto_ptr<CraftedResponse> errSocksProxyError(
+		std::string url, std::string host, int port, std::string problem);
 	
 	static std::auto_ptr<CraftedResponse> errDummyError();
 private:
