@@ -98,7 +98,9 @@ FilteringServer::submitRequest(
 	IntrusivePtr<AbstractResponseHandler> const& handler)
 {
 	IntrusivePtr<AbstractRequestHandler> res;
-	if ((res = processSubstitutionRequest(request_metadata, request_tag, handler))) {
+	if ((res = processConnectRequest(request_metadata, request_tag, handler))) {
+		// do nothing
+	} else if ((res = processSubstitutionRequest(request_metadata, request_tag, handler))) {
 		// do nothing
 	} else if ((res = processUrlPatterns(request_metadata, request_tag, handler))) {
 		// do nothing
@@ -125,6 +127,20 @@ FilteringServer::submitRequest(
 	return res;
 }
 
+IntrusivePtr<AbstractRequestHandler>
+FilteringServer::processConnectRequest(
+	ConstRequestPtr const& request_metadata,
+	RequestTag const& request_tag,
+	IntrusivePtr<AbstractResponseHandler> const& handler)
+{
+	if (request_metadata->requestLine().getMethod() == BString("CONNECT")) {
+		return Server::submitRequest(
+			request_metadata, request_tag, handler
+		);
+	}
+	return IntrusivePtr<AbstractRequestHandler>();
+}
+
 IntrusivePtr<AbstractRequestHandler>
 FilteringServer::processSubstitutionRequest(
 	ConstRequestPtr const& request_metadata,
