@@ -145,8 +145,8 @@ private:
 	
 	static void skipString(DataIter& pos, DataIter const& end);
 	
-	static int const INSPECTION_FRAME_LIMIT = 3;
-	static size_t const INSPECTION_SIZE_LIMIT = 70*1024;
+	static int const INSPECTION_FRAME_LIMIT = 5; // check N first frames
+	static size_t const INSPECTION_SIZE_LIMIT = 80*1024;
 	static double const BUTTON_AREA_THRESHOLD;
 	
 	Status m_status;
