@@ -38,6 +38,7 @@
 #include "ErrorCodes.h"
 #include "IntrusivePtr.h"
 #include "RequestTag.h"
+#include "BString.h"
 #include "URI.h"
 #include <string>
 #include <stddef.h>
@@ -121,6 +122,9 @@ FlashResponseFilter::outputAdSubstitution()
 	
 	getRequestTag()->flags().set(RequestTag::RESPONSE_CRAFTED);
 	response->metadata().setClientRequestId(m_ptrMetadata->getClientRequestId());
+	response->metadata().headers().setHeader(
+		BString("X-BFilter-Content"), BString("generated")
+	);
 	outputMetadata(response->retrieveMetadata());
 	outputBodyData(response->body(), true);
 }
