@@ -150,7 +150,7 @@ HeuristicScore::getUrlStatus(URI const& url)
 		}
 	}
 	
-	if (!url.hasQuery() && num_equals == 0 && num_slashes <= 3 && num_digits <= 6
+	if (!url.hasQuery() && num_equals == 0 && num_slashes <= 3 && num_digits <= 8
 	    && num_digit_groups <= 2 && max_digit_group_size <= 4) {
 		// a URL like: http://somehost.com/1024x768/ would be innocent 
 		return URL_INNOCENT;
@@ -212,6 +212,12 @@ HeuristicScore::getHintModifier(URI const& url)
 	return GlobalState::ReadAccessor()->urlPatterns().getHintFor(url) * 10;
 }
 
+bool
+HeuristicScore::checkFileExtension(BString const& file, BString const& ext)
+{
+	return StringUtils::ciEndsWith(file.begin(), file.end(), ext.begin(), ext.end());
+}
+
 HeuristicScore::UrlRelationship
 HeuristicScore::getDomainRelationship(
 	char const* d1_begin, char const* d1_end,
