@@ -381,6 +381,13 @@ private:
 };
 
 
+class HtmlProcessor::ListenerSuspender : public JsEnvironment::ListenerSuspender
+{
+public:
+	ListenerSuspender(JsEnvironment& js_env);
+};
+
+
 SplittableBuffer const HtmlProcessor::m_sEmptyBuffer;
 
 inline void
@@ -411,6 +418,7 @@ HtmlProcessor::getScriptNestLevel() const
 HtmlProcessor::HtmlProcessor(ServiceContext& service_context)
 :	m_rContext(service_context),
 	m_pScriptContext(0),
+	m_pageCleanupLevel(Config::CLEANUP_OFF),
 	m_baseURL(BString()),
 	m_contentType(CONTENT_HTML),
 	m_pageHint(0),
@@ -426,6 +434,7 @@ HtmlProcessor::HtmlProcessor(ServiceContext& service_context)
 HtmlProcessor::HtmlProcessor(HtmlProcessor const& parent, ScriptContext& script_context)
 :	m_rContext(parent.m_rContext),
 	m_pScriptContext(&script_context),
+	m_pageCleanupLevel(parent.m_pageCleanupLevel),
 	m_ptrRequestMetadata(parent.m_ptrRequestMetadata),
 	m_baseURL(parent.m_baseURL),
 	m_jsEnv(parent.m_jsEnv),
@@ -451,13 +460,20 @@ HtmlProcessor::~HtmlProcessor()
 }
 
 void
-HtmlProcessor::setRelatedInfo(ConstRequestPtr const& request, bool xhtml_content)
-{
+HtmlProcessor::setRelatedInfo(ConstRequestPtr const& request,
+	bool xhtml_content, bool bf_analyze)
+{	
 	m_ptrRequestMetadata = request;
 	m_baseURL = request->requestLine().getURI();
 	m_jsEnv.setPageURL(m_baseURL);
 	m_contentType = (xhtml_content ? CONTENT_XHTML : CONTENT_HTML);
 	m_pageHint = HeuristicScore::getHintModifier(m_baseURL);
+	if (bf_analyze) {
+		m_pageHint = std::max(m_pageHint, 40);
+	}
+	
+	GlobalState::ReadAccessor state;
+	m_pageCleanupLevel = state->config().getPageCleanupLevel();
 }
 
 void
@@ -473,6 +489,7 @@ void
 HtmlProcessor::reset()
 {
 	HtmlLexerCore::reset();
+	m_pageCleanupLevel = Config::CLEANUP_OFF;
 	m_ptrRequestMetadata.reset(0);
 	BString empty_str;
 	URI(empty_str).swap(m_baseURL);
@@ -537,6 +554,14 @@ HtmlProcessor::processAsText(Iterator const& begin, Iterator const& end)
 	}
 }
 
+void
+HtmlProcessor::processAsText(BString const& str)
+{
+	SplittableBuffer buf;
+	buf.append(str);
+	processAsText(buf);
+}
+
 void
 HtmlProcessor::processOpeningTagName(Iterator const& begin, Iterator const& end)
 {
@@ -563,7 +588,7 @@ HtmlProcessor::processOpeningTag(Iterator const& begin, Iterator const& end, boo
 		return;
 	}
 	
-	typedef bool (HtmlProcessor::*FuncPtr)(HtmlNode*, bool);
+	typedef NodeHandled (HtmlProcessor::*FuncPtr)(HtmlNode*, bool);
 	static FuncPtr const func_table[] = {
 		&HtmlProcessor::processOpeningOther,    // TAG_OTHER
 		&HtmlProcessor::processOpeningA,        // TAG_A
@@ -728,13 +753,36 @@ HtmlProcessor::isLexerInTheMiddleOfSomething() const
 	return true;
 }
 
-bool
-HtmlProcessor::processOpeningOther(HtmlNode* node, bool explicit_empty)
+HtmlProcessor::Action
+HtmlProcessor::adviceAction(HeuristicScore const& score) const
 {
-	return false;
+	HeuristicScore::Status const status = score.getStatus();
+	if (status == HeuristicScore::AD) {
+		return SUBSTITUTE;
+	} else if (status == HeuristicScore::PROBABLY_AD) {
+		if (m_pageCleanupLevel == Config::CLEANUP_MAXIMUM) {
+			return SUBSTITUTE;
+		} else {
+			return ANALYZE;
+		}
+	} else if (status == HeuristicScore::PROBABLY_NOT_AD) {
+		if (m_pageCleanupLevel == Config::CLEANUP_MAXIMUM) {
+			return PASS_THROUGH;
+		} else {
+			return ANALYZE;
+		}
+	} else { // status == NOT_AD
+		return PASS_THROUGH;
+	}
 }
 
-bool
+HtmlProcessor::NodeHandled
+HtmlProcessor::processOpeningOther(HtmlNode* node, bool explicit_empty)
+{
+	return NODE_NOT_HANDLED;
+}
+
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningA(HtmlNode* anchor, bool explicit_empty)
 {
 	if (!explicit_empty) {
@@ -755,27 +803,27 @@ HtmlProcessor::processOpeningA(HtmlNode* anchor, bool explicit_empty)
 			}
 		}
 	}
-	return false;
+	return NODE_NOT_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningArea(HtmlNode* area, bool explicit_empty)
 {
 	if (m_ptrContainingMap) {
 		m_ptrContainingMap->appendChild(area);
 	}
-	return false;
+	return NODE_NOT_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningBase(HtmlNode* base, bool explicit_empty)
 {
 	URI url(m_baseURL, URI(base->getAttribute(BString("href"))));
 	url.swap(m_baseURL);
-	return false;
+	return NODE_NOT_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningEmbed(HtmlNode* embed, bool explicit_empty)
 {
 	BString const flash_mimetype("application/x-shockwave-flash");
@@ -800,7 +848,10 @@ HtmlProcessor::processOpeningEmbed(HtmlNode* embed, bool explicit_empty)
 		
 		FlashHeuristicScore score;
 		score.processPageHint(m_pageHint);
-		score.processSize(width, height);
+		if (!memchr(width_str.begin(), '%', width_str.size()) &&
+		    !memchr(height_str.begin(), '%', height_str.size())) {
+			score.processSize(width, height);
+		}
 		score.processUrl(url, url_rel, m_baseURL);
 		score.processOptions(loop, menu);
 		score.processNoscript(m_isInsideNoscript);
@@ -820,36 +871,48 @@ HtmlProcessor::processOpeningEmbed(HtmlNode* embed, bool explicit_empty)
 				))
 			);
 		}
-		if (score.getStatus() == HeuristicScore::AD || embed->hasAdMark()) {
+		
+		Action action = adviceAction(score);
+		if (action == ANALYZE && m_ptrContainingAnchor.get()) {
+			action = SUBSTITUTE;
+		}
+		
+		if (action == SUBSTITUTE || embed->hasAdMark()) {
 			m_pContainingNode->setAdMark(true);
-			BString new_url = FlashAdSuspect::getSubstitutionURL(url, width, height);
-			embed->setAttribute(BString("src"), new_url);
-			processOpeningTagAsText(embed, explicit_empty);
-			return true;
-		} else if (score.getStatus() > HeuristicScore::NOT_AD) {
+			if (m_pageCleanupLevel == Config::CLEANUP_OFF) {
+				BString new_url(FlashAdSuspect::getSubstitutionURL(url, width, height));
+				embed->setAttribute(BString("src"), new_url);
+				processOpeningTagAsText(embed, explicit_empty);
+			} else {
+				BString const replacement(
+					"<!-- [BFilter] EMBED ad removed (page_cleanup is in effect) -->");
+				processAsText(replacement);
+			}
+			return NODE_HANDLED;
+		} else if (action == ANALYZE) {
 			if (InsensitiveEqual()(url.getScheme(), BString("http")) &&
 			    StringUtils::ltrim(src.begin(), src.end()) != src.end()) {
 				BString new_url = AdSuspect::getAnalyzeURL(url, score);
 				embed->setAttribute(BString("src"), new_url);
 				processOpeningTagAsText(embed, explicit_empty);
-				return true;
+				return NODE_HANDLED;
 			}
 		}
 	}
-	return false;
+	return NODE_NOT_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningIframe(HtmlNode* iframe, bool explicit_empty)
 {
 	docFragmentPutContainer(iframe);
 	if (explicit_empty) {
 		processClosingIframe(m_sEmptyBuffer.begin(), m_sEmptyBuffer.end());
 	}
-	return true;
+	return NODE_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningImg(HtmlNode* img, bool explicit_empty)
 {
 	if (explicit_empty) {
@@ -858,7 +921,7 @@ HtmlProcessor::processOpeningImg(HtmlNode* img, bool explicit_empty)
 	return processImg(img, explicit_empty, m_ptrContainingAnchor.get());
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningMap(HtmlNode* map, bool explicit_empty)
 {
 	m_ptrContainingMap.reset(map);
@@ -866,51 +929,51 @@ HtmlProcessor::processOpeningMap(HtmlNode* map, bool explicit_empty)
 	if (explicit_empty) {
 		processClosingMap(m_sEmptyBuffer.begin(), m_sEmptyBuffer.end());
 	}
-	return false;
+	return NODE_NOT_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningNoscript(HtmlNode* noscript, bool explicit_empty)
 {
 	m_isInsideNoscript = true;
 	if (!m_noscriptFollowsScript) {
-		return false;
+		return NODE_NOT_HANDLED;
 	}
 	docFragmentPutContainer(noscript);
 	if (explicit_empty) {
 		processClosingNoscript(m_sEmptyBuffer.begin(), m_sEmptyBuffer.end());
 	}
-	return true;
+	return NODE_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningObject(HtmlNode* object, bool explicit_empty)
 {
 	if (explicit_empty) {
-		return false;
+		return NODE_NOT_HANDLED;
 	}
 	docFragmentPutContainer(object);
-	return true;
+	return NODE_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningScript(HtmlNode* script, bool explicit_empty)
 {
 	docFragmentPutContainer(script);
 	if (explicit_empty) {
 		processClosingScript(m_sEmptyBuffer.begin(), m_sEmptyBuffer.end());
 	}
-	return true;
+	return NODE_HANDLED;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processOpeningParam(HtmlNode* param, bool explicit_empty)
 {
 	if (explicit_empty) {
 		param->setClosed(true);
 	}
 	docFragmentPutElement(param);
-	return true;
+	return NODE_HANDLED;
 }
 
 void
@@ -946,7 +1009,10 @@ HtmlProcessor::processClosingIframe(Iterator const& begin, Iterator const& end)
 	
 	IframeHeuristicScore score;
 	score.processPageHint(m_pageHint);
-	score.processSize(width, height);
+	if (!memchr(width_str.begin(), '%', width_str.size()) &&
+	    !memchr(height_str.begin(), '%', height_str.size())) {
+		score.processSize(width, height);
+	}
 	score.processUrl(url, url_rel, m_baseURL);
 	score.processNoscript(m_isInsideNoscript);
 	if (m_pScriptContext) {
@@ -965,12 +1031,21 @@ HtmlProcessor::processClosingIframe(Iterator const& begin, Iterator const& end)
 			))
 		);
 	}
-	if (score.getStatus() == HeuristicScore::AD || iframe->hasAdMark()) {
+	
+	Action const action = adviceAction(score);
+	
+	if (action == SUBSTITUTE || iframe->hasAdMark()) {
 		iframe->getParent()->setAdMark(true);
-		IframeAdSuspect ad(width, height, url, url_rel.isAbsolute(), score);
-		processDocFragmentNodeReplacement(iframe, *ad.getHtmlSubstitution());
+		if (m_pageCleanupLevel == Config::CLEANUP_OFF) {
+			IframeAdSuspect ad(width, height, url, url_rel.isAbsolute(), score);
+			processDocFragmentNodeReplacement(iframe, *ad.getHtmlSubstitution());
+		} else {
+			BString const replacement(
+				"<!-- [BFilter] IFRAME ad removed (page_cleanup is in effect) -->");
+			processDocFragmentNodeReplacement(iframe, replacement);
+		}
 		return;
-	} else if (score.getStatus() > HeuristicScore::NOT_AD) {
+	} else if (action == ANALYZE) {
 		if (InsensitiveEqual()(url.getScheme(), BString("http")) &&
 		    StringUtils::ltrim(src.begin(), src.end()) != src.end()) {
 			BString new_url = AdSuspect::getAnalyzeURL(url, score);
@@ -993,8 +1068,10 @@ HtmlProcessor::processClosingMap(Iterator const& begin, Iterator const& end)
 				// this means it has been removed from the tree (together with its parent)
 				continue;
 			}
-			bool handled = processImg(img, m_contentType==CONTENT_XHTML, anchor);
-			if (!handled) {
+			NodeHandled handled = processImg(
+				img, m_contentType==CONTENT_XHTML, anchor
+			);
+			if (handled != NODE_HANDLED) {
 				processDocFragmentNode(img);
 			}
 		}
@@ -1075,8 +1152,6 @@ HtmlProcessor::processClosingObject(Iterator const& begin, Iterator const& end)
 	    ieq(object->getAttribute(BString("type")), flash_mimetype)) {
 		BString width_str = object->getAttribute(BString("width"));
 		BString height_str = object->getAttribute(BString("height"));
-		int width = StringUtils::toNumber<int>(width_str, -1);
-		int height = StringUtils::toNumber<int>(height_str, -1);
 		bool loop = false;
 		bool menu = true;
 		HtmlNode* movie_node = 0;
@@ -1097,20 +1172,26 @@ HtmlProcessor::processClosingObject(Iterator const& begin, Iterator const& end)
 		bool is_ad = false;
 		
 		if (object->hasAttribute(data_str)) {
-			is_ad |= processFlashObject(
+			is_ad = processFlashObject(
 				object, data_str,
-				width, height, loop, menu, object->hasAdMark()
+				width_str, height_str, loop, menu
 			);
 		}
-		if (movie_node) {
-			is_ad |= processFlashObject(
+		if (movie_node && !is_ad) {
+			is_ad = processFlashObject(
 				movie_node, value_str,
-				width, height, loop, menu, object->hasAdMark()
+				width_str, height_str, loop, menu
 			);
 		}
 		
 		if (is_ad) {
 			object->getParent()->setAdMark(true);
+			if (m_pageCleanupLevel != Config::CLEANUP_OFF) {
+				BString const replacement(
+					"<!-- [BFilter] OBJECT ad removed (page_cleanup is in effect) -->");
+				processDocFragmentNodeReplacement(object, replacement);
+				return;
+			}
 		}
 	}
 	
@@ -1121,24 +1202,25 @@ HtmlProcessor::processClosingObject(Iterator const& begin, Iterator const& end)
 bool
 HtmlProcessor::processFlashObject(
 	HtmlNode* node, BString const& attr_name,
-	int width, int height, bool loop, bool menu, bool ad_mark)
+	BString const& width_str, BString const& height_str,
+	bool loop, bool menu)
 {
 	BString src = node->getAttribute(attr_name);
+	int width = StringUtils::toNumber<int>(width_str, -1);
+	int height = StringUtils::toNumber<int>(height_str, -1);
 	URI url_rel(src);
 	URI url(m_baseURL, url_rel);
 	
 	FlashHeuristicScore score;
 	score.processPageHint(m_pageHint);
-	score.processSize(width, height);
+	if (!memchr(width_str.begin(), '%', width_str.size()) &&
+	    !memchr(height_str.begin(), '%', height_str.size())) {
+		score.processSize(width, height);
+	}
 	score.processUrl(url, url_rel, m_baseURL);
 	score.processOptions(loop, menu);
 	score.processNoscript(m_isInsideNoscript);
 	if (m_pScriptContext) {
-		if (node->hasAdMark()) {
-			node->getParent()->setAdMark(true);
-			// TODO: do something with the score as well,
-			// because node->getParent() may be a fragment node
-		}
 		score.processScriptLevel(
 			m_pScriptContext->getNestLevel(),
 			m_pScriptContext->isExternalScriptInvolved()
@@ -1154,11 +1236,16 @@ HtmlProcessor::processFlashObject(
 		);
 	}
 	
-	if (score.getStatus() == HeuristicScore::AD || ad_mark) {
-		BString new_url = FlashAdSuspect::getSubstitutionURL(url, width, height);
+	Action action = adviceAction(score);
+	if (action == ANALYZE && m_ptrContainingAnchor.get()) {
+		action = SUBSTITUTE;
+	}
+	
+	if (action == SUBSTITUTE || node->hasAdMark()) {
+		BString new_url(FlashAdSuspect::getSubstitutionURL(url, width, height));
 		node->setAttribute(attr_name, new_url);
 		return true;
-	} else if (score.getStatus() > HeuristicScore::NOT_AD) {
+	} else if (action == ANALYZE) {
 		if (InsensitiveEqual()(url.getScheme(), BString("http")) &&
 		    StringUtils::ltrim(src.begin(), src.end()) != src.end()) {
 			BString new_url = AdSuspect::getAnalyzeURL(url, score);
@@ -1196,14 +1283,7 @@ HtmlProcessor::processScript(HtmlNode* script, bool& nuke_adjacent_noscript)
 	size_t const nest_level = getScriptNestLevel();
 	size_t const max_nest_level = GlobalState::ReadAccessor()->config().getMaxScriptNestLevel();
 	if (nest_level < max_nest_level) {
-		if (isJavaScript(script) && !isXHTML()) {
-			/*
-			There is no point in processing javascripts in XHTML,
-			as document.write() doesn't work there. Of course it's
-			theoretically possible to generate ads with DOM functions, but:
-			1. BFilter doesn't support DOM.
-			2. I haven't ever seen this method used to generate ads.
-			*/
+		if (isJavaScript(script)) {
 			processJavaScript(script, nuke_adjacent_noscript);
 			return;
 		}
@@ -1404,10 +1484,9 @@ HtmlProcessor::processJavaScriptOutput(
 		//DEBUGLOG("Suspect score = " << suspect->getScore().getNumericScore()
 		//	<< ", location: " << suspect->getLocation());
 		
-		HeuristicScore::Status const status = suspect->getScore().getStatus();
+		Action const action = adviceAction(suspect->getScore());
 		
-		if (status == HeuristicScore::PROBABLY_AD ||
-		    status == HeuristicScore::PROBABLY_NOT_AD) {
+		if (action == ANALYZE) {
 			res = tryMarkScriptForAnalyzing(
 				context, *suspect, script, script_code, script_src
 			);
@@ -1416,9 +1495,8 @@ HtmlProcessor::processJavaScriptOutput(
 			}
 		}
 		
-		if (status == HeuristicScore::AD ||
-		    status == HeuristicScore::PROBABLY_AD) {
-			if (status == HeuristicScore::AD) {
+		if (action != PASS_THROUGH) {
+			if (action == SUBSTITUTE) {
 				script->getParent()->setAdMark(true);
 			}
 			res = substituteAdScript(
@@ -1572,7 +1650,7 @@ HtmlProcessor::substituteAdScript(
 	return res;
 }
 
-bool
+HtmlProcessor::NodeHandled
 HtmlProcessor::processImg(HtmlNode* img, bool explicit_empty, HtmlNode const* anchor)
 {
 	BString src = img->getAttribute(BString("src"));
@@ -1614,9 +1692,10 @@ HtmlProcessor::processImg(HtmlNode* img, bool explicit_empty, HtmlNode const* an
 				m_pContainingNode->setAdMark(true);
 				processOpeningTagAsText(img, explicit_empty);
 			}
-			return true;
+			return NODE_HANDLED;
 		}
 	}
+	
 	if (!anchor && !img->getParent()) {
 		BString const usemap_str("usemap");
 		if (img->hasAttribute(usemap_str)) {
@@ -1630,87 +1709,123 @@ HtmlProcessor::processImg(HtmlNode* img, bool explicit_empty, HtmlNode const* an
 			} else {
 				docFragmentPutElement(img);
 				m_imgTagsUsingMap.insert(MultiTagsByName::value_type(mapname, HtmlNodePtr(img)));
-				return true;
+				return NODE_HANDLED;
 			}
 		}
 	}
-	if (!anchor) {
+	
+	BString const onclick_str("onclick");
+	BString const img_onclick = img->getAttribute(onclick_str);
+	BString anchor_onclick;
+	if (anchor) {
+		anchor_onclick = anchor->getAttribute(onclick_str);
+	} else if (img_onclick.empty()) {
 		if (m_pScriptContext && !webbug_size) {
 			m_pScriptContext->incUnclickableImageCount();
 		}
-	} else {
-		BString anchor_link;
-		BString anchor_target;
-		
-		BString onclick = anchor->getAttribute(BString("onclick"));
-		if (!onclick.empty()) {
-			PageOpenListener listener(*m_jsEnv);
-			m_jsEnv->executeScriptAsFunction(onclick, "<onclick>", 1);
-			anchor_link = listener.getURL();
-			anchor_target = listener.getTarget();
+	}
+	
+	BString link;
+	BString target;
+	
+	if (!img_onclick.empty()) {
+		ListenerSuspender listener_suspender(*m_jsEnv);
+		PageOpenListener listener(*m_jsEnv);
+		m_jsEnv->executeScriptAsFunction(img_onclick, "<onclick>", 1);
+		link = listener.getURL();
+		target = listener.getTarget();
+	}
+	
+	if (link.empty() && !anchor_onclick.empty()) {
+		ListenerSuspender listener_suspender(*m_jsEnv);
+		PageOpenListener listener(*m_jsEnv);
+		m_jsEnv->executeScriptAsFunction(anchor_onclick, "<onclick>", 1);
+		link = listener.getURL();
+		target = listener.getTarget();
+	}
+	
+	if (link.empty()) {
+		if (!anchor) {
+			return NODE_NOT_HANDLED;
 		}
-		
-		if (anchor_link.empty()) {
-			anchor_link = anchor->getAttribute(BString("href"));
-			anchor_target = anchor->getAttribute(BString("target"));
-		}
-		
-		URI aurl_rel(anchor_link);
-		URI aurl(m_baseURL, aurl_rel);
-		
-		ImageHeuristicScore score;
-		score.processPageHint(m_pageHint);
+		link = anchor->getAttribute(BString("href"));
+		target = anchor->getAttribute(BString("target"));
+	}
+	
+	URI aurl_rel(link);
+	URI aurl(m_baseURL, aurl_rel);
+	
+	ImageHeuristicScore score;
+	score.processPageHint(m_pageHint);
+	if (!memchr(width_str.begin(), '%', width_str.size()) &&
+		!memchr(height_str.begin(), '%', height_str.size())) {
 		score.processSize(width, height);
-		score.processUrls(m_baseURL, imgurl, imgurl_rel, aurl, aurl_rel);
-		score.processLinkTarget(anchor_target);
-		score.processAssociatedMap(img->hasAttribute(BString("usemap")));
-		score.processNoscript(m_isInsideNoscript);
-		//DEBUGLOG("imgurl = " << imgurl);
-		//DEBUGLOG("score = " << score.getNumericScore());
-		if (m_pScriptContext) {
-			score.processScriptLevel(
-				m_pScriptContext->getNestLevel(),
-				m_pScriptContext->isExternalScriptInvolved()
-			);
-			score.processScriptRedirect(m_pScriptContext->isRedirectInvolved());
-			score.processScriptPersistentCookies(
-				m_pScriptContext->isPersistentCookieInvolved()
-			);
-			
-			m_pScriptContext->adSuspects().add(
-				IntrusivePtr<AdSuspect>(new ImageAdSuspect(
-					width, height, imgurl, imgurl_rel.isAbsolute(),
-					aurl.toBString(), score
-				))
-			);
+	}
+	score.processUrls(m_baseURL, imgurl, imgurl_rel, aurl, aurl_rel);
+	score.processLinkTarget(target);
+	score.processAssociatedMap(img->hasAttribute(BString("usemap")));
+	score.processNoscript(m_isInsideNoscript);
+	//DEBUGLOG("imgurl = " << imgurl);
+	//DEBUGLOG("score = " << score.getNumericScore());
+	if (m_pScriptContext) {
+		score.processScriptLevel(
+			m_pScriptContext->getNestLevel(),
+			m_pScriptContext->isExternalScriptInvolved()
+		);
+		score.processScriptRedirect(m_pScriptContext->isRedirectInvolved());
+		score.processScriptPersistentCookies(
+			m_pScriptContext->isPersistentCookieInvolved()
+		);
+		
+		m_pScriptContext->adSuspects().add(
+			IntrusivePtr<AdSuspect>(new ImageAdSuspect(
+				width, height, imgurl, imgurl_rel.isAbsolute(),
+				aurl.toBString(), score
+			))
+		);
+	}
+	
+	Action const action = adviceAction(score);
+	
+	if (action == SUBSTITUTE) {
+		if (img->getParent()) {
+			img->getParent()->setAdMark(true);
+		} else {
+			m_pContainingNode->setAdMark(true);
 		}
-		if (score.getStatus() == HeuristicScore::AD) {
+		if (m_pageCleanupLevel == Config::CLEANUP_OFF) {
 			BString new_url = ImageAdSuspect::getSubstitutionURL(imgurl, width, height);
 			img->setAttribute(BString("src"), new_url);
 			if (img->getParent()) {
-				img->getParent()->setAdMark(true);
 				processDocFragmentNode(img);
 			} else {
-				m_pContainingNode->setAdMark(true);
 				processOpeningTagAsText(img, explicit_empty);
 			}
-			return true;
-		} else if (score.getStatus() > HeuristicScore::NOT_AD) {
-			if (InsensitiveEqual()(imgurl.getScheme(), BString("http")) &&
-			    StringUtils::ltrim(src.begin(), src.end()) != src.end()) {
-				BString new_url = AdSuspect::getAnalyzeURL(imgurl, score);
-				img->setAttribute(BString("src"), new_url);
-				if (img->getParent()) {
-					processDocFragmentNode(img);
-				} else {
-					processOpeningTagAsText(img, explicit_empty);
-				}
-				return true;
+		} else {
+			BString const replacement(
+				"<!-- [BFilter] IMG ad removed (page_cleanup is in effect) -->");
+			if (img->getParent()) {
+				processDocFragmentNodeReplacement(img, replacement);
+			} else {
+				processAsText(replacement);
 			}
 		}
-	} // if (anchor)
+		return NODE_HANDLED;
+	} else if (action == ANALYZE) {
+		if (InsensitiveEqual()(imgurl.getScheme(), BString("http")) &&
+			StringUtils::ltrim(src.begin(), src.end()) != src.end()) {
+			BString new_url = AdSuspect::getAnalyzeURL(imgurl, score);
+			img->setAttribute(BString("src"), new_url);
+			if (img->getParent()) {
+				processDocFragmentNode(img);
+			} else {
+				processOpeningTagAsText(img, explicit_empty);
+			}
+			return NODE_HANDLED;
+		}
+	}
 	
-	return false;
+	return NODE_NOT_HANDLED;
 }
 
 void
@@ -2280,13 +2395,21 @@ HtmlProcessor::isDynamicResource(HttpResponseMetadata const& metadata)
 auto_ptr<HtmlProcessor::AbstractScriptOperation>
 HtmlProcessor::createScriptSubstitution(HtmlNode* node, AdSuspect const& ad)
 {
-	BString const comment(
-		"<!--\r\n[BFilter] Script removed [replacement follows].\r\n-->");
-	return auto_ptr<AbstractScriptOperation>(
-		new ReplaceBodyScriptOperation(
-			*this, node, *ad.getJsSubstitution(), comment
-		)
-	);
+	if (m_pageCleanupLevel == Config::CLEANUP_OFF) {
+		BString const comment(
+			"<!--\r\n[BFilter] Script removed [replacement follows].\r\n-->");
+		return auto_ptr<AbstractScriptOperation>(
+			new ReplaceBodyScriptOperation(
+				*this, node, *ad.getJsSubstitution(), comment
+			)
+		);
+	} else {
+		BString const comment(
+			"<!--\r\n[BFilter] Script removed (page_cleanup is in effect) -->");
+		return auto_ptr<AbstractScriptOperation>(
+			new RemoveScriptOperation(*this, node, comment)
+		);
+	}
 }
 
 
@@ -2354,6 +2477,14 @@ HtmlProcessor::ScriptContext::processInnerHtmlAssignment(char const* data)
 {
 	discardSubprocessorOutput();
 	
+	size_t const nest_level = getNestLevel();
+	size_t const max_nest_level = GlobalState::ReadAccessor()->config().getMaxScriptNestLevel();
+	if (nest_level >= max_nest_level) {
+		DEBUGLOG("MAX_SCRIPT_NEST_LEVEL (" << max_nest_level
+			<< ") reached while processing innerHtml");
+		return;
+	}
+	
 	if (!m_pSubProcessor) {
 		// should not happen
 		return;
@@ -2560,3 +2691,11 @@ HtmlProcessor::PageOpenListener::processPageOpening(
 		m_target = BString(target, strlen(target));
 	}
 }
+
+
+/*================= HtmlProcessor::ListenerSuspender ==================*/
+
+HtmlProcessor::ListenerSuspender::ListenerSuspender(JsEnvironment& js_env)
+{
+	js_env.setListenerSuspender(*this);
+}
