@@ -34,6 +34,7 @@
 #include "HtmlFragmentNode.h"
 #include "JsEnvProxy.h"
 #include "URI.h"
+#include "Conf.h"
 #include <stddef.h>
 #include <string>
 #include <memory>
@@ -46,6 +47,7 @@ class HttpResponseMetadata;
 class ErrorDescriptor;
 class AdSuspect;
 class AdSuspectList;
+class HeuristicScore;
 
 class HtmlProcessor : private HtmlLexer
 {
@@ -55,7 +57,8 @@ public:
 	
 	~HtmlProcessor();
 public:
-	void setRelatedInfo(ConstRequestPtr const& info, bool xhtml_content);
+	void setRelatedInfo(ConstRequestPtr const& info,
+		bool xhtml_content, bool bf_analyze);
 	
 	void consume(SplittableBuffer& data, bool eof);
 	
@@ -66,6 +69,8 @@ private:
 	enum ContentType { CONTENT_HTML, CONTENT_XHTML };
 	enum { MAX_DESCENDANT_SCRIPT_FETCHES = 20 };
 	enum { MAX_SCRIPT_FETCH_REDIRECTS = 3 };
+	enum NodeHandled { NODE_NOT_HANDLED, NODE_HANDLED };
+	enum Action { PASS_THROUGH, SUBSTITUTE, ANALYZE };
 	
 	enum TagType {
 		TAG_OTHER = 0,
@@ -97,6 +102,7 @@ private:
 	class TempStreamScope;
 	struct ScriptFetchResult;
 	class PageOpenListener;
+	class ListenerSuspender;
 	
 	friend class HtmlProcessor::ScriptContext;
 	friend class HtmlProcessor::LeaveAsIsScriptOperation;
@@ -117,6 +123,8 @@ private:
 	
 	void processAsText(SplittableBuffer const& buf) { processAsText(buf.begin(), buf.end()); }
 	
+	void processAsText(BString const& str);
+	
 	virtual void processOpeningTagName(Iterator const& begin, Iterator const& end);
 	
 	virtual void processOpeningTag(Iterator const& begin, Iterator const& end, bool explicit_empty);
@@ -139,29 +147,31 @@ private:
 	
 	bool isLexerInTheMiddleOfSomething() const;
 	
-	bool processOpeningOther(HtmlNode* node, bool explicit_empty);
+	Action adviceAction(HeuristicScore const& score) const;
 	
-	bool processOpeningA(HtmlNode* anchor, bool explicit_empty);
+	NodeHandled processOpeningOther(HtmlNode* node, bool explicit_empty);
 	
-	bool processOpeningArea(HtmlNode* area, bool explicit_empty);
+	NodeHandled processOpeningA(HtmlNode* anchor, bool explicit_empty);
 	
-	bool processOpeningBase(HtmlNode* base, bool explicit_empty);
+	NodeHandled processOpeningArea(HtmlNode* area, bool explicit_empty);
 	
-	bool processOpeningEmbed(HtmlNode* embed, bool explicit_empty);
+	NodeHandled processOpeningBase(HtmlNode* base, bool explicit_empty);
 	
-	bool processOpeningIframe(HtmlNode* iframe, bool explicit_empty);
+	NodeHandled processOpeningEmbed(HtmlNode* embed, bool explicit_empty);
 	
-	bool processOpeningImg(HtmlNode* img, bool explicit_empty);
+	NodeHandled processOpeningIframe(HtmlNode* iframe, bool explicit_empty);
 	
-	bool processOpeningMap(HtmlNode* map, bool explicit_empty);
+	NodeHandled processOpeningImg(HtmlNode* img, bool explicit_empty);
 	
-	bool processOpeningNoscript(HtmlNode* noscript, bool explicit_empty);
+	NodeHandled processOpeningMap(HtmlNode* map, bool explicit_empty);
 	
-	bool processOpeningObject(HtmlNode* object, bool explicit_empty);
+	NodeHandled processOpeningNoscript(HtmlNode* noscript, bool explicit_empty);
 	
-	bool processOpeningParam(HtmlNode* param, bool explicit_empty);
+	NodeHandled processOpeningObject(HtmlNode* object, bool explicit_empty);
 	
-	bool processOpeningScript(HtmlNode* script, bool explicit_empty);
+	NodeHandled processOpeningParam(HtmlNode* param, bool explicit_empty);
+	
+	NodeHandled processOpeningScript(HtmlNode* script, bool explicit_empty);
 	
 	void processClosingOther(Iterator const& begin, Iterator const& end);
 	
@@ -179,7 +189,8 @@ private:
 	
 	bool processFlashObject(
 		HtmlNode* node, BString const& attr_name,
-		int width, int height, bool loop, bool menu, bool ad_mark);
+		BString const& width_str, BString const& height_str,
+		bool loop, bool menu);
 	
 	void processClosingScript(Iterator const& begin, Iterator const& end);
 	
@@ -204,7 +215,7 @@ private:
 		SplittableBuffer const& script_code, std::string const& script_src,
 		bool& nuke_adjacent_noscript);
 	
-	bool processImg(HtmlNode* img, bool explicit_empty, HtmlNode const* anchor);
+	NodeHandled processImg(HtmlNode* img, bool explicit_empty, HtmlNode const* anchor);
 	
 	void processUnclosedTags(HtmlNode* to);
 	
@@ -284,6 +295,7 @@ private:
 	
 	ServiceContext& m_rContext;
 	ScriptContext* m_pScriptContext;
+	Config::PageCleanupLevel m_pageCleanupLevel;
 	ConstRequestPtr m_ptrRequestMetadata;
 	URI m_baseURL;
 	JsEnvProxy m_jsEnv;
