@@ -36,9 +36,15 @@ HttpRequestLine::HttpRequestLine(BString const& method, URI const& uri, HttpVers
 }
 
 void
-HttpRequestLine::toStream(std::ostream& strm, bool relative_url) const
+HttpRequestLine::toStream(std::ostream& strm, Style style) const
 {
+	URI::Style url_style = URI::S_DEFAULT;
+	if (m_method == BString("CONNECT")) {
+		url_style = URI::S_START_WITH_HOST;
+	} else if (style == RELATIVE_URL) {
+		url_style = URI::S_RELATIVE;
+	}
 	strm << m_method << ' ';
-	m_uri.toStream(strm, relative_url ? URI::S_RELATIVE : URI::S_DEFAULT);
+	m_uri.toStream(strm, url_style);
 	strm << ' ' << m_httpVersion << "\r\n";
 }
