@@ -102,6 +102,12 @@ HttpStateRequestHeaders::headersReceived(bool& body_eof)
 		return m_rParser.activateStateError(edesc);
 	}
 	
+	if (method == BString("CONNECT")) {
+		m_ptrMetadata->setBodyStatus(HttpRequestMetadata::BODY_SIZE_UNKNOWN);
+		m_rParser.eventGotMetadata(m_ptrMetadata, /* is_persistent */ false);
+		return m_rParser.activateStateUnsizedFlatBody();
+	}
+	
 	bool const is_persistent = isPersistentConnection(
 		m_ptrMetadata->headers(),
 		m_ptrMetadata->requestLine().getHttpVersion()
