@@ -50,52 +50,50 @@ using namespace std;
 HttpStreamWriter::HttpStreamWriter(AbstractDataConsumer& consumer)
 :	m_rConsumer(consumer),
 	m_state(WAITING_FOR_METADATA),
-	m_isEOF(false),
-	m_isConnClose(false)
+	m_connection(CONN_KEEP_ALIVE),
+	m_isEOF(false)
 {
 }
 
 void
 HttpStreamWriter::startRequest(HttpRequestMetadata& metadata,
-	bool set_conn_close, bool through_proxy, bool chunked_ok)
+	Connection conn, HttpProxy http_proxy, Chunked chunked)
 {
 	reset();
 	
+	bool const connect_method = (metadata.requestLine().getMethod() == BString("CONNECT"));
+	if (connect_method && http_proxy == NO_HTTP_PROXY) {
+		m_state = FLAT_BODY;
+		m_connection = CONN_CLOSE;
+		return;
+	}
+	
 	// We always send HTTP/1.1 requests, although we keep them
 	// HTTP/1.0 compatible.
 	HttpVersion const& http_ver = HttpVersion::HTTP_1_1;
 	HttpHeadersCollection& headers = metadata.headers();
-	processRequestLine(metadata.requestLine(), http_ver, through_proxy);
+	processRequestLine(metadata.requestLine(), http_ver, http_proxy);
 	prepareRequestHeaders(headers, metadata.requestLine().getURI());
 	
-	
-	if (metadata.requestLine().getHttpVersion() < HttpVersion::HTTP_1_0) {
-		m_state = FLAT_BODY;
-		m_isConnClose = true;
-		return;
-	}
-	
 	switch (metadata.getBodyStatus()) {
 		case HttpMessageMetadata::BODY_FORBIDDEN: {
-			prepareForbiddenBodyMode(headers, http_ver, set_conn_close);
+			prepareForbiddenBodyMode(headers, http_ver, conn);
 			break;
 		}
 		case HttpMessageMetadata::BODY_SIZE_KNOWN: {
 			prepareSizedFlatMode(
 				headers, http_ver,
-				set_conn_close, metadata.getBodySize()
+				conn, metadata.getBodySize()
 			);
 			break;
 		}
 		case HttpMessageMetadata::BODY_SIZE_UNKNOWN: {
-			if (chunked_ok) {
-				prepareChunkedMode(
-					headers, http_ver, set_conn_close
-				);
+			if (connect_method) {
+				prepareUnsizedFlatMode(headers, http_ver);
+			} else if (chunked == CHUNKED_OK) {
+				prepareChunkedMode(headers, http_ver, conn);
 			} else {
-				prepareBufferedFlatMode(
-					headers, http_ver, set_conn_close
-				);
+				prepareBufferedFlatMode(headers, http_ver, conn);
 			}
 			break;
 		}
@@ -108,37 +106,42 @@ HttpStreamWriter::startRequest(HttpRequestMetadata& metadata,
 
 void
 HttpStreamWriter::startResponse(HttpResponseMetadata& metadata,
-	HttpVersion const& max_http_ver, bool set_conn_close)
+	HttpRequestLine const& request_line, Connection conn)
 {
 	reset();
 	
-	if (max_http_ver < HttpVersion::HTTP_1_0) {
+	if (request_line.getHttpVersion() < HttpVersion::HTTP_1_0 ||
+	    metadata.statusLine().getHttpVersion() < HttpVersion::HTTP_1_0) {
 		m_state = FLAT_BODY;
+		m_connection = CONN_CLOSE;
 		return;
 	}
 	
 	HttpHeadersCollection& headers = metadata.headers();
-	HttpVersion http_ver = std::min(max_http_ver, HttpVersion::HTTP_1_1);
+	HttpVersion http_ver(
+		std::min(request_line.getHttpVersion(), HttpVersion::HTTP_1_1)
+	);
 	processStatusLine(metadata.statusLine(), http_ver);
 	prepareResponseHeaders(headers);
 	
 	switch (metadata.getBodyStatus()) {
 		case HttpMessageMetadata::BODY_FORBIDDEN: {
-			prepareForbiddenBodyMode(headers, http_ver, set_conn_close);
+			prepareForbiddenBodyMode(headers, http_ver, conn);
 			break;
 		}
 		case HttpMessageMetadata::BODY_SIZE_KNOWN: {
 			prepareSizedFlatMode(
 				headers, http_ver,
-				set_conn_close, metadata.getBodySize()
+				conn, metadata.getBodySize()
 			);
 			break;
 		}
 		case HttpMessageMetadata::BODY_SIZE_UNKNOWN: {
-			if (http_ver >= HttpVersion::HTTP_1_1) {
-				prepareChunkedMode(headers, http_ver, set_conn_close);
+			if (http_ver < HttpVersion::HTTP_1_1 ||
+			    request_line.getMethod() == BString("CONNECT")) {
+				prepareUnsizedFlatMode(headers, http_ver);
 			} else {
-				prepareUnsizedFlatMode(headers, http_ver, true);
+				prepareChunkedMode(headers, http_ver, conn);
 			}
 			break;
 		}
@@ -202,8 +205,8 @@ HttpStreamWriter::reset()
 	m_outStream.clear();
 	m_bufferedBodyPart.clear();
 	m_state = WAITING_FOR_METADATA;
+	m_connection = CONN_KEEP_ALIVE;
 	m_isEOF = false;
-	m_isConnClose = false;
 }
 
 void
@@ -217,10 +220,15 @@ HttpStreamWriter::output(bool eof)
 void
 HttpStreamWriter::processRequestLine(
 	HttpRequestLine& request_line, HttpVersion const& http_ver,
-	bool through_proxy)
+	HttpProxy http_proxy)
 {
+	HttpRequestLine::Style style = (
+		http_proxy == THROUGH_HTTP_PROXY
+		? HttpRequestLine::DEFAULT
+		: HttpRequestLine::RELATIVE_URL
+	);
 	request_line.setHttpVersion(http_ver);
-	request_line.toStream(m_outStream, !through_proxy);
+	request_line.toStream(m_outStream, style);
 }
 
 void
@@ -272,38 +280,37 @@ HttpStreamWriter::prepareRequestHeaders(HttpHeadersCollection& headers, URI cons
 void
 HttpStreamWriter::prepareForbiddenBodyMode(
 	HttpHeadersCollection& headers, HttpVersion const& http_ver,
-	bool set_conn_close)
+	Connection conn)
 {
 	m_state = FLAT_BODY;
-	m_isConnClose = set_conn_close;
-	handleConnClose(headers, http_ver, set_conn_close);
+	m_connection = conn;
+	handleConnection(headers, http_ver, conn);
 	m_outStream << headers << "\r\n";
 }
 
 void
 HttpStreamWriter::prepareSizedFlatMode(
 	HttpHeadersCollection& headers, HttpVersion const& http_ver,
-	bool set_conn_close, uintmax_t body_size)
+	Connection conn, uintmax_t body_size)
 {
 	m_state = FLAT_BODY;
-	m_isConnClose = set_conn_close;
-	handleConnClose(headers, http_ver, set_conn_close);
-	headers.setHeader(HttpHeader(
+	m_connection = conn;
+	handleConnection(headers, http_ver, conn);
+	headers.setHeader(
 		BString("Content-Length"),
 		BString(StringUtils::fromNumber(body_size))
-	));
+	);
 	headers.removeHeader(BString("Transfer-Encoding"));
 	m_outStream << headers << "\r\n";
 }
 
 void
 HttpStreamWriter::prepareUnsizedFlatMode(
-	HttpHeadersCollection& headers, HttpVersion const& http_ver,
-	bool set_conn_close)
+	HttpHeadersCollection& headers, HttpVersion const& http_ver)
 {
 	m_state = FLAT_BODY;
-	m_isConnClose = true;
-	handleConnClose(headers, http_ver, set_conn_close);
+	m_connection = CONN_CLOSE;
+	handleConnection(headers, http_ver, CONN_CLOSE);
 	headers.removeHeader(BString("Content-Length"));
 	headers.removeHeader(BString("Transfer-Encoding"));
 	m_outStream << headers << "\r\n";
@@ -312,11 +319,11 @@ HttpStreamWriter::prepareUnsizedFlatMode(
 void
 HttpStreamWriter::prepareBufferedFlatMode(
 	HttpHeadersCollection& headers, HttpVersion const& http_ver,
-	bool set_conn_close)
+	Connection conn)
 {
 	m_state = BUFFERED_FLAT_BODY;
-	m_isConnClose = set_conn_close;
-	handleConnClose(headers, http_ver, set_conn_close);
+	m_connection = conn;
+	handleConnection(headers, http_ver, conn);
 	headers.removeHeader(BString("Content-Length"));
 	headers.removeHeader(BString("Transfer-Encoding"));
 	m_outStream << headers; // Content-Length will be appended later
@@ -325,11 +332,11 @@ HttpStreamWriter::prepareBufferedFlatMode(
 void
 HttpStreamWriter::prepareChunkedMode(
 	HttpHeadersCollection& headers, HttpVersion const& http_ver,
-	bool set_conn_close)
+	Connection conn)
 {
 	m_state = CHUNKED_BODY;
-	m_isConnClose = set_conn_close;
-	handleConnClose(headers, http_ver, set_conn_close);
+	m_connection = conn;
+	handleConnection(headers, http_ver, conn);
 	headers.removeHeader(BString("Content-Length"));
 	BString const transfer_encoding("Transfer-Encoding");
 	BString const chunked("chunked");
@@ -338,31 +345,31 @@ HttpStreamWriter::prepareChunkedMode(
 		structure.appendElement(HttpHeaderElement(chunked));
 		structure.commitChanges(*teh);
 	} else {
-		headers.setHeader(HttpHeader(transfer_encoding, chunked));
+		headers.setHeader(transfer_encoding, chunked);
 	}
 	m_outStream <<  headers << "\r\n";
 }
 
 void
-HttpStreamWriter::handleConnClose(
+HttpStreamWriter::handleConnection(
 	HttpHeadersCollection& headers, HttpVersion const& http_ver,
-	bool set_conn_close)
+	Connection conn)
 {
 	BString const connection("Connection");
 	BString const close("close");
 	BString const keep_alive("keep-alive");
-	HttpHeader* ch = headers.getHeaderPtr(BString("Connection"));
+	HttpHeader* ch = headers.getHeaderPtr(connection);
 	if (!ch) {
-		if (set_conn_close) {
-			headers.setHeader(HttpHeader(connection, close));
+		if (conn == CONN_CLOSE) {
+			headers.setHeader(connection, close);
 		} else {
 			if (http_ver < HttpVersion::HTTP_1_1) {
-				headers.setHeader(HttpHeader(connection, keep_alive));
+				headers.setHeader(connection, keep_alive);
 			}
 		}
 	} else {
 		HttpHeaderStructure structure(*ch);
-		if (set_conn_close) {
+		if (conn == CONN_CLOSE) {
 			structure.appendElement(HttpHeaderElement(close));
 		} else {
 			structure.removeElementsByName(close);
