@@ -45,23 +45,27 @@ class HttpStreamWriter
 {
 	DECLARE_NON_COPYABLE(HttpStreamWriter)
 public:
+	enum Connection { CONN_KEEP_ALIVE, CONN_CLOSE };
+	enum HttpProxy { NO_HTTP_PROXY, THROUGH_HTTP_PROXY };
+	enum Chunked { CHUNKED_DONT_USE, CHUNKED_OK };
+	
 	HttpStreamWriter(AbstractDataConsumer& consumer);
 	
 	void startRequest(HttpRequestMetadata& metadata,
-		bool set_conn_close, bool through_proxy, bool chunked_ok);
+		Connection conn, HttpProxy http_proxy, Chunked chunked);
 	
 	void startResponse(HttpResponseMetadata& metadata,
-		HttpVersion const& max_http_ver, bool set_conn_close);
+		HttpRequestLine const& request_line, Connection conn);
 	
 	void appendBodyData(SplittableBuffer& data, bool eof);
 	
 	void reset();
 	
-	bool isConnCloseSet() const { return m_isConnClose; }
+	Connection getConnPersistence() const { return m_connection; }
 	
 	size_t getBufferedBodyDataSize() const { return m_bufferedBodyPart.size(); }
 private:
-	enum State { 
+	enum State {
 		WAITING_FOR_METADATA,
 		FLAT_BODY,
 		BUFFERED_FLAT_BODY,
@@ -72,7 +76,7 @@ private:
 	void output(bool eof=false);
 	
 	void processRequestLine(HttpRequestLine& request_line,
-		HttpVersion const& http_ver, bool through_proxy);
+		HttpVersion const& http_ver, HttpProxy http_proxy);
 	
 	void processStatusLine(HttpStatusLine& status_line, HttpVersion const& http_ver);
 	
@@ -85,29 +89,29 @@ private:
 	}
 	
 	void prepareForbiddenBodyMode(HttpHeadersCollection& headers,
-		HttpVersion const& http_ver, bool set_conn_close);
+		HttpVersion const& http_ver, Connection conn);
 	
 	void prepareSizedFlatMode(HttpHeadersCollection& headers,
-		HttpVersion const& http_ver, bool set_conn_close, uintmax_t body_size);
+		HttpVersion const& http_ver, Connection conn, uintmax_t body_size);
 	
 	void prepareUnsizedFlatMode(HttpHeadersCollection& headers,
-		HttpVersion const& http_ver, bool set_conn_close);
+		HttpVersion const& http_ver);
 	
 	void prepareBufferedFlatMode(HttpHeadersCollection& headers,
-		HttpVersion const& http_ver, bool set_conn_close);
+		HttpVersion const& http_ver, Connection conn);
 	
 	void prepareChunkedMode(HttpHeadersCollection& headers,
-		HttpVersion const& http_ver, bool set_conn_close);
+		HttpVersion const& http_ver, Connection conn);
 	
-	void handleConnClose(HttpHeadersCollection& headers,
-		HttpVersion const& http_ver, bool set_conn_close);
+	void handleConnection(HttpHeadersCollection& headers,
+		HttpVersion const& http_ver, Connection conn);
 	
 	AbstractDataConsumer& m_rConsumer;
 	SBOutStream m_outStream;
 	SplittableBuffer m_bufferedBodyPart;
 	State m_state;
+	Connection m_connection;
 	bool m_isEOF;
-	bool m_isConnClose;
 };
 
 #endif
