@@ -38,7 +38,8 @@ using namespace std;
 
 ImageHeuristicScore::ImageHeuristicScore(int score)
 :	m_score(score),
-	m_sizeScore(0)
+	m_sizeScore(0),
+	m_flags(0)
 {
 	PRINT_SCORE("-----------------------------");
 }
@@ -46,6 +47,11 @@ ImageHeuristicScore::ImageHeuristicScore(int score)
 int
 ImageHeuristicScore::getNumericScore() const
 {
+	if (m_flags == (IMG_EXT_JPG|IMG_PAGE_UNRELATED|IMG_LINK_SAME_HOST)) {
+		if (m_score >= 100 && m_score <= 160 && m_sizeScore < 80) {
+			return 99;
+		}
+	}
 	return m_score;
 }
 
@@ -73,7 +79,20 @@ ImageHeuristicScore::processUrls(URI const& base,
 	URI const& link_abs, URI const& link_rel)
 {
 	int score = 0;
+	
+	if (checkFileExtension(img_abs.getRawPath(), BString(".jpg")) ||
+	    checkFileExtension(img_abs.getRawPath(), BString(".jpeg"))) {
+		m_flags |= IMG_EXT_JPG;
+	} else {
+		m_flags &= ~IMG_EXT_JPG;
+	}
+	
 	UrlRelationship link_img_rship = getUrlRelationship(link_abs, img_abs);
+	if (link_img_rship == URLS_SAME_HOST) {
+		m_flags |= IMG_LINK_SAME_HOST;
+	} else {
+		m_flags &= ~IMG_LINK_SAME_HOST;
+	}
 	
 	PRINT_SCORE("LINK URL: " << link_abs);
 	PRINT_SCORE("LINK MODIFIER: " << getHintModifier(link_abs));
@@ -111,9 +130,13 @@ ImageHeuristicScore::processUrls(URI const& base,
 			PRINT_SCORE("+15 (link UNRELATED to img) && (link NOT SAME HOST as page)")
 			score += 15;
 		}
-	} else if (img_rship == URLS_SAME_HOST && img_status != URL_SUSPICIOUS) {
-		PRINT_SCORE("-15 (link SAME HOST as page) && (img NOT SUSPICIOUS)");
-		score -= 15;
+		m_flags |= IMG_PAGE_UNRELATED;
+	} else {
+		m_flags &= ~IMG_PAGE_UNRELATED;
+		if (img_rship == URLS_SAME_HOST && img_status != URL_SUSPICIOUS) {
+			PRINT_SCORE("-15 (link SAME HOST as page) && (img NOT SUSPICIOUS)");
+			score -= 15;
+		}
 	}
 	if (img_status == URL_INNOCENT) {
 		PRINT_SCORE("-10 img INNOCENT");
@@ -159,20 +182,18 @@ ImageHeuristicScore::processAssociatedMap(bool val)
 	}
 }
 
-void
-ImageHeuristicScore::processAnimation(bool val)
-{
-	if (val) {
-		m_score += 70;
-	} else {
-		m_score -= 30;
-	}
-}
-
+// This method may or may not be called.
 void
 ImageHeuristicScore::processScriptLevel(int level, bool external)
 {
 	PRINT_SCORE("SCRIPT LEVEL: " << level << ", external: " << external);
+	
+	if (level == 0) {
+		m_flags &= ~FROM_SCRIPT;
+	} else {
+		m_flags |= FROM_SCRIPT;
+	}
+	
 	if (level > 1) {
 		m_score += 5;
 		if (external) {
@@ -184,7 +205,6 @@ ImageHeuristicScore::processScriptLevel(int level, bool external)
 			m_score += 10;
 		}
 	}
-	
 }
 
 void
