@@ -54,8 +54,6 @@ public:
 	// does the image have the usemap attribute?
 	void processAssociatedMap(bool val);
 	
-	void processAnimation(bool val);
-	
 	// level 0: no script, 1: from script, 2: from script inside script, ...
 	void processScriptLevel(int level, bool external);
 	
@@ -65,8 +63,16 @@ public:
 	// were persistent cookies sent when fetching an external script?
 	void processScriptPersistentCookies(bool val);
 private:
+	enum Flags {
+		IMG_EXT_JPG        = 1 << 0,
+		FROM_SCRIPT        = 1 << 1,
+		IMG_PAGE_UNRELATED = 1 << 2,
+		IMG_LINK_SAME_HOST = 1 << 3
+	};
+	
 	int m_score;
 	int m_sizeScore;
+	int m_flags;
 };
 
 #endif
