@@ -25,6 +25,7 @@
 #endif
 
 #include "SharedPtr.h"
+#include "URI.h"
 
 class JsEnvironment;
 
@@ -45,6 +46,8 @@ public:
 	
 	JsEnvironment& operator*() { return *access(); }
 	
+	void setPageURL(URI const& url);
+	
 	void reset();
 private:
 	void operator=(JsEnvProxy const&); // forbidden
@@ -52,6 +55,7 @@ private:
 	JsEnvironment* access();
 	
 	SharedPtr<JsEnvironment, ACE_NULL_SYNCH> m_ptrEnv;
+	URI m_pageURL;
 };
 
 #endif
