@@ -30,9 +30,9 @@
 class ProxyDescriptor
 {
 public:
-	explicit ProxyDescriptor(SymbolicInetAddr const& addr);
+	enum ProxyType { INVALID, HTTP, SOCKS4, SOCKS4A, SOCKS5 };
 	
-	explicit ProxyDescriptor(std::string const& config_problem);
+	ProxyDescriptor();
 	
 	ProxyDescriptor(ProxyDescriptor const& other);
 	
@@ -42,14 +42,31 @@ public:
 	
 	void swap(ProxyDescriptor& other);
 	
-	SymbolicInetAddr const& getProxyAddr() const { return m_addr; }
-	
-	bool hasProblem() const { return !m_configProblem.empty(); }
+	void setInvalid(std::string const& config_problem);
 	
 	std::string const& getProblem() const { return m_configProblem; }
+	
+	ProxyType getType() const { return m_type; }
+	
+	void setType(ProxyType type) { m_type = type; }
+	
+	SymbolicInetAddr const& getAddr() const { return m_addr; }
+	
+	void setAddr(SymbolicInetAddr const& addr) { m_addr = addr; }
+	
+	std::string const& getUserName() const { return m_username; }
+	
+	void setUserName(std::string const& username) { m_username = username; }
+	
+	std::string const& getPassword() const { return m_password; }
+	
+	void setPassword(std::string const& password) { m_password = password; }
 private:
-	SymbolicInetAddr m_addr;
+	ProxyType m_type;
 	std::string m_configProblem;
+	SymbolicInetAddr m_addr;
+	std::string m_username;
+	std::string m_password;
 };
 
 inline void swap(ProxyDescriptor& o1, ProxyDescriptor& o2)
