@@ -24,7 +24,7 @@
 #include <config.h>
 #endif
 
-#include <memory>
+#include "IntrusivePtr.h"
 
 class ErrorDescriptor;
 
@@ -33,7 +33,7 @@ class ResponseErrorInitiator
 public:
 	virtual ~ResponseErrorInitiator() {}
 	
-	virtual void raiseError(std::auto_ptr<ErrorDescriptor> edesc) = 0;
+	virtual void raiseError(IntrusivePtr<ErrorDescriptor> const& edesc) = 0;
 	
 	virtual bool isErrorRaised() const = 0;
 };
