@@ -60,6 +60,7 @@
 #include "NetworkActivityReporter.h"
 #include "RequestLog.h"
 #include "RequestTag.h"
+#include "SocksError.h"
 #include "Debug.h"
 #include <algorithm>
 #include <list>
@@ -120,9 +121,17 @@ public:
 	
 	HttpRequestMetadata const& getOutgoingMetadata() const { return *m_ptrOutgoingMetadata; }
 	
+	bool isConnectRequest() const { return m_ptrOrigMetadata->requestLine().getMethod() == BString("CONNECT"); }
+	
 	RequestTag const& getTag() const { return m_tag; }
 	
-	SymbolicInetAddr const& getNextHopAddr() const { return m_nextHopAddr; }
+	bool isGoingThroughHttpProxy() const { return m_nextHopProxy.getType() == ProxyDescriptor::HTTP; }
+	
+	ProxyDescriptor const* getNextHopProxy() const;
+	
+	SymbolicInetAddr const& getNextHopAddr() const;
+	
+	SymbolicInetAddr const& getHttpAddr() const { return m_httpAddr; }
 	
 	void updateRoute();
 protected:
@@ -146,13 +155,14 @@ private:
 	std::auto_ptr<HttpRequestMetadata> m_ptrOutgoingMetadata;
 	RequestTag m_tag;
 	std::auto_ptr<ResponseFilterChain> m_ptrFilterChain;
-	SymbolicInetAddr m_nextHopAddr;
-	bool m_willGoThroughProxy;
+	ProxyDescriptor m_nextHopProxy;
+	SymbolicInetAddr m_httpAddr; // http proxy or server address
 	IntrusivePtr<ErrorDescriptor> m_ptrErrDesc;
 	std::string m_errorMsg;
 	SplittableBuffer m_data;
 	bool m_isEOF;
 	bool m_isConsumingStarted;
+	bool m_isConstructionDone;
 	HttpStreamWriter m_streamWriter;
 };
 
@@ -226,16 +236,15 @@ Server::Server(ServiceContext& context, ServerTimeouts const& timeouts)
 :	m_rContext(context),
 	m_timeouts(timeouts),
 	m_pipelineSize(0),
-	m_connectingTo(string(), -1),
 	m_lastResponseVersion(0, 0),
 	m_readTimer(
 		sigc::mem_fun(*this, &Server::onReadTimeout),
-		TimeDelta::fromSec(m_timeouts.getReadTimeout()),
+		m_timeouts.getReadTimeout(),
 		&context.reactor()
 	),
 	m_writeTimer(
 		sigc::mem_fun(*this, &Server::onWriteTimeout),
-		TimeDelta::fromSec(m_timeouts.getWriteTimeout()),
+		m_timeouts.getWriteTimeout(),
 		&context.reactor()
 	),
 	m_delayedActionTimer(
@@ -248,14 +257,13 @@ Server::Server(ServiceContext& context, ServerTimeouts const& timeouts)
 	m_isWaitingForWrite(0),
 	m_ptrSelf(this)
 {
-	m_rContext.reactor().beforeSleepSignal().connect(sigc::mem_fun(*this, &Server::updateState));
-	m_connector.setListener(this);
+	m_rContext.reactor().beforeSleepSignal().connect(
+		sigc::mem_fun(*this, &Server::updateState)
+	);
 }
 
 Server::~Server()
 {
-	m_connector.removeListener();
-	dropConnection();
 }
 
 IntrusivePtr<AbstractRequestHandler>
@@ -269,7 +277,7 @@ Server::submitRequest(
 		m_rContext, m_ptrSelf, request_metadata,
 		request_tag, handler, filter_try_list
 	));
-	inspectRequest(*request);
+	validateRequest(*request);
 	enqueueRequest(request);
 	updatePipeline();
 	return request;
@@ -286,7 +294,7 @@ Server::submitRequest(
 		m_rContext, m_ptrSelf, request_metadata,
 		request_tag, handler, filter_try_list
 	));
-	inspectRequest(*request);
+	validateRequest(*request);
 	enqueueRequest(request);
 	updatePipeline();
 	return request;
@@ -328,6 +336,7 @@ void
 Server::reset()
 {
 	storeConnection();
+	abortInProgressConnection();
 	assert(!m_ptrConnection.get());
 	assert(!m_handlerId);
 	assert(!m_readTimer.isActive());
@@ -337,12 +346,10 @@ Server::reset()
 	std::for_each(m_requestQueue.begin(), m_requestQueue.end(), rc);
 	std::for_each(m_phantomRequestQueue.begin(), m_phantomRequestQueue.end(), rc);
 	
-	
 	m_requestQueue.clear();
 	m_phantomRequestQueue.clear();
 	m_pipelineSize = 0;
-	m_connectingTo.set(string(), -1);
-	m_connector.abort();
+	
 	m_inputData.clear();
 	m_lastResponseVersion.set(0, 0);
 	m_delayedActionTimer.deactivate();
@@ -418,7 +425,12 @@ Server::updatePipeline()
 				//DEBUGLOG("Pipelining is only useful for servers that support HTTP/1.1");
 				break;
 			}
-			if (!m_ptrConnection->isSameAddress(next_req.getNextHopAddr())) {
+			if (!prev_req.isFullyConsumed()) {
+				// This will also catch the case of a CONNECT request.
+				//DEBUGLOG("The previous request is not complete");
+				break;
+			}
+			if (!m_ptrConnection->isSameAddress(next_req.getHttpAddr())) {
 				//DEBUGLOG("Request to a different address");
 				break;
 			}
@@ -430,10 +442,6 @@ Server::updatePipeline()
 				//DEBUGLOG("MAX_PIPELINE_SIZE reached");
 				break;
 			}
-			if (!prev_req.isFullyConsumed()) {
-				//DEBUGLOG("The previous request is not complete");
-				break;
-			}
 			if (!prev_req.isRestartable() || !next_req.isRestartable()) {
 				// only requests without a body may be pipelined
 				//DEBUGLOG("Either this or the previous request is not restartable");
@@ -452,53 +460,102 @@ Server::updatePipeline()
 	} while (false);
 }
 
-void
-Server::inspectRequest(Request& req)
+bool
+Server::validateRequest(Request& req)
+{
+	if (!validateRequestProtocol(req)) {
+		return false;
+	}
+	if (!validateRequestExpectations(req)) {
+		return false;
+	}
+	if (!validateTunnelPort(req)) {
+		return false;
+	}
+	return true;
+}
+
+bool
+Server::validateRequestProtocol(Request& req)
+{
+	if (req.isConnectRequest()) {
+		return true;
+	}
+	
+	BString const http("http");
+	URI const& uri = req.getRequestURI();
+	BString const& scheme = uri.getScheme();
+	if (!StringUtils::ciEqual(scheme.begin(), scheme.end(), http.begin(), http.end())) {
+		req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
+			new ErrorDescriptor(
+				ErrorCodes::UNSUPPORTED_PROTOCOL,
+				"unsupported protocol",
+				sigc::bind(
+					sigc::ptr_fun(&ErrorResponse::errUnsupportedProtocol),
+					uri.toString(), uri.getScheme().toStdString()
+				)
+			)
+		));
+		return false;
+	}
+	
+	return true;
+}
+
+bool
+Server::validateRequestExpectations(Request& req)
 {
 	URI const& uri = req.getRequestURI();
-	
-	do { // check if the protocol is supported
-		static char const http[] = {'h','t','t','p'};
-		BString const& scheme = uri.getScheme();
-		if (!StringUtils::ciEqual(scheme.begin(), scheme.end(), http, http + sizeof(http))) {
-			req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-				new ErrorDescriptor(
-					ErrorCodes::UNSUPPORTED_PROTOCOL,
-					"unsupported protocol",
-					sigc::bind(
-						sigc::ptr_fun(&ErrorResponse::errUnsupportedProtocol),
-						uri.toString(), uri.getScheme().toStdString()
-					)
-				)
-			));
-			return;
-		}
-	} while (false);
-	
-	do { // check for the unsupported expectation
-		HttpHeadersCollection const& headers = req.getRequestHeaders();
-		HttpHeader const* expect_hdr = headers.getHeaderPtr(BString("Expect"));
-		if (expect_hdr) {
-			HttpHeaderStructure structure(*expect_hdr);
-			list<HttpHeaderElement>::iterator it = structure.elements().begin();
-			list<HttpHeaderElement>::iterator const end = structure.elements().end();
-			for (; it != end; ++it) {
-				if (InsensitiveEqual()(it->getName(), BString("100-continue"))) {
-					req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-						new ErrorDescriptor(
-							ErrorCodes::EXPECTATION_FAILED,
-							"expectation failed",
-							sigc::bind(
-								sigc::ptr_fun(&ErrorResponse::errExpectationFailed),
-								uri.toString(), it->getName().toStdString()
-							)
+	HttpHeadersCollection const& headers = req.getRequestHeaders();
+	HttpHeader const* expect_hdr = headers.getHeaderPtr(BString("Expect"));
+	if (expect_hdr) {
+		HttpHeaderStructure structure(*expect_hdr);
+		list<HttpHeaderElement>::iterator it = structure.elements().begin();
+		list<HttpHeaderElement>::iterator const end = structure.elements().end();
+		for (; it != end; ++it) {
+			if (InsensitiveEqual()(it->getName(), BString("100-continue"))) {
+				req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
+					new ErrorDescriptor(
+						ErrorCodes::EXPECTATION_FAILED,
+						"expectation failed",
+						sigc::bind(
+							sigc::ptr_fun(&ErrorResponse::errExpectationFailed),
+							uri.toString(), it->getName().toStdString()
 						)
-					));
-					return;
-				}
+					)
+				));
+				return false;
 			}
 		}
-	} while (false);
+	}
+	
+	return true;
+}
+
+bool
+Server::validateTunnelPort(Request& req)
+{
+	if (!req.isConnectRequest()) {
+		return true;
+	}
+	
+	URI const& uri = req.getRequestURI();
+	unsigned const port = uri.guessPort();
+	if (!GlobalState::ReadAccessor()->config().isTunnelPortAllowed(port)) {
+		req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
+			new ErrorDescriptor(
+				ErrorCodes::TUNNEL_PORT_NOT_ALLOWED,
+				"tunnel port not allowed",
+				sigc::bind(
+					sigc::ptr_fun(&ErrorResponse::errUrlForbidden),
+					uri.toString()
+				)
+			)
+		));
+		return false;
+	}
+	
+	return true;
 }
 
 bool
@@ -510,7 +567,7 @@ Server::isNewConnectionRequiredFor(Request const& req) const
 	if (!m_ptrConnection->isExpectedToPersist()) {
 		return true;
 	}
-	if (!m_ptrConnection->isSameAddress(req.getNextHopAddr())) {
+	if (!m_ptrConnection->isSameAddress(req.getHttpAddr())) {
 		return true;
 	}
 	if (!req.isRestartable()) {
@@ -526,18 +583,24 @@ Server::requestConnectionFor(Request const& req)
 	if (req.isRestartable()) {
 		// we always use a fresh connection for unrestartable requests
 		// (requests with bodies)
-		m_ptrConnection = ServerConnectionPool::instance()
-			->retrieveConnection(req.getNextHopAddr());
-		if (m_ptrConnection.get()) {
+		auto_ptr<ServerConnection> conn(
+			ServerConnectionPool::instance()->retrieveConnection(
+				req.getHttpAddr()
+			)
+		);
+		if (conn.get()) {
 			//DEBUGLOG("Reusing an existing server connection");
-			processNewConnection();
+			processNewHttpConnection(conn);
 			return;
 		}
 	}
-	//DEBUGLOG("Initiating a new server connection");
-	TimeDelta timeout = TimeDelta::fromSec(m_timeouts.getConnectTimeout());
-	m_connectingTo = req.getNextHopAddr();
-	m_connector.initiate(m_rContext.reactor(), m_connectingTo, &timeout);
+	
+	//DEBUGLOG("Initiating a new server (or proxy) connection");
+	abortInProgressConnection(); // just in case
+	m_connector.initiate(
+		*this, m_rContext.reactor(), req.getNextHopAddr(),
+		&m_timeouts.getConnectTimeout()
+	);
 }
 
 bool
@@ -645,11 +708,7 @@ Server::writeRequestData(Request& req)
 		if (req.isFullyConsumed()) {
 			//DEBUGLOG("Request " << req.getRequestURI() << " has been fully consumed");
 #ifdef DEBUG
-			AbstractDebugAgent::HttpMessageType msg_type =
-				(&m_rContext.scriptServer() == this)
-				? AbstractDebugAgent::SCRIPT_REQUEST
-				: AbstractDebugAgent::OUTGOING_REQUEST;
-			DBG_HTTP_MESSAGE_END(msg_type);
+			dbgReportOutgoingRequestSent();
 #endif
 			updatePipeline();
 		}
@@ -673,29 +732,39 @@ Server::processInputData(bool eof)
 		if (m_inputData.empty() && !eof) {
 			break;
 		}
+		
+		if (m_pipelineSize == 0) {
+			assert(m_requestQueue.empty());
+			DEBUGLOG("Unexpected data (or EOF) received from server");
+			// we don't need to call restartPendingRequests() because m_pipelineSize == 0
+			dropConnection();
+			m_inputData.clear();
+			updatePipeline();
+			break;
+		}
+		
+		assert(!m_requestQueue.empty());
+		IntrusivePtr<Request> req(m_requestQueue.front());
+		
 		if (isInactiveState()) {
-			if (m_pipelineSize != 0) {
-				switchToResponseStart(m_requestQueue.front()->getOrigMetadata());
+			if (req->isConnectRequest() && !req->isGoingThroughHttpProxy()) {
+				switchToUnsizedFlatBody();
 			} else {
-				assert(m_requestQueue.empty());
-				DEBUGLOG("Unexpected data (or EOF) received from server");
-				// we don't need to call restartPendingRequests() because m_pipelineSize == 0
-				dropConnection();
-				m_inputData.clear();
-				updatePipeline();
-				break;
+				switchToResponseStart(req->getOrigMetadata());
 			}
 		}
-		// at this point the parser is guaranteed to be in the ResponseStart state
-		assert(!m_requestQueue.empty());
-		IntrusivePtr<Request> req = m_requestQueue.front();
+		
 		SplittableBuffer body_data;
 		bool body_eof = false;
 		bool const empty_input = m_inputData.empty();
-		bool message_finishes = processNewData(m_inputData, eof, body_data, body_eof);
+		Status const status = processNewData(
+			m_inputData, eof, body_data, body_eof
+		);
+		
 		req->getTag()->downloadProgress().received() += body_data.size();
+		
 		if (!req->errorDescriptor().get()) {
-			if (message_finishes) {
+			if (status == MESSAGE_ENDS) {
 				RequestLog::logResponseEnd(req->getTag());
 #ifdef DEBUG
 				AbstractDebugAgent::HttpMessageType msg_type =
@@ -708,19 +777,24 @@ Server::processInputData(bool eof)
 			
 			if (!body_data.empty() || body_eof) {
 				ScopedSuspender suspender(*this);
+				
+				// This may result in a recursive invocation
+				// of reactor event loop.
 				req->responseHandler().processBodyData(body_data, body_eof);
 			}
 		}
+		
 		if (m_pipelineSize == 0 || m_requestQueue.front() != req) {
-			// removed from the pipeline by the recursive invocation of reactor event loop
+			// Removed from the pipeline by recursive invocation
+			// of reactor event loop
 		} else if (req->errorDescriptor().get()) {
 			// request has been cancelled
 			processArtificialResponses();
 			// This will drop the server connection if neccessary.
 			// Dropping the connection also causes switchToInactive() to be called.
-		} else if (message_finishes) {
+		} else if (status == MESSAGE_ENDS) {
 			DEBUGLOG("Response finishes: " << req->getRequestURI());
-			assert(isInactiveState()); // message_finishes implies that
+			assert(isInactiveState()); // MESSAGE_ENDS implies that
 			if (m_ptrConnection.get()) {
 				m_ptrConnection->registerResponse(m_lastResponseVersion);
 			}
@@ -732,7 +806,7 @@ Server::processInputData(bool eof)
 			}
 			processArtificialResponses();
 			updatePipeline();
-		} else if (isErrorState()) {
+		} else if (status == ERROR_STATE) {
 			// keep in mind: an eof while in ResponseStart state generates an error
 			/*
 			DEBUGLOG("ERROR parsing response: "
@@ -799,13 +873,15 @@ Server::processInputData(bool eof)
 			updatePipeline();
 			break;
 		}
-		if (!message_finishes) {
-			// This means more input data is needed.
-			// Example: the input ends in the middle of a header.
+		
+		if (status == NEED_MORE_DATA) {
+			// Can happen for example if input ends in the middle of a header.
 			break;
-		} else if (eof && m_inputData.empty()) {
-			// We have to break here, or the eof will act
-			// on the next response (if any).
+		}
+		
+		if (eof && m_inputData.empty()) {
+			// We have to break here, or the eof will be interpreted
+			// as a response to the next queued request (if any).
 			break;
 		}
 	}
@@ -814,6 +890,18 @@ Server::processInputData(bool eof)
 	}
 }
 
+void
+Server::dbgReportOutgoingRequestSent()
+{
+#ifdef DEBUG
+	AbstractDebugAgent::HttpMessageType msg_type =
+		(&m_rContext.scriptServer() == this)
+		? AbstractDebugAgent::SCRIPT_REQUEST
+		: AbstractDebugAgent::OUTGOING_REQUEST;
+	DBG_HTTP_MESSAGE_END(msg_type);
+#endif
+}
+
 void
 Server::shiftRequestQueue()
 {
@@ -888,13 +976,29 @@ Server::retrieveConnection()
 	return conn;
 }
 
+bool
+Server::isConnectionInProgress() const
+{
+	return m_connector.isInProgress() || m_socksConnector.isInProgress();
+}
+
+void
+Server::abortInProgressConnection()
+{
+	m_connector.abort();
+	m_socksConnector.abort();
+}
+
 void
 Server::restartPendingRequests()
 {
-	for (size_t i = 0; i < m_pipelineSize; ++i) {
-		m_requestQueue[i]->restart();
+	if (m_pipelineSize > 0) {
+		for (size_t i = 0; i < m_pipelineSize; ++i) {
+			m_requestQueue[i]->restart();
+		}
+		m_pipelineSize = 0;
+		abortInProgressConnection();
 	}
-	m_pipelineSize = 0;
 }
 
 void
@@ -976,10 +1080,10 @@ Server::updateState()
 		if (isArtificialResponseRequired()) {
 			m_delayedActionTimer.ensureActive();
 		} else if (m_pipelineSize > 0 && !m_ptrConnection.get() &&
-		           !m_connector.isInProgress()) {
+		           !isConnectionInProgress()) {
 			assert(m_pipelineSize == 1);
 			Request& req = *m_requestQueue.front();
-			if (req.hasData()) {
+			if (req.hasData() || req.isConnectRequest()) {
 				m_delayedActionTimer.ensureActive();
 			}
 		}
@@ -1047,6 +1151,9 @@ Server::processLeadingPhantomResponse()
 			
 			req->getTag()->flags().set(RequestTag::RESPONSE_CRAFTED);
 			response->metadata().setClientRequestId(req->getClientRequestId());
+			response->metadata().headers().setHeader(
+				BString("X-BFilter-Content"), BString("generated")
+			);
 #ifdef DEBUG
 			response->metadata().headers().setHeader(
 				BString("X-Request-URI"),
@@ -1184,61 +1291,52 @@ Server::eventGotMetadata(auto_ptr<HttpResponseMetadata> metadata, bool is_persis
 }
 
 void
-Server::connectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm)
+Server::onConnectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm)
 {
 	//DEBUGLOG("Connection established");
-	m_ptrConnection.reset(new ServerConnection(m_connectingTo, strm));
-	m_connectingTo.set(string(), -1);
-	strm.set_handle(ACE_INVALID_HANDLE); // prevent double closing
+	assert(m_pipelineSize > 0);
+	assert(m_requestQueue.size() >= m_pipelineSize);
 	
-	processNewConnection();
-}
-
-void
-Server::processNewConnection()
-{
-	assert(m_ptrConnection.get());
+	Request& req = *m_requestQueue.front();
+	ProxyDescriptor const* proxy = req.getNextHopProxy();
 	
-	m_ptrConnection->setTcpNoDelay();
-	m_ptrConnection->peer().enable(ACE_NONBLOCK);
-	ACE_HANDLE handle = m_ptrConnection->peer().get_handle();
-	m_flags &= ~WRITE_BUFFER_FULL;
-	m_eventMask = Reactor::READ|Reactor::WRITE;
-	// here we don't check if the current request has data available,
-	// as we do it before initiating a connection
-	try {
-		m_handlerId = m_rContext.reactor().registerHandler(
-			handle, IntrusivePtr<EventHandlerBase>(this),
-			m_eventMask
-		);
-	} catch (Reactor::Exception& e) {
-		string problem("Could not register a socket in the reactor: "+string(e.what()));
-		DEBUGLOG2(problem);
-		m_ptrConnection.reset(0);
-		Request const& req = *m_requestQueue.front();
-		answerWithError(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
-				ErrorCodes::UNKNOWN_ERROR,
-				problem,
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errGenericError),
-					req.getRequestURI().toString(), problem,
-					int(HttpStatusLine::SC_INTERNAL_SERVER_ERROR)
-				)
-			)
-		));
+	if (!proxy || proxy->getType() == ProxyDescriptor::HTTP) {
+		processNewHttpConnection(strm);
 		return;
 	}
+	
+	SocksAsyncConnector::SocksType socks_type;
+	switch (proxy->getType()) {
+		case ProxyDescriptor::SOCKS4:
+		socks_type = SocksAsyncConnector::SOCKS4;
+		break;
+		case ProxyDescriptor::SOCKS4A:
+		socks_type = SocksAsyncConnector::SOCKS4A;
+		break;
+		case ProxyDescriptor::SOCKS5:
+		socks_type = SocksAsyncConnector::SOCKS5;
+		break;
+		default:
+		assert(0 && "unknown proxy type");
+		return;
+	}
+	
+	m_socksConnector.initiate(
+		socks_type, *this, m_rContext.reactor(),
+		strm, req.getHttpAddr(),
+		proxy->getUserName(), proxy->getPassword(),
+		&m_timeouts.getSocksConnectTimeout()
+	);
 }
 
 void
-Server::connectionFailed(FailureCode fcode, int errnum)
+Server::onConnectionFailed(FailureCode fcode, int errnum)
 {
 	DEBUGLOG("Connection failed");
 	assert(m_pipelineSize > 0);
 	assert(m_requestQueue.size() >= m_pipelineSize);
 	
-	m_ptrConnection.reset(0);
+	m_ptrConnection.reset(0); // Do we need this?
 	Request const& req = *m_requestQueue.front();
 	URI const& uri = req.getRequestURI();
 	
@@ -1312,6 +1410,40 @@ Server::connectionFailed(FailureCode fcode, int errnum)
 	updatePipeline();
 }
 
+void
+Server::onConnThroughSocksDone(AutoClosingSAP<ACE_SOCK_Stream>& strm)
+{
+	//DEBUGLOG("Connection through socks proxy established");
+	processNewHttpConnection(strm);
+}
+
+void
+Server::onConnThroughSocksFailed(SocksError const& err)
+{
+	DEBUGLOG("Connection through socks proxy failed");
+	assert(m_pipelineSize > 0);
+	assert(m_requestQueue.size() >= m_pipelineSize);
+	
+	m_ptrConnection.reset(0); // Do we need this?
+	Request const& req = *m_requestQueue.front();
+	URI const& uri = req.getRequestURI();
+	std::string problem(err.toString());
+	
+	answerWithError(IntrusivePtr<ErrorDescriptor>(
+		new ErrorDescriptor(
+			ErrorCodes::SOCKS_PROXY_ERROR,
+			problem,
+			sigc::bind(
+				sigc::ptr_fun(&ErrorResponse::errSocksProxyError),
+				uri.toString(), req.getNextHopAddr().getHost(),
+				req.getNextHopAddr().getPort(), problem
+			)
+		)
+	));
+	
+	updatePipeline();
+}
+
 void
 Server::handleRead(ACE_HANDLE)
 {
@@ -1320,7 +1452,12 @@ Server::handleRead(ACE_HANDLE)
 		// all of the triggered events are processed
 		return;
 	}
-	processArtificialResponses();
+	
+	processArtificialResponses(); // it may drop the connection
+	if (!m_ptrConnection.get()) {
+		return;
+	}
+	
 	auto_ptr<DataChunk> chunk = DataChunk::create(READ_BUF_SIZE);
 	ssize_t received = m_ptrConnection->peer().recv(
 		chunk->getDataAddr(), chunk->getDataSize()
@@ -1357,6 +1494,83 @@ Server::handleWrite(ACE_HANDLE)
 	processOutputData();
 }
 
+void
+Server::processNewHttpConnection(AutoClosingSAP<ACE_SOCK_Stream>& strm)
+{
+	assert(m_pipelineSize > 0);
+	assert(m_requestQueue.size() >= m_pipelineSize);
+	
+	Request const& req = *m_requestQueue.front();
+	auto_ptr<ServerConnection> conn(
+		new ServerConnection(req.getHttpAddr(), strm)
+	);
+	processNewHttpConnection(conn);
+}
+
+void
+Server::processNewHttpConnection(std::auto_ptr<ServerConnection> conn)
+{
+	assert(conn.get());
+	assert(m_pipelineSize > 0);
+	assert(m_requestQueue.size() >= m_pipelineSize);
+	
+	Request const& req = *m_requestQueue.front();
+	m_ptrConnection = conn;
+	m_ptrConnection->setTcpNoDelay();
+	m_ptrConnection->peer().enable(ACE_NONBLOCK);
+	ACE_HANDLE handle = m_ptrConnection->peer().get_handle();
+	m_flags &= ~WRITE_BUFFER_FULL;
+	m_eventMask = Reactor::READ;
+	if (req.hasData()) {
+		m_eventMask |= Reactor::WRITE;
+	}
+	
+	try {
+		m_handlerId = m_rContext.reactor().registerHandler(
+			handle, IntrusivePtr<EventHandlerBase>(this),
+			m_eventMask
+		);
+	} catch (Reactor::Exception& e) {
+		string problem("Could not register a socket in the reactor: "+string(e.what()));
+		DEBUGLOG2(problem);
+		m_ptrConnection.reset(0);
+		answerWithError(IntrusivePtr<ErrorDescriptor>(
+			new ErrorDescriptor(
+				ErrorCodes::UNKNOWN_ERROR,
+				problem,
+				sigc::bind(
+					sigc::ptr_fun(&ErrorResponse::errGenericError),
+					req.getRequestURI().toString(), problem,
+					int(HttpStatusLine::SC_INTERNAL_SERVER_ERROR)
+				)
+			)
+		));
+		return;
+	}
+	
+	if (req.isConnectRequest() && !req.isGoingThroughHttpProxy()) {
+		auto_ptr<HttpResponseMetadata> metadata(
+			createConnectionEstablishedResponse(req)
+		);
+		eventGotMetadata(metadata, /* is_persistent = */ false);
+	}
+}
+
+std::auto_ptr<HttpResponseMetadata>
+Server::createConnectionEstablishedResponse(Request const& req)
+{
+	auto_ptr<HttpResponseMetadata> metadata(
+		new HttpResponseMetadata(
+			HttpStatusLine(
+				req.getOrigMetadata()->requestLine().getHttpVersion(),
+				200, BString("Connection established")
+			)
+		)
+	);
+	metadata->setBodyStatus(HttpResponseMetadata::BODY_SIZE_UNKNOWN);
+	return metadata;
+}
+
 void
 Server::onDelayedAction()
 {
@@ -1369,7 +1583,7 @@ Server::onDelayedAction()
 		return;
 	}
 	
-	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !m_connector.isInProgress()) {
+	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !isConnectionInProgress()) {
 		assert(m_pipelineSize == 1);
 		m_requestQueue.front()->updateRoute();
 		// This may trigger a proxy configuration error, which
@@ -1382,10 +1596,10 @@ Server::onDelayedAction()
 	
 	assert(!isArtificialResponseRequired());
 	
-	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !m_connector.isInProgress()) {
+	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !isConnectionInProgress()) {
 		assert(m_pipelineSize == 1);
 		Request& req = *m_requestQueue.front();
-		if (req.hasData()) {
+		if (req.hasData() || req.isConnectRequest()) {
 			requestConnectionFor(req);
 		}
 	}
@@ -1409,10 +1623,10 @@ Server::Request::Request(
 		context, m_ptrOrigMetadata, request_tag,
 		filter_try_list, response_handler, *this
 	)),
-	m_nextHopAddr("", -1),
-	m_willGoThroughProxy(false),
+	m_httpAddr(string(), -1),
 	m_isEOF(false),
 	m_isConsumingStarted(false),
+	m_isConstructionDone(false),
 	m_streamWriter(*this)
 {
 	{
@@ -1420,13 +1634,46 @@ Server::Request::Request(
 		maybeReportClientIP(global_state->config());
 		updateRoute(global_state->config());
 	}
-	m_streamWriter.startRequest(*m_ptrOutgoingMetadata, false, m_willGoThroughProxy, false);
+	
+	HttpStreamWriter::HttpProxy const http_proxy = (
+		isGoingThroughHttpProxy()
+		? HttpStreamWriter::THROUGH_HTTP_PROXY
+		: HttpStreamWriter::NO_HTTP_PROXY
+	);
+	m_streamWriter.startRequest(
+		*m_ptrOutgoingMetadata, HttpStreamWriter::CONN_KEEP_ALIVE,
+		 http_proxy, HttpStreamWriter::CHUNKED_DONT_USE
+	);
+	
+	m_isConstructionDone = true;
 }
 
 Server::Request::~Request()
 {
 }
 
+ProxyDescriptor const*
+Server::Request::getNextHopProxy() const
+{
+	if (m_nextHopProxy.getType() == ProxyDescriptor::INVALID) {
+		return 0;
+	} else {
+		return &m_nextHopProxy;
+	}
+}
+
+SymbolicInetAddr const&
+Server::Request::getNextHopAddr() const
+{
+	switch (m_nextHopProxy.getType()) {
+		case ProxyDescriptor::INVALID:
+		case ProxyDescriptor::HTTP:
+		return m_httpAddr;
+		default:
+		return m_nextHopProxy.getAddr();
+	}
+}
+
 void
 Server::Request::updateRoute()
 {
@@ -1438,31 +1685,33 @@ Server::Request::updateRoute(Config const& conf)
 {
 	URI const& uri = getRequestURI();
 	std::string host = uri.getHost().toStdString();
+	auto_ptr<ProxyDescriptor> proxy(conf.getProxyFor(host));
 	
-	auto_ptr<ProxyDescriptor> proxy;
-	
-	if (!conf.getNextHopProxy().hasProblem()) {
-		proxy = conf.getProxyFor(host);
+	if (proxy.get() && proxy->getType() == ProxyDescriptor::HTTP) {
+		m_httpAddr = proxy->getAddr();
 	} else {
-		std::string problem = conf.getNextHopProxy().getProblem();
-		setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
-				ErrorCodes::REMOTE_PROXY_CONFIG_PROBLEM,
-				problem,
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errRemoteProxyConfig),
-					getRequestURI().toString(), problem
-				)
-			)
-		));
+		m_httpAddr.set(host, uri.guessPort());
 	}
 	
 	if (proxy.get()) {
-		m_willGoThroughProxy = true;
-		m_nextHopAddr = proxy->getProxyAddr();
+		m_nextHopProxy = *proxy;
 	} else {
-		m_willGoThroughProxy = false;
-		m_nextHopAddr.set(host, uri.guessPort());
+		ProxyDescriptor().swap(m_nextHopProxy);
+		return;
+	}
+	
+	if (proxy->getType() == ProxyDescriptor::INVALID) {
+		setErrorResponse(IntrusivePtr<ErrorDescriptor>(
+			new ErrorDescriptor(
+				ErrorCodes::REMOTE_PROXY_CONFIG_PROBLEM,
+				proxy->getProblem(),
+				sigc::bind(
+					sigc::ptr_fun(&ErrorResponse::errRemoteProxyConfig),
+					getRequestURI().toString(),
+					proxy->getProblem()
+				)
+			)
+		));
 	}
 }
 
@@ -1510,10 +1759,20 @@ Server::Request::cancelRequest(IntrusivePtr<ErrorDescriptor> const& edesc)
 void
 Server::Request::processNewData(SplittableBuffer& data, bool eof)
 {
+	bool const was_eof = m_isEOF;
 	m_data.appendDestructive(data);
 	m_isEOF |= eof;
 	
+	if (!m_isConstructionDone) {
+		return;
+	}
+	
 	if (Server* server = m_ptrServer.get()) {
+#ifdef DEBUG
+		if (!was_eof && m_isEOF && m_data.empty()) {
+			server->dbgReportOutgoingRequestSent();
+		}
+#endif
 		server->onNewOutput();
 	} else {
 		// the server is being (or have already been) destroyed
@@ -1536,7 +1795,17 @@ Server::Request::restart()
 	m_data.clear();
 	m_isConsumingStarted = false;
 	updateRoute(); // user settings might have changed
-	m_streamWriter.startRequest(*m_ptrOutgoingMetadata, false, m_willGoThroughProxy, false);
+	
+	HttpStreamWriter::HttpProxy const http_proxy = (
+		isGoingThroughHttpProxy()
+		? HttpStreamWriter::THROUGH_HTTP_PROXY
+		: HttpStreamWriter::NO_HTTP_PROXY
+	);
+	m_streamWriter.startRequest(
+		*m_ptrOutgoingMetadata, HttpStreamWriter::CONN_KEEP_ALIVE,
+		 http_proxy, HttpStreamWriter::CHUNKED_DONT_USE
+	);
+	
 	SplittableBuffer body_data;
 	m_streamWriter.appendBodyData(body_data, true);
 	return true;
