@@ -27,6 +27,8 @@
 #include "HttpResponseParser.h"
 #include "AsyncConnector.h"
 #include "AsyncConnectorListener.h"
+#include "SocksAsyncConnector.h"
+#include "SocksConnectorListener.h"
 #include "SplittableBuffer.h"
 #include "CraftedResponseGenerator.h"
 #include "ServerTimeouts.h"
@@ -62,6 +64,7 @@ class ACE_SOCK_Stream;
 class Server :
 	private HttpResponseParser,
 	private AsyncConnectorListener,
+	private SocksConnectorListener,
 	private EventHandler<NullRefCounter>,
 	public sigc::trackable
 {
@@ -128,7 +131,13 @@ private:
 	
 	void updatePipeline();
 	
-	static void inspectRequest(Request& req);
+	static bool validateRequest(Request& req);
+	
+	static bool validateRequestProtocol(Request& req);
+	
+	static bool validateRequestExpectations(Request& req);
+	
+	static bool validateTunnelPort(Request& req);
 	
 	bool isNewConnectionRequiredFor(Request const& req) const;
 	
@@ -146,6 +155,8 @@ private:
 	
 	void processInputData(bool eof);
 	
+	void dbgReportOutgoingRequestSent();
+	
 	void shiftRequestQueue();
 	
 	bool connectionMayBeStored() const;
@@ -156,6 +167,10 @@ private:
 	
 	std::auto_ptr<ServerConnection> retrieveConnection();
 	
+	bool isConnectionInProgress() const;
+	
+	void abortInProgressConnection();
+	
 	void restartPendingRequests();
 	
 	void answerWithError(IntrusivePtr<ErrorDescriptor> const& error_response);
@@ -190,16 +205,25 @@ private:
 	virtual void eventGotMetadata(
 		std::auto_ptr<HttpResponseMetadata> metadata, bool is_persistent);
 	
-	virtual void connectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm);
+	virtual void onConnectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm);
 	
-	void processNewConnection();
+	virtual void onConnectionFailed(FailureCode fcode, int errnum);
 	
-	virtual void connectionFailed(FailureCode fcode, int errnum);
+	virtual void onConnThroughSocksDone(AutoClosingSAP<ACE_SOCK_Stream>& strm);
+	
+	virtual void onConnThroughSocksFailed(SocksError const& err);
 	
 	virtual void handleRead(ACE_HANDLE);
 	
 	virtual void handleWrite(ACE_HANDLE);
 	
+	void processNewHttpConnection(AutoClosingSAP<ACE_SOCK_Stream>& strm);
+	
+	void processNewHttpConnection(std::auto_ptr<ServerConnection> conn);
+	
+	std::auto_ptr<HttpResponseMetadata>
+		createConnectionEstablishedResponse(Request const& req);
+	
 	void onDelayedAction();
 	
 	ServiceContext& m_rContext;
@@ -207,8 +231,8 @@ private:
 	RequestQueue m_requestQueue;
 	PhantomRequestQueue m_phantomRequestQueue;
 	size_t m_pipelineSize;
-	SymbolicInetAddr m_connectingTo;
 	AsyncConnector m_connector;
+	SocksAsyncConnector m_socksConnector;
 	std::auto_ptr<ServerConnection> m_ptrConnection;
 	SplittableBuffer m_inputData;
 	HttpVersion m_lastResponseVersion;
