@@ -28,7 +28,8 @@
 #include <ace/os_include/netinet/os_tcp.h> // FOR TCP_NODELAY
 
 ServerConnection::ServerConnection(
-	SymbolicInetAddr const& addr, ACE_SOCK_Stream const& peer)
+	SymbolicInetAddr const& addr,
+	AutoClosingSAP<ACE_SOCK_Stream>& peer)
 :	m_symbolicAddr(addr),
 	m_peer(peer),
 	m_willPersist(true),
@@ -40,7 +41,6 @@ ServerConnection::ServerConnection(
 
 ServerConnection::~ServerConnection()
 {
-	m_peer.close();
 }
 
 void
