@@ -26,13 +26,15 @@
 
 #include "SymbolicInetAddr.h"
 #include "HttpVersion.h"
+#include "AutoClosingSAP.h"
 #include <ace/config-lite.h>
 #include <ace/SOCK_Stream.h>
 
 class ServerConnection
 {
 public:
-	ServerConnection(SymbolicInetAddr const& addr, ACE_SOCK_Stream const& peer);
+	ServerConnection(SymbolicInetAddr const& addr,
+		AutoClosingSAP<ACE_SOCK_Stream>& peer);
 	
 	~ServerConnection();
 	
@@ -57,7 +59,7 @@ public:
 	void setTcpNoDelay(bool value = true);
 private:
 	SymbolicInetAddr m_symbolicAddr;
-	ACE_SOCK_Stream m_peer;
+	AutoClosingSAP<ACE_SOCK_Stream> m_peer;
 	bool m_willPersist;
 	int m_numResponses;
 	HttpVersion m_minResponseVersion;
