@@ -29,9 +29,10 @@
 #include "RefCountable.h"
 #include "RefCounter.h"
 #include "SymbolicInetAddr.h"
-#include "Timer.h"
 #include "TimeStamp.h"
 #include "TimeDelta.h"
+#include "Reactor.h"
+#include "EventHandler.h"
 #include <ace/config-lite.h>
 #include <ace/Synch.h>
 #include <ace/Singleton.h>
@@ -42,7 +43,6 @@
 #include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
 #include <boost/multi_index/composite_key.hpp>
-#include <sigc++/sigc++.h>
 #include <iterator>
 #include <utility>
 #include <cassert>
@@ -77,7 +77,9 @@ struct ServerConnectionPool::Entry
 };
 
 
-class ServerConnectionPool::Impl : public ServerConnectionPool
+class ServerConnectionPool::Impl :
+	public ServerConnectionPool,
+	private EventHandlerBase
 {
 public:
 	Impl();
@@ -96,7 +98,7 @@ private:
 	class AddrTag {};
 	class SequenceTag {};
 	
-	typedef ACE_Thread_Mutex Mutex;
+	typedef ACE_Recursive_Thread_Mutex Mutex;
 	typedef multi_index_container<
 		Entry,
 		indexed_by<
@@ -116,9 +118,15 @@ private:
 	typedef Container::index<AddrTag>::type AddrIdx;
 	typedef Container::index<SequenceTag>::type SequenceIdx;
 	
+	virtual void handleTimeout(ReactorTimerId const&);
+	
+	virtual void ref();
+	
+	virtual void unref();
+	
 	void removeTimedOut();
 	
-	void onCleanupTimeout();
+	void unregisterCleanupTimer();
 	
 	void refreshCleanupTimer();
 	
@@ -126,7 +134,10 @@ private:
 	
 	Mutex m_mutex;
 	Container m_container;
-	Timer m_cleanupTimer;
+	Reactor* m_pReactor;
+	ReactorTimerId m_cleanupTimerId;
+	int m_refCount;
+	bool m_isCleanupScheduled;
 };
 
 
@@ -141,10 +152,9 @@ ServerConnectionPool::instance()
 /*======================== ServerConnectionPool::Impl =====================*/
 
 ServerConnectionPool::Impl::Impl()
-:	m_cleanupTimer(
-		sigc::mem_fun(*this, &Impl::onCleanupTimeout),
-		TimeDelta::fromSec(CLEANUP_TIMEOUT)
-	)
+:	m_pReactor(0),
+	m_refCount(0),
+	m_isCleanupScheduled(false)
 {
 }
 
@@ -216,7 +226,13 @@ ServerConnectionPool::Impl::connectToReactor(Reactor& reactor)
 {
 	ACE_GUARD_RETURN(Mutex, guard, m_mutex, );
 	
-	m_cleanupTimer.connectToReactor(reactor);
+	unregisterCleanupTimer();
+	
+	TimeDelta timeout(TimeDelta::fromSec(CLEANUP_TIMEOUT));
+	m_pReactor = &reactor;
+	m_cleanupTimerId = m_pReactor->registerTimer(
+		Reactor::EventHandlerPtr(this)
+	);
 	refreshCleanupTimer();
 }
 
@@ -224,7 +240,39 @@ void
 ServerConnectionPool::Impl::disconnectFromReactor()
 {
 	ACE_GUARD_RETURN(Mutex, guard, m_mutex, );
-	m_cleanupTimer.disconnectFromReactor();
+	
+	unregisterCleanupTimer();
+}
+
+void
+ServerConnectionPool::Impl::handleTimeout(ReactorTimerId const&)
+{
+	ACE_GUARD_RETURN(Mutex, guard, m_mutex, );
+	
+	m_isCleanupScheduled = false;
+	removeTimedOut();
+	refreshCleanupTimer();
+}
+
+void
+ServerConnectionPool::Impl::ref()
+{
+	// we are protected by the caller's guard
+	
+	++m_refCount;
+}
+
+// This one may be called both from inside and outside of this class.
+void
+ServerConnectionPool::Impl::unref()
+{
+	ACE_GUARD_RETURN(Mutex, guard, m_mutex, );
+	
+	if (!--m_refCount) {
+		m_cleanupTimerId = ReactorTimerId();
+		m_pReactor = 0;
+		m_isCleanupScheduled = false;
+	}
 }
 
 void
@@ -255,21 +303,42 @@ ServerConnectionPool::Impl::removeTimedOut()
 }
 
 void
-ServerConnectionPool::Impl::onCleanupTimeout()
+ServerConnectionPool::Impl::unregisterCleanupTimer()
 {
-	ACE_GUARD_RETURN(Mutex, guard, m_mutex, );
+	// we are protected by the caller's guard
 	
-	removeTimedOut();
-	refreshCleanupTimer();
+	if (m_pReactor) {
+		assert(m_cleanupTimerId);
+		m_pReactor->unregisterTimer(m_cleanupTimerId);
+		// unregisterTimer() will cause unref() to be called
+	}
+	assert(m_refCount == 0);
+	assert(!m_pReactor);
+	assert(!m_isCleanupScheduled);
 }
 
 void
 ServerConnectionPool::Impl::refreshCleanupTimer()
 {
+	// we are protected by the caller's guard
+	
+	if (!m_pReactor) {
+		return;
+	}
+	
+	assert(m_cleanupTimerId);
+	
 	if (m_container.empty()) {
-		m_cleanupTimer.deactivate();
+		if (m_isCleanupScheduled) {
+			m_pReactor->rescheduleTimer(m_cleanupTimerId);
+			m_isCleanupScheduled = false;
+		}
 	} else {
-		m_cleanupTimer.ensureActive();
+		if (!m_isCleanupScheduled) {
+			TimeDelta timeout(TimeDelta::fromSec(CLEANUP_TIMEOUT));
+			m_pReactor->rescheduleTimer(m_cleanupTimerId, &timeout);
+			m_isCleanupScheduled = true;
+		}
 	}
 }
 
@@ -277,7 +346,7 @@ bool
 ServerConnectionPool::Impl::isConnAlive(ServerConnection& conn)
 {
 	// I don't see a problem with changing handle mode to nonblocking.
-	// After all, AsyncConnection also returns non-blocking sockets.
+	// After all, AsyncConnector also returns non-blocking sockets.
 	if (conn.peer().enable(ACE_NONBLOCK) == -1) {
 		return false;
 	}
