@@ -25,11 +25,12 @@
 
 #include "ServiceContext.h"
 #include "ServerTimeouts.h"
+#include "TimeDelta.h"
 
 ServiceContext::ServiceContext(Reactor& reactor)
 :	m_rReactor(reactor),
-	m_server(*this, ServerTimeouts(500, 100, 34)),
-	m_scriptServer(*this, ServerTimeouts(14, 14, 18)),
+	m_server(*this, getServerTimeouts()),
+	m_scriptServer(*this, getScriptServerTimeouts()),
 	m_htmlProcessor(*this)
 {
 }
@@ -45,3 +46,25 @@ ServiceContext::reset()
 	//m_scriptServer.reset(); // HttpFetcher resets it
 	//m_htmlProcessor.reset(); // HtmlResponseFilter resets it
 }
+
+ServerTimeouts
+ServiceContext::getServerTimeouts()
+{
+	ServerTimeouts st;
+	st.setReadTimeout(TimeDelta::fromSec(500));
+	st.setWriteTimeout(TimeDelta::fromSec(100));
+	st.setConnectTimeout(TimeDelta::fromSec(34));
+	st.setSocksConnectTimeout(TimeDelta::fromSec(37));
+	return st;
+}
+
+ServerTimeouts
+ServiceContext::getScriptServerTimeouts()
+{
+	ServerTimeouts st;
+	st.setReadTimeout(TimeDelta::fromSec(14));
+	st.setWriteTimeout(TimeDelta::fromSec(14));
+	st.setConnectTimeout(TimeDelta::fromSec(18));
+	st.setSocksConnectTimeout(TimeDelta::fromSec(21));
+	return st;
+}
