@@ -29,6 +29,7 @@
 #include "InetAddr.h"
 
 class Reactor;
+class ServerTimeouts;
 
 class ServiceContext
 {
@@ -49,6 +50,10 @@ public:
 	
 	InetAddr& clientAddr() { return m_clientAddr; }
 private:
+	static ServerTimeouts getServerTimeouts();
+	
+	static ServerTimeouts getScriptServerTimeouts();
+	
 	Reactor& m_rReactor;
 	FilteringServer m_server;
 	Server m_scriptServer;
