@@ -391,7 +391,7 @@ URI::toBString(Style style) const
 void
 URI::parse(BString const& str)
 {
-	char const* begin = StringUtils::ltrim(str.begin(), str.begin());
+	char const* begin = StringUtils::ltrim(str.begin(), str.end());
 	char const* end = StringUtils::rtrim(begin, str.end());
 	begin = parseLeadingScheme(str, begin, end);
 	begin = parseLeadingAuthority(str, begin, end);
