@@ -56,6 +56,13 @@ private:
 
 WorkerThreadPool::WorkerThreadPool()
 :	m_cond(m_mutex),
+#if defined(ACE_WIN32) && !ACE_HAS_WINNT4
+	// CriticalSection-based mutexes don't support trylock() on win9x
+	m_leaderMutex(USYNC_PROCESS),
+#else
+	m_leaderMutex(USYNC_THREAD),
+#endif
+	m_connAcceptor(ConnAcceptor::DONT_STOP_ON_SIGNALS),
 	m_numWaiters(0),
 	m_isClosing(false)
 {
@@ -64,43 +71,74 @@ WorkerThreadPool::WorkerThreadPool()
 WorkerThreadPool::~WorkerThreadPool()
 {
 	{
-		ACE_GUARD(Mutex, guard, m_mutex);
+		ACE_Guard<Mutex> guard(m_mutex);
 		
 		m_isClosing = true;
-		m_cond.broadcast();
+		
+		m_connAcceptor.abort();
 		
 		ReactorList::iterator it = m_runningReactors.begin();
 		ReactorList::iterator const end = m_runningReactors.end();
 		for (; it != end; ++it) {
 			it->reactor()->stop();
 		}
+		
+		m_cond.broadcast();
 	}
 	m_threadManager.wait();
 }
 
 void
-WorkerThreadPool::handleConnection(
-	SocketPtr const& socket, InetAddr const& client_addr)
+WorkerThreadPool::start()
 {
-	ACE_GUARD(Mutex, guard, m_mutex);
+	prepareNewThread();
+}
+
+bool
+WorkerThreadPool::addAcceptor(ConnAcceptor::AcceptorPtr const& acceptor)
+{
+	return m_connAcceptor.add(acceptor);
+}
+
+void
+WorkerThreadPool::removeAcceptor(ConnAcceptor::AcceptorPtr const& acceptor)
+{
+	m_connAcceptor.remove(acceptor);
+}
+
+void
+WorkerThreadPool::removeAllAcceptors()
+{
+	m_connAcceptor.removeAll();
+}
+
+WorkerThreadPool::NewThreadStatus
+WorkerThreadPool::prepareNewThread()
+{
+	ACE_Guard<Mutex> guard(m_mutex);
 	
-	m_requestQueue.push_back(Connection(socket, client_addr));
-	
-	if (m_numWaiters == 0) {
-		m_lastThreadCreateTime = MonotonicTimer::getTimestamp();
-		if (m_threadManager.spawn(&WorkerThreadPool::runThread,
-		    this, THR_SCOPE_SYSTEM|THR_DETACHED) == -1) {
-			DEBUGLOG("could not spawn a new thread");
-			m_requestQueue.pop_back();
-		}
-	} else {
-		m_cond.signal();
+	if (m_isClosing) {
+		return CLOSING;
 	}
+	
+	int res = 0;
+	if (m_numWaiters > 0) {
+		res = m_cond.signal();
+	} else {
+		m_lastThreadCreateTime = MonotonicTimer::getTimestamp();
+		res = m_threadManager.spawn(
+			&WorkerThreadPool::runThread,
+			this, THR_SCOPE_SYSTEM|THR_DETACHED
+		);
+	}
+	return res == 0 ? READY : FAILURE;
 }
 
 int
 WorkerThreadPool::wait()
 {
+	// m_mutex is locked by our caller
+	
 	ScopedIncDec<int> waiters_manager(m_numWaiters);
 	
 	ACE_Time_Value tv(WAIT_FOR_NEW_TASK);
@@ -116,53 +154,79 @@ WorkerThreadPool::runService()
 		reactor = ReactorFactory::createBestReactor(SynchFactory<ACE_NULL_SYNCH>());
 	} catch (Reactor::Exception& e) {
 		DEBUGLOG2("Could not initialize the reactor: " << e.what());
-		if (!m_requestQueue.empty()) {
-			m_requestQueue.pop_front();
-		}
 		return;
 	}
 	ServiceContext service_context(*reactor);
 	
-	ACE_GUARD(Mutex, guard, m_mutex);
+	ACE_Guard<Mutex> guard(m_mutex);
+	
+	ReactorRegistration reactor_registration(reactor.get(), m_runningReactors);
 	
 	while (!m_isClosing) {
-		while (m_requestQueue.empty()) {
-			if (m_numWaiters >= WAITERS_KEEP_ALIVE && MonotonicTimer::getTimestamp()
-			    > m_lastThreadCreateTime + TimeDelta::fromSec(WAIT_FOR_NEW_TASK)) {
-				return;
-			}
-			if (wait() == -1) {
-				if (m_numWaiters >= WAITERS_KEEP_ALIVE) {
-					return;
-				}
-			}
-			if (m_isClosing) {
-				return;
-			}
+		if (tryBecomingLeader(service_context)) {
+			continue;
 		}
 		
-		SocketPtr socket = m_requestQueue.front().first;
-		service_context.clientAddr() = m_requestQueue.front().second;
-		m_requestQueue.pop_front();
+		if (m_numWaiters >= WAITERS_KEEP_ALIVE && MonotonicTimer::getTimestamp()
+		    > m_lastThreadCreateTime + TimeDelta::fromSec(WAIT_FOR_NEW_TASK)) {
+			break;
+		}
 		
-		ReactorRegistration reactor_registration(reactor.get(), m_runningReactors);
-		
-		{
-			typedef ACE_Reverse_Lock<Mutex> ReverseLock;
-			ReverseLock reverse_lock(m_mutex);
-			ACE_GUARD(ReverseLock, unguard, reverse_lock);
-			
-			Client client(service_context, socket);
-			if (reactor->runEventLoop() == Reactor::FAILURE) {
-				return;
+		if (wait() == -1) {
+			if (m_numWaiters >= WAITERS_KEEP_ALIVE) {
+				break;
 			}
-			
-			service_context.reset(); // should be called after the event loop, not before it
-			reactor->restart(); // in case it have been stopped
 		}
 	}
 }
 
+/*
+Leader is a worker that is currently waiting for connections.
+For details, search google for Leader/Followers pattern.
+*/
+bool
+WorkerThreadPool::tryBecomingLeader(ServiceContext& service_context)
+{
+	// m_mutex is locked by our caller
+	
+	typedef ACE_Reverse_Lock<Mutex> ReverseLock;
+	ReverseLock reverse_lock(m_mutex);
+	ACE_Guard<ReverseLock> unguard(reverse_lock);
+	
+	// m_mutex is unlocked
+	
+	Client::SocketPtr client_sock(new RefCountableSAP<ACE_SOCK_Stream>);
+	
+	{ // leader lock scope
+		ACE_Guard<LeaderMutex> leader_guard(m_leaderMutex, /* block = */0);
+		if (!leader_guard.locked()) {
+			return false;
+		}
+		
+		ConnAcceptor::Status const conn_status = m_connAcceptor.accept(
+			*client_sock, service_context.clientAddr()
+		);
+		if (conn_status != ConnAcceptor::ACCEPTED) {
+			return true;
+		}
+	}
+	
+	// We need a new leader.
+	if (prepareNewThread() == CLOSING) {
+		return true;
+	}
+	
+	Client client(service_context, client_sock);
+	Reactor& reactor = service_context.reactor();
+	Reactor::Status const reactor_status = reactor.runEventLoop();
+	if (reactor_status == Reactor::STOPPED) {
+		reactor.restart();
+	}
+	service_context.reset(); // should be called after the event loop, not before it
+	
+	return true;
+}
+
 ACE_THR_FUNC_RETURN
 WorkerThreadPool::runThread(void* arg)
 {
