@@ -24,7 +24,8 @@
 #include <config.h>
 #endif
 
-#include "InetAddr.h"
+#include "NonCopyable.h"
+#include "ConnAcceptor.h"
 #include "RefCountableSAP.h"
 #include "IntrusivePtr.h"
 #include "IntrusiveList.h"
@@ -37,42 +38,53 @@
 #include <utility>
 
 class Reactor;
+class ServiceContext;
 
 class WorkerThreadPool
 {
+	DECLARE_NON_COPYABLE(WorkerThreadPool)
 public:
-	typedef IntrusivePtr<RefCountableSAP<ACE_SOCK_Stream> > SocketPtr;
-	
 	WorkerThreadPool();
 	
 	~WorkerThreadPool();
 	
-	void handleConnection(
-		SocketPtr const& socket, InetAddr const& client_addr);
+	void start();
+	
+	bool addAcceptor(ConnAcceptor::AcceptorPtr const& acceptor);
+	
+	void removeAcceptor(ConnAcceptor::AcceptorPtr const& acceptor);
+	
+	void removeAllAcceptors();
 private:
 	enum { WAIT_FOR_NEW_TASK = 10 }; // seconds
-	enum { WAITERS_KEEP_ALIVE = 4 };
+	enum { WAITERS_KEEP_ALIVE = 4 }; // not counting the Leader
+	enum NewThreadStatus { READY, CLOSING, FAILURE };
 	
 	class ReactorRegistration;
 	
-	typedef std::pair<SocketPtr, InetAddr> Connection;
+	typedef IntrusivePtr<RefCountableSAP<ACE_SOCK_Stream> > SocketPtr;
 	typedef IntrusiveList<ReactorRegistration> ReactorList;
 	typedef ACE_Thread_Mutex Mutex;
+	typedef ACE_Mutex LeaderMutex;
 	
+	NewThreadStatus prepareNewThread();
 	
 	int wait();
 	
 	void runService();
 	
+	bool tryBecomingLeader(ServiceContext& service_context);
+	
 	static ACE_THR_FUNC_RETURN runThread(void *arg);
 	
 	ACE_Thread_Manager m_threadManager;
 	Mutex m_mutex;
 	ACE_Condition<Mutex> m_cond;
-	std::deque<Connection> m_requestQueue;
+	LeaderMutex m_leaderMutex;
+	ConnAcceptor m_connAcceptor;
 	ReactorList m_runningReactors;
 	TimeStamp m_lastThreadCreateTime;
-	int m_numWaiters;
+	int m_numWaiters; // running threads waiting for tasks
 	bool m_isClosing;
 };
 
