@@ -33,20 +33,28 @@ using namespace std;
 
 Config::Config()
 :	m_listenAddrs(),
-	m_nextHopProxy(SymbolicInetAddr("", -1)),
+	m_nextHopProxy(),
 	m_isNextHopProxyEnabled(false),
 	m_isClientCompressionEnabled(false),
 	m_isTrayAnimationEnabled(true),
 	m_proxySkipList(),
 	m_ptrProxySkipRegex(),
 	m_ptrBorderColor(new Color(0x67, 0x67, 0x67)),
+	m_pageCleanupLevel(CLEANUP_OFF),
 	m_maxScriptNestLevel(6),
 	m_maxScriptFetchSize(45),
 	m_maxScriptEvalSize(180),
 	m_saveTrafficThreshold(15),
 	m_reportClientIP(REPORT_IP_OFF),
-	m_fixedClientIP()
+	m_fixedClientIP(),
+	m_allowedTunnelPorts()
 {
+	// backwards compatibility
+	m_nextHopProxy.setType(ProxyDescriptor::HTTP);
+	
+	// ports allowed by default
+	bool success = m_allowedTunnelPorts.fromString("443, 563");
+	assert(success);
 }
 
 Config::Config(Config const& other)
@@ -58,12 +66,14 @@ Config::Config(Config const& other)
 	m_proxySkipList(other.m_proxySkipList),
 	m_ptrProxySkipRegex(other.getProxySkipRegex()),
 	m_ptrBorderColor(other.getBorderColor()),
+	m_pageCleanupLevel(other.m_pageCleanupLevel),
 	m_maxScriptNestLevel(other.m_maxScriptNestLevel),
 	m_maxScriptFetchSize(other.m_maxScriptFetchSize),
 	m_maxScriptEvalSize(other.m_maxScriptEvalSize),
 	m_saveTrafficThreshold(other.m_saveTrafficThreshold),
 	m_reportClientIP(other.m_reportClientIP),
-	m_fixedClientIP(other.m_fixedClientIP)
+	m_fixedClientIP(other.m_fixedClientIP),
+	m_allowedTunnelPorts(other.m_allowedTunnelPorts)
 {
 }
 
@@ -85,12 +95,14 @@ Config::operator=(Config const& other)
 	m_proxySkipList = other.m_proxySkipList;
 	m_ptrProxySkipRegex = other.getProxySkipRegex();
 	m_ptrBorderColor = other.getBorderColor();
+	m_pageCleanupLevel = other.m_pageCleanupLevel;
 	m_maxScriptNestLevel = other.m_maxScriptNestLevel;
 	m_maxScriptFetchSize = other.m_maxScriptFetchSize;
 	m_maxScriptEvalSize = other.m_maxScriptEvalSize;
 	m_saveTrafficThreshold = other.m_saveTrafficThreshold;
 	m_reportClientIP = other.m_reportClientIP;
 	m_fixedClientIP = other.m_fixedClientIP;
+	m_allowedTunnelPorts = other.m_allowedTunnelPorts;
 	return *this;
 }
 
@@ -142,6 +154,12 @@ Config::setBorderColor(auto_ptr<Color> color)
 	m_ptrBorderColor = color;
 }
 
+bool
+Config::isTunnelPortAllowed(unsigned port) const
+{
+	return m_allowedTunnelPorts.contains(port);
+}
+
 auto_ptr<boost::regex>
 Config::getProxySkipRegex() const
 {
