@@ -26,6 +26,7 @@
 
 #include "SymbolicInetAddr.h"
 #include "ProxyDescriptor.h"
+#include "PortSet.h"
 #include <ace/config-lite.h>
 #include <boost/regex_fwd.hpp>
 #include <string>
@@ -39,6 +40,7 @@ class Config
 {
 public:
 	enum ReportClientIP { REPORT_IP_ON, REPORT_IP_OFF, REPORT_IP_FIXED };
+	enum PageCleanupLevel { CLEANUP_OFF, CLEANUP_SAFE, CLEANUP_MAXIMUM };
 	
 	Config();
 	
@@ -74,8 +76,12 @@ public:
 	
 	void setBorderColor(std::auto_ptr<Color> color);
 	
+	PageCleanupLevel getPageCleanupLevel() const { return m_pageCleanupLevel; }
+	
+	void setPageCleanupLevel(PageCleanupLevel level) { m_pageCleanupLevel = level; }
+	
 	bool isTrayAnimationEnabled() const { return m_isTrayAnimationEnabled; }
-
+	
 	void setTrayAnimationEnabled(bool val) { m_isTrayAnimationEnabled = val; }
 	
 	size_t getMaxScriptNestLevel() const { return m_maxScriptNestLevel; }
@@ -102,6 +108,12 @@ public:
 		m_reportClientIP = report;
 		m_fixedClientIP = fixed;
 	}
+	
+	PortSet getAllowedTunnelPorts() const { return m_allowedTunnelPorts; }
+	
+	void setAllowedTunnelPorts(PortSet const& ports) { m_allowedTunnelPorts = ports; }
+	
+	bool isTunnelPortAllowed(unsigned port) const;
 private:
 	std::auto_ptr<boost::regex> getProxySkipRegex() const;
 	
@@ -117,12 +129,14 @@ private:
 	std::list<std::string> m_proxySkipList;
 	std::auto_ptr<boost::regex> m_ptrProxySkipRegex;
 	std::auto_ptr<Color> m_ptrBorderColor;
+	PageCleanupLevel m_pageCleanupLevel;
 	size_t m_maxScriptNestLevel;
 	size_t m_maxScriptFetchSize;
 	size_t m_maxScriptEvalSize;
 	size_t m_saveTrafficThreshold;
 	ReportClientIP m_reportClientIP;
 	std::string m_fixedClientIP;
+	PortSet m_allowedTunnelPorts;
 };
 
 #endif
