@@ -30,6 +30,7 @@
 #include "Color.h"
 #include "SymbolicInetAddr.h"
 #include "ProxyDescriptor.h"
+#include "PortSet.h"
 #include "ArraySize.h"
 #include "StringUtils.h"
 #include "types.h"
@@ -92,6 +93,8 @@ private:
 	
 	bool handleAdBorder(std::string const& value, int line);
 	
+	bool handlePageCleanup(std::string const& value, int line);
+	
 	bool handleTrayIconAnimation(std::string const& value, int line);
 	
 	bool handleNoFlash(std::string const& value, int line);
@@ -106,12 +109,20 @@ private:
 	
 	bool handleReportClientIP(std::string const& value, int line);
 	
+	bool handleAllowedTunnelPorts(std::string const& value, int line);
+	
 	bool handleUseProxy(std::string const& value, int line);
 	
+	bool handleProxyType(std::string const& value, int line);
+	
 	bool handleProxyHost(std::string const& value, int line);
 	
 	bool handleProxyPort(std::string const& value, int line);
 	
+	bool handleProxyUser(std::string const& value, int line);
+	
+	bool handleProxyPass(std::string const& value, int line);
+	
 	bool handleNoProxyFor(std::string const& value, int line);
 	
 	std::list<Element>& m_rElements;
@@ -120,8 +131,11 @@ private:
 	std::string m_curSection;
 	std::set<std::string> m_sections;
 	std::set<std::string> m_keys; // within the current section
+	int m_proxyTypeLine;
 	int m_proxyHostLine;
 	int m_proxyPortLine;
+	int m_proxyUserLine;
+	int m_proxyPassLine;
 };
 
 
@@ -201,6 +215,8 @@ private:
 	
 	void handleAdBorder(Element& el);
 	
+	void handlePageCleanup(Element& el);
+	
 	void handleTrayIconAnimation(Element& el);
 	
 	void handleMaxScriptNestLevel(Element& el);
@@ -213,12 +229,20 @@ private:
 	
 	void handleReportClientIP(Element& el);
 	
+	void handleAllowedTunnelPorts(Element& el);
+	
 	void handleUseProxy(Element& el);
 	
+	void handleProxyType(Element& el);
+	
 	void handleProxyHost(Element& el);
 	
 	void handleProxyPort(Element& el);
 	
+	void handleProxyUser(Element& el);
+	
+	void handleProxyPass(Element& el);
+	
 	void handleNoProxyFor(Element& el);
 	
 	static void commentOut(Element& el);
@@ -272,8 +296,11 @@ ConfigFile::Reader::Reader(
 	m_rTarget(target),
 	m_rErrorHandler(eh),
 	m_curSection("global"), // backward compatibilty
+	m_proxyTypeLine(0),
 	m_proxyHostLine(0),
-	m_proxyPortLine(0)
+	m_proxyPortLine(0),
+	m_proxyUserLine(0),
+	m_proxyPassLine(0)
 {
 }
 
@@ -330,8 +357,22 @@ bool
 ConfigFile::Reader::finalValidation()
 {
 	if (m_rTarget.isNextHopProxyEnabled()) {
-		SymbolicInetAddr addr = m_rTarget.getNextHopProxy().getProxyAddr();
-		if (addr.getHost().empty()) {
+		ProxyDescriptor proxy(m_rTarget.getNextHopProxy());
+		
+		if (proxy.getType() == ProxyDescriptor::INVALID) {
+			// If it's not given at all, then it's HTTP,
+			// for backwards compatibility.
+			m_rTarget.setNextHopProxyEnabled(false);
+			ConfError err(
+				ConfError::T_ERROR,
+				"Illegal value for \"proxy_type\"",
+				m_proxyTypeLine
+			);
+			return m_rErrorHandler.handleError(err);
+		}
+		
+		if (proxy.getAddr().getHost().empty()) {
+			m_rTarget.setNextHopProxyEnabled(false);
 			ConfError err(
 				ConfError::T_ERROR,
 				"Illegal value for \"proxy_host\"",
@@ -339,7 +380,9 @@ ConfigFile::Reader::finalValidation()
 			);
 			return m_rErrorHandler.handleError(err);
 		}
-		if (addr.getPort() <= 0) {
+		
+		if (proxy.getAddr().getPort() <= 0) {
+			m_rTarget.setNextHopProxyEnabled(false);
 			ConfError err(
 				ConfError::T_ERROR,
 				"Illegal value for \"proxy_port\"",
@@ -347,6 +390,29 @@ ConfigFile::Reader::finalValidation()
 			);
 			return m_rErrorHandler.handleError(err);
 		}
+		
+		if (!(proxy.getUserName().empty() && proxy.getPassword().empty()) &&
+		    proxy.getType() != ProxyDescriptor::SOCKS4 &&
+		    proxy.getType() != ProxyDescriptor::SOCKS4A &&
+		    proxy.getType() != ProxyDescriptor::SOCKS5) {
+			ConfError err(
+				ConfError::T_WARNING,
+				"Sorry, authentication is only implemented for socks proxies",
+				m_proxyUserLine
+			);
+			return m_rErrorHandler.handleError(err);
+		}
+		
+		if (!proxy.getPassword().empty() &&
+		    (proxy.getType() == ProxyDescriptor::SOCKS4 ||
+		     proxy.getType() == ProxyDescriptor::SOCKS4A)) {
+			ConfError err(
+				ConfError::T_WARNING,
+				"Socks4 proxies don't use password",
+				m_proxyPassLine
+			);
+			return m_rErrorHandler.handleError(err);
+		}
 	}
 	
 	return true;
@@ -369,13 +435,15 @@ ConfigFile::Reader::processKeyValue(
 			{ "listen_address", &Reader::handleListenAddress },
 			{ "client_compression", &Reader::handleClientCompression },
 			{ "ad_border", &Reader::handleAdBorder },
+			{ "page_cleanup", &Reader::handlePageCleanup },
 			{ "tray_icon_animation", &Reader::handleTrayIconAnimation },
 			{ "no_flash", &Reader::handleNoFlash },
 			{ "max_script_nest_level", &Reader::handleMaxScriptNestLevel },
 			{ "max_script_fetch_size", &Reader::handleMaxScriptFetchSize },
 			{ "max_script_eval_size", &Reader::handleMaxScriptEvalSize },
 			{ "save_traffic_threshold", &Reader::handleSaveTrafficThreshold },
-			{ "report_client_ip", &Reader::handleReportClientIP }
+			{ "report_client_ip", &Reader::handleReportClientIP },
+			{ "allowed_tunnel_ports", &Reader::handleAllowedTunnelPorts }
 		};
 		return handleParam(
 			handlers, handlers + ARRAY_SIZE(handlers),
@@ -385,8 +453,11 @@ ConfigFile::Reader::processKeyValue(
 	} else if (section == "forwarding") {
 		static ParamHandlerNode const handlers[] = {
 			{ "use_proxy", &Reader::handleUseProxy },
+			{ "proxy_type", &Reader::handleProxyType },
 			{ "proxy_host", &Reader::handleProxyHost },
 			{ "proxy_port", &Reader::handleProxyPort },
+			{ "proxy_user", &Reader::handleProxyUser },
+			{ "proxy_pass", &Reader::handleProxyPass },
 			{ "no_proxy_for", &Reader::handleNoProxyFor }
 		};
 		return handleParam(
@@ -510,6 +581,27 @@ ConfigFile::Reader::handleAdBorder(std::string const& value, int line)
 	return true;
 }
 
+bool
+ConfigFile::Reader::handlePageCleanup(std::string const& value, int line)
+{
+	Config::PageCleanupLevel cleanup = Config::CLEANUP_OFF;
+	if (value == "maximum") {
+		cleanup = Config::CLEANUP_MAXIMUM;
+	} else if (value == "safe") {
+		cleanup = Config::CLEANUP_SAFE;
+	} else if (value == "off") {
+		cleanup = Config::CLEANUP_OFF;
+	} else {
+		ConfError err(
+			ConfError::T_ERROR,
+			"Illegal value for \"page_cleanup\"", line
+		);
+		return m_rErrorHandler.handleError(err);
+	}
+	m_rTarget.setPageCleanupLevel(cleanup);
+	return true;
+}
+
 bool
 ConfigFile::Reader::handleTrayIconAnimation(std::string const& value, int line)
 {
@@ -625,6 +717,22 @@ ConfigFile::Reader::handleReportClientIP(std::string const& value, int line)
 	return true;
 }
 
+bool
+ConfigFile::Reader::handleAllowedTunnelPorts(std::string const& value, int line)
+{
+	PortSet ports;
+	if (!ports.fromString(value)) {
+		ConfError err(
+			ConfError::T_ERROR,
+			"Error parsing \"allowed_tunnel_ports\"", line
+		);
+		return m_rErrorHandler.handleError(err);
+	}
+	
+	m_rTarget.setAllowedTunnelPorts(ports);
+	return true;
+}
+
 bool
 ConfigFile::Reader::handleUseProxy(std::string const& value, int line)
 {
@@ -639,13 +747,36 @@ ConfigFile::Reader::handleUseProxy(std::string const& value, int line)
 	return true;
 }
 
+bool
+ConfigFile::Reader::handleProxyType(std::string const& value, int line)
+{
+	m_proxyTypeLine = line;
+	ProxyDescriptor::ProxyType type = ProxyDescriptor::INVALID;
+	if (value == "http") {
+		type = ProxyDescriptor::HTTP;
+	} else if (value == "socks4") {
+		type = ProxyDescriptor::SOCKS4;
+	} else if (value == "socks4a") {
+		type = ProxyDescriptor::SOCKS4A;
+	} else if (value == "socks5") {
+		type = ProxyDescriptor::SOCKS5;
+	}
+	
+	ProxyDescriptor proxy(m_rTarget.getNextHopProxy());
+	proxy.setType(type);
+	m_rTarget.setNextHopProxy(proxy);
+	return true;
+}
+
 bool
 ConfigFile::Reader::handleProxyHost(std::string const& value, int line)
 {
 	m_proxyHostLine = line;
-	SymbolicInetAddr addr(m_rTarget.getNextHopProxy().getProxyAddr());
+	ProxyDescriptor proxy(m_rTarget.getNextHopProxy());
+	SymbolicInetAddr addr(proxy.getAddr());
 	addr.setHost(value);
-	m_rTarget.setNextHopProxy(ProxyDescriptor(addr));
+	proxy.setAddr(addr);
+	m_rTarget.setNextHopProxy(proxy);
 	return true;
 }
 
@@ -665,9 +796,33 @@ ConfigFile::Reader::handleProxyPort(std::string const& value, int line)
 			return m_rErrorHandler.handleError(err);
 		}
 	}
-	SymbolicInetAddr addr(m_rTarget.getNextHopProxy().getProxyAddr());
+	
+	ProxyDescriptor proxy(m_rTarget.getNextHopProxy());
+	SymbolicInetAddr addr(proxy.getAddr());
 	addr.setPort(port);
-	m_rTarget.setNextHopProxy(ProxyDescriptor(addr));
+	proxy.setAddr(addr);
+	m_rTarget.setNextHopProxy(proxy);
+	
+	return true;
+}
+
+bool
+ConfigFile::Reader::handleProxyUser(std::string const& value, int line)
+{
+	m_proxyUserLine = line;
+	ProxyDescriptor proxy(m_rTarget.getNextHopProxy());
+	proxy.setUserName(value);
+	m_rTarget.setNextHopProxy(proxy);
+	return true;
+}
+
+bool
+ConfigFile::Reader::handleProxyPass(std::string const& value, int line)
+{
+	m_proxyPassLine = line;
+	ProxyDescriptor proxy(m_rTarget.getNextHopProxy());
+	proxy.setPassword(value);
+	m_rTarget.setNextHopProxy(proxy);
 	return true;
 }
 
@@ -802,12 +957,14 @@ ConfigFile::Updater::handleGlobalSection(
 		{ false, "listen_address", &Updater::handleListenAddress },
 		{ false, "client_compression", &Updater::handleClientCompression },
 		{ false, "ad_border", &Updater::handleAdBorder },
+		{ false, "page_cleanup", &Updater::handlePageCleanup },
 		{ false, "tray_icon_animation", &Updater::handleTrayIconAnimation },
 		{ false, "max_script_nest_level", &Updater::handleMaxScriptNestLevel },
 		{ false, "max_script_fetch_size", &Updater::handleMaxScriptFetchSize },
 		{ false, "max_script_eval_size", &Updater::handleMaxScriptEvalSize },
 		{ false, "save_traffic_threshold", &Updater::handleSaveTrafficThreshold },
-		{ false, "report_client_ip", &Updater::handleReportClientIP }
+		{ false, "report_client_ip", &Updater::handleReportClientIP },
+		{ false, "allowed_tunnel_ports", &Updater::handleAllowedTunnelPorts }
 	};
 	ParamHandlerNode handlers[ARRAY_SIZE(handlers_proto)];
 	memcpy(handlers, handlers_proto, sizeof(handlers));
@@ -822,8 +979,11 @@ ConfigFile::Updater::handleForwardingSection(
 {
 	static ParamHandlerNode const handlers_proto[] = {
 		{ false, "use_proxy", &Updater::handleUseProxy },
+		{ false, "proxy_type", &Updater::handleProxyType },
 		{ false, "proxy_host", &Updater::handleProxyHost },
 		{ false, "proxy_port", &Updater::handleProxyPort },
+		{ false, "proxy_user", &Updater::handleProxyUser },
+		{ false, "proxy_pass", &Updater::handleProxyPass },
 		{ false, "no_proxy_for", &Updater::handleNoProxyFor }
 	};
 	ParamHandlerNode handlers[ARRAY_SIZE(handlers_proto)];
@@ -927,6 +1087,23 @@ ConfigFile::Updater::handleAdBorder(Element& el)
 	}
 }
 
+void
+ConfigFile::Updater::handlePageCleanup(Element& el)
+{
+	char const* val = 0;
+	switch (m_rSource.getPageCleanupLevel()) {
+		case Config::CLEANUP_MAXIMUM:
+		val = "maximum";
+		break;
+		case Config::CLEANUP_SAFE:
+		val = "safe";
+		break;
+		default:
+		val = "off";
+	}
+	el.setValue(val);
+}
+
 void
 ConfigFile::Updater::handleTrayIconAnimation(Element& el)
 {
@@ -971,6 +1148,14 @@ ConfigFile::Updater::handleReportClientIP(Element& el)
 	}
 }
 
+void
+ConfigFile::Updater::handleAllowedTunnelPorts(Element& el)
+{
+	ostringstream strm;
+	m_rSource.getAllowedTunnelPorts().toStream(strm);
+	el.setValue(strm.str());
+}
+
 void
 ConfigFile::Updater::handleUseProxy(Element& el)
 {
@@ -978,16 +1163,38 @@ ConfigFile::Updater::handleUseProxy(Element& el)
 	el.setValue(val);
 }
 
+void
+ConfigFile::Updater::handleProxyType(Element& el)
+{
+	string val;
+	switch (m_rSource.getNextHopProxy().getType()) {
+		case ProxyDescriptor::INVALID:
+		case ProxyDescriptor::HTTP:
+		val = "http";
+		break;
+		case ProxyDescriptor::SOCKS4:
+		val = "socks4";
+		break;
+		case ProxyDescriptor::SOCKS4A:
+		val = "socks4a";
+		break;
+		case ProxyDescriptor::SOCKS5:
+		val = "socks5";
+		break;
+	}
+	el.setValue(val);
+}
+
 void
 ConfigFile::Updater::handleProxyHost(Element& el)
 {
-	el.setValue(m_rSource.getNextHopProxy().getProxyAddr().getHost());
+	el.setValue(m_rSource.getNextHopProxy().getAddr().getHost());
 }
 
 void
 ConfigFile::Updater::handleProxyPort(Element& el)
 {
-	int port = m_rSource.getNextHopProxy().getProxyAddr().getPort();
+	int port = m_rSource.getNextHopProxy().getAddr().getPort();
 	if (port <= 0) {
 		el.setValue("");
 	} else {
@@ -995,6 +1202,18 @@ ConfigFile::Updater::handleProxyPort(Element& el)
 	}
 }
 
+void
+ConfigFile::Updater::handleProxyUser(Element& el)
+{
+	el.setValue(m_rSource.getNextHopProxy().getUserName());
+}
+
+void
+ConfigFile::Updater::handleProxyPass(Element& el)
+{
+	el.setValue(m_rSource.getNextHopProxy().getPassword());
+}
+
 void
 ConfigFile::Updater::handleNoProxyFor(Element& el)
 {
