@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 #include "StringUtils.h"
 #include "IntrusivePtr.h"
 #include "ArraySize.h"
-#include <ace/config.h>
+#include <ace/config-lite.h>
 #include <boost/regex.hpp>
 #include <boost/lexical_cast.hpp>
 #include <set>
@@ -337,7 +337,7 @@ ContentFiltersFile::Reader::processElement(Element const& element, int line)
 			return false;
 		}
 		m_ptrFilter.reset(new RegexFilterDescriptor(
-			FilterTag::create(), m_groupTag, element.getText()
+			FilterTag(), m_groupTag, element.getText()
 		));
 		m_sections.insert(element.getText());
 		m_keys.clear();
