@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,27 +20,51 @@
 #ifndef FILTERGROUPTAG_H_
 #define FILTERGROUPTAG_H_
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include "Tag.h"
 #include "RefCountable.h"
 #include "RefCounter.h"
-#include "IntrusivePtr.h"
 
-class FilterGroupTag :
-	public IntrusivePtr<RefCountable<RefCounter<ACE_MT_SYNCH> > >
+class ACE_MT_SYNCH;
+
+class FilterGroupTagRep : public RefCountable<RefCounter<ACE_MT_SYNCH> >
 {
-private:
-	typedef RefCountable<RefCounter<ACE_MT_SYNCH> > Rep;
-	
-	FilterGroupTag() {}
-	
-	FilterGroupTag(Rep* rep) : IntrusivePtr<Rep>(rep) {}
-public:
-	static FilterGroupTag create() {
-		return FilterGroupTag(new Rep);
-	}
 };
 
+class FilterGroupTag : public Tag<FilterGroupTagRep>
+{
+public:
+	FilterGroupTag() {}
+protected:
+	FilterGroupTag(FilterGroupTagRep* obj) : Tag<FilterGroupTagRep>(obj) {}
+};
+
+class WildcardFilterGroupTag : public FilterGroupTag
+{
+public:
+	WildcardFilterGroupTag() : FilterGroupTag(0) {}
+};
+
+
+inline bool operator<(FilterGroupTag const& lhs, FilterGroupTag const& rhs)
+{
+	if (!lhs || !rhs) {
+		// Wildcard tag is equivalent to any other tag.
+		return false;
+	}
+	
+	typedef Tag<FilterGroupTagRep> Base;
+	return static_cast<Base const&>(lhs) < static_cast<Base const&>(rhs);
+}
+
+inline bool operator>(FilterGroupTag const& lhs, FilterGroupTag const& rhs)
+{
+	if (!lhs || !rhs) {
+		// Wildcard tag is equivalent to any other tag.
+		return false;
+	}
+	
+	typedef Tag<FilterGroupTagRep> Base;
+	return static_cast<Base const&>(lhs) > static_cast<Base const&>(rhs);
+}
+
 #endif
