@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,27 +20,18 @@
 #ifndef FILTERTAG_H_
 #define FILTERTAG_H_
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include "Tag.h"
 #include "RefCountable.h"
 #include "RefCounter.h"
-#include "IntrusivePtr.h"
 
-class FilterTag :
-	public IntrusivePtr<RefCountable<RefCounter<ACE_MT_SYNCH> > >
+class ACE_MT_SYNCH;
+
+class FilterTagRep : public RefCountable<RefCounter<ACE_MT_SYNCH> >
+{
+};
+
+class FilterTag : public Tag<FilterTagRep>
 {
-private:
-	typedef RefCountable<RefCounter<ACE_MT_SYNCH> > Rep;
-	
-	FilterTag() {}
-	
-	FilterTag(Rep* rep) : IntrusivePtr<Rep>(rep) {}
-public:
-	static FilterTag create() {
-		return FilterTag(new Rep);
-	}
 };
 
 #endif
