@@ -22,6 +22,7 @@
 #endif
 
 #include "Daemon.h"
+#include "NonCopyable.h"
 #include "Conf.h"
 #include "ConfError.h"
 #include "ConfErrorHandler.h"
@@ -42,13 +43,18 @@
 #include "AutoClosingSAP.h"
 #include "RefCountableSAP.h"
 #include "StringUtils.h"
+#include "ArraySize.h"
 #include "Alarm.h"
+#include "types.h"
 #ifdef ENABLE_BINRELOC
 #include "binreloc/prefix.h"
 #endif
 #ifdef ENABLE_PROXYWATCHER
 #include "ProxyWatcherThread.h"
 #endif
+#ifdef ENABLE_RECONFIGURE
+#include "LocationUpdater.h"
+#endif
 #include <ace/config-lite.h>
 #include <ace/Dirent.h>
 #include <ace/SOCK_Acceptor.h>
@@ -66,6 +72,7 @@
 #include <vector>
 #include <list>
 #include <cstdlib>
+#include <exception>
 #include <sys/types.h>
 #include <sys/file.h>
 #include <unistd.h>
@@ -76,6 +83,28 @@
 namespace po = boost::program_options;
 using namespace std;
 
+class Daemon::ScopeSetEUID
+{
+public:
+	ScopeSetEUID(uid_t euid);
+	
+	~ScopeSetEUID();
+private:
+	uid_t m_prevEUID;
+};
+
+
+class Daemon::ScopeSetEGID
+{
+public:
+	ScopeSetEGID(gid_t egid);
+	
+	~ScopeSetEGID();
+private:
+	uid_t m_prevEGID;
+};
+
+
 class Daemon::ConfigErrorHandler : public ConfErrorHandler
 {
 public:
@@ -94,11 +123,8 @@ private:
 class Daemon::ConfigLoader
 {
 public:
-	bool load(std::string const& confdir);
-	
-	Config const& getConfig() const { return m_config; }
-private:
-	Config m_config;
+	bool load(std::string const& confdir,
+		Config& target, ConfigFile& target_structure);
 };
 
 
@@ -134,6 +160,71 @@ private:
 };
 
 
+#if defined(ENABLE_RECONFIGURE)
+
+class Daemon::Reconfigurer
+{
+	DECLARE_NON_COPYABLE(Reconfigurer)
+public:
+	Reconfigurer(std::string const& confdir, uid_t bind_uid, gid_t bind_gid);
+	
+	~Reconfigurer();
+	
+	bool reconfigure();
+private:
+	bool loadConfig();
+	
+	SymbolicInetAddr checkAndFixListenAddr(SymbolicInetAddr const& addr);
+	
+	uint16_t tryBind(InetAddr const& addr);
+	
+	uint16_t tryBindWithFeedback(InetAddr const& addr);
+	
+	bool writeListenAddrToConfig(SymbolicInetAddr const& addr);
+	
+	bool updateNetworkProfiles(int listen_port);
+	
+	static uint16_t const m_sAlternativePorts[];
+	std::string m_confDir;
+	uid_t m_bindUID;
+	gid_t m_bindGID;
+	SymbolicInetAddr const m_defaultListenAddr;
+	SymbolicInetAddr const m_badAddr;
+	Config m_config;
+	ConfigFile m_configFile;
+};
+
+#endif // ENABLE_RECONFIGURE
+
+
+/*========================= Daemon::ScopeSetEUID =========================*/
+
+Daemon::ScopeSetEUID::ScopeSetEUID(uid_t euid)
+{
+	m_prevEUID = geteuid();
+	seteuid(euid);
+}
+
+Daemon::ScopeSetEUID::~ScopeSetEUID()
+{
+	seteuid(m_prevEUID);
+}
+
+
+/*========================= Daemon::ScopeSetEGID =========================*/
+
+Daemon::ScopeSetEGID::ScopeSetEGID(gid_t egid)
+{
+	m_prevEGID = getegid();
+	setegid(egid);
+}
+
+Daemon::ScopeSetEGID::~ScopeSetEGID()
+{
+	setegid(m_prevEGID);
+}
+
+
 /* ==================== Daemon::ConfigErrorHandler =======================*/
 
 bool
@@ -161,7 +252,8 @@ Daemon::ConfigErrorHandler::handleError(ConfError const& err)
 /* ====================== Daemon::ConfigLoader =========================*/
 
 bool
-Daemon::ConfigLoader::load(std::string const& confdir)
+Daemon::ConfigLoader::load(
+	std::string const& confdir, Config& target, ConfigFile& target_structure)
 {
 	string const fname("config");
 	string const abs_fname(confdir+"/"+fname);
@@ -177,9 +269,8 @@ Daemon::ConfigLoader::load(std::string const& confdir)
 		return false;
 	}
 	
-	ConfigFile cfile;
 	ConfigErrorHandler eh(fname);
-	cfile.load(ostrm.str(), m_config, eh);
+	target_structure.load(ostrm.str(), target, eh);
 	
 	if (eh.hasCriticalErrors()) {
 		return false;
@@ -341,6 +432,219 @@ Daemon::ContentFiltersLoader::appendEnabledFilters(ContentFilters const& filters
 }
 
 
+/* ====================== Daemon::Reconfigurer =========================*/
+
+#if defined(ENABLE_RECONFIGURE)
+
+uint16_t const Daemon::Reconfigurer::m_sAlternativePorts[] = {
+	3128, 8099, 8111, 8188, 0
+	// 0 will cause the OS to select an available port 
+};
+
+Daemon::Reconfigurer::Reconfigurer(
+	std::string const& confdir, uid_t bind_uid, gid_t bind_gid)
+:	m_confDir(confdir),
+	m_bindUID(bind_uid),
+	m_bindGID(bind_gid),
+	m_defaultListenAddr("127.0.0.1", 8080),
+	m_badAddr(string(), -1)
+{
+}
+
+Daemon::Reconfigurer::~Reconfigurer()
+{
+}
+
+bool
+Daemon::Reconfigurer::reconfigure()
+{
+	if (getuid() != 0 || geteuid() != 0) {
+		std::cout << "Reconfiguring must be done by root" << std::endl;
+		return false;
+	}
+	
+	loadConfig(); // if it fails, it's not a problem
+	
+	SymbolicInetAddr user_listen_addr(string(), -1);
+	
+	list<SymbolicInetAddr> addrs(m_config.getListenAddresses());
+	if (!addrs.empty()) {
+		user_listen_addr = addrs.front();
+	}
+	if (addrs.size() > 1 ||
+	    user_listen_addr.getHost() != m_defaultListenAddr.getHost()) {
+		// We've got a power user here who modified settings by hand.
+		// It's better to leave them as they are.
+		return updateNetworkProfiles(user_listen_addr.getPort());
+	}
+	
+	SymbolicInetAddr listen_addr(checkAndFixListenAddr(user_listen_addr));
+	if (listen_addr == m_badAddr) {
+		return false;
+	}
+	
+	if (listen_addr != user_listen_addr) {
+		if (!writeListenAddrToConfig(listen_addr)) {
+			return false;
+		}
+	}
+	
+	return updateNetworkProfiles(listen_addr.getPort());
+}
+
+bool
+Daemon::Reconfigurer::loadConfig()
+{
+	ConfigLoader loader;
+	return loader.load(m_confDir, m_config, m_configFile);
+}
+
+SymbolicInetAddr
+Daemon::Reconfigurer::checkAndFixListenAddr(SymbolicInetAddr const& addr)
+{
+	std::cout << "Checking listen address" << std::endl;
+	
+	SymbolicInetAddr new_addr(addr);
+	if (new_addr.getPort() <= 0 || new_addr.getPort() >= 1<<16) {
+		new_addr = m_defaultListenAddr;
+	}
+	
+	vector<InetAddr> resolved_addrs(DnsResolver::resolve(new_addr));
+	if (resolved_addrs.empty() && new_addr != m_defaultListenAddr) {
+		// This indicates a problem with host, not with port.
+		std::cout << "Could not resolve listen address. "
+			"Falling back to the default one." << std::endl;
+		new_addr = m_defaultListenAddr;
+		DnsResolver::resolve(new_addr).swap(resolved_addrs);
+	}
+	if (resolved_addrs.empty()) {
+		std::cout << "Could not resolve listen address " << new_addr << std::endl;
+		return m_badAddr;
+	}
+	
+	InetAddr try_addr(resolved_addrs[0]);
+	uint16_t port = tryBindWithFeedback(try_addr);
+	if (port) {
+		new_addr.setPort(port);
+		return new_addr;
+	}
+	
+	for (unsigned i = 0; i < ARRAY_SIZE(m_sAlternativePorts); ++i) {
+		try_addr.set_port_number(m_sAlternativePorts[i]);
+		port = tryBindWithFeedback(try_addr);
+		if (port) {
+			new_addr.setPort(port);
+			return new_addr;
+		}
+	}
+	
+	return m_badAddr;
+}
+
+uint16_t
+Daemon::Reconfigurer::tryBind(InetAddr const& addr)
+{
+	ScopeSetEGID gid_setter(m_bindGID);
+	ScopeSetEUID uid_setter(m_bindUID);
+	
+	/*
+	Why can't we just do the bind as root?
+	Consider the following situation:
+	Apache listens on INADDR_ANY:8080.
+	We are trying to bind to 127.0.0.1:8080 and are using SO_REUSEADDR.
+	In *BSD systems (including OSX), the following rule is applied:
+	You are allowed to "steal" a bound address from another process
+	if you have the same uids or you are root. That's why we
+	temporarely drop our priviledges here.
+	*/
+	
+	AutoClosingSAP<ACE_SOCK_Acceptor> acceptor;
+	if (acceptor.open(addr, /* reuse_addr = */1) != -1) {
+		InetAddr new_addr;
+		if (acceptor.get_local_addr(new_addr) != -1) {
+			return new_addr.get_port_number();
+		}
+	}
+	
+	return 0;
+}
+
+uint16_t
+Daemon::Reconfigurer::tryBindWithFeedback(InetAddr const& addr)
+{
+	uint16_t const req_port = addr.get_port_number();
+	if (req_port == 0) {
+		std::cout << "Letting OS to select a port for us" << std::endl;
+	} else {
+		std::cout << "Trying port " << req_port << " ..." << std::endl;
+	}
+	
+	uint16_t const res_port = tryBind(addr);
+	if (res_port == 0) {
+		std::cout << "Port is unavailable" << std::endl;
+	} else {
+		std::cout << "Port " << res_port << " is available" << std::endl;
+	}
+	
+	return res_port;
+}
+
+bool
+Daemon::Reconfigurer::writeListenAddrToConfig(SymbolicInetAddr const& addr)
+{
+	list<SymbolicInetAddr> addrs;
+	addrs.push_back(addr);
+	m_config.setListenAddresses(addrs);
+	m_configFile.updateWith(m_config);
+	
+	std::string fname = m_confDir+"/config";
+	ofstream strm(fname.c_str());
+	if (strm.fail()) {
+		std::cout << "Could not open "+fname+" for writing." << std::endl;
+		return false;
+	}
+	m_configFile.toStream(strm);
+	if (strm.bad()) {
+		std::cout << "Error writing to "+fname << std::endl;
+		return false;
+	}
+	
+	return true;
+}
+
+bool
+Daemon::Reconfigurer::updateNetworkProfiles(int listen_port)
+{
+	std::cout << "Updating network profiles" << std::endl;
+	try {
+		LocationUpdater prefs;
+		for (int i = 0; i < 2; ++i) {
+			if (prefs.tryLock() || SCError() != kSCStatusPrefsBusy) {
+				break;
+			}
+			sleep(1);
+		}
+		if (!prefs.isLocked()) {
+			std::cout << "Unable to lock the preferences" << std::endl;
+			return false;
+		}
+		
+		prefs.updateLocations(listen_port);
+		if (!prefs.applyChanges()) {
+			std::cout << "Could not apply changes" << std::endl;
+			return false;
+		}
+	} catch (std::exception& e) {
+		std::cout << "Error updating network locations: " << e.what() << std::endl;
+		return false;
+	}
+	
+	return true;
+}
+
+#endif // ENABLE_RECONFIGURE
+
+
 /*=========================== Daemon ==================================*/
 
 volatile bool Daemon::m_sSigtermReceived = false;
@@ -387,8 +691,10 @@ Daemon::run(bool nodaemon, ACE_FILE_IO& pid_file)
 	ProxyWatcherThread proxy_wather;
 #endif
 	
+	m_workerPool.start();
+	
 	while (!m_sSigtermReceived && !m_sSigintReceived) {
-		m_acceptor.acceptConnections();
+		sleep(10000);
 	}
 	
 	if (pid_file.get_handle() != ACE_INVALID_HANDLE) {
@@ -519,8 +825,13 @@ bool
 Daemon::loadConfiguration(std::string const& confdir)
 {
 	ConfigLoader config_loader;
-	if (!config_loader.load(confdir)) {
-		return false;
+	Config config;
+	
+	{
+		ConfigFile config_file;
+		if (!config_loader.load(confdir, config, config_file)) {
+			return false;
+		}
 	}
 	
 	UrlPatternsLoader patterns_loader;
@@ -534,7 +845,7 @@ Daemon::loadConfiguration(std::string const& confdir)
 	}
 	
 	GlobalState::WriteAccessor global_state;
-	global_state->config() = config_loader.getConfig();
+	global_state->config() = config;
 	global_state->urlPatterns() = patterns_loader.getCombinedPatterns();
 	global_state->contentFilters() = cf_loader.getFilters();
 	
@@ -549,10 +860,10 @@ Daemon::bind()
 	
 	bool ok = true;
 	deque<AcceptorPtr> acceptors;
-	list<SymbolicInetAddr> addrs = GlobalState::ReadAccessor()->config().getListenAddresses();
+	list<SymbolicInetAddr> addrs(GlobalState::ReadAccessor()->config().getListenAddresses());
 	for (; !addrs.empty(); addrs.pop_front()) {
 		SymbolicInetAddr const& addr = addrs.front();
-		vector<InetAddr> resolved_addrs = DnsResolver::resolve(addr);
+		vector<InetAddr> resolved_addrs(DnsResolver::resolve(addr));
 		if (resolved_addrs.empty()) {
 			ok = false;
 			cerr << "Could not resolve listen address \"" << addr << '"' << endl;
@@ -575,7 +886,7 @@ Daemon::bind()
 	}
 	
 	for (; !acceptors.empty(); acceptors.pop_front()) {
-		m_acceptor.add(acceptors.front());
+		m_workerPool.addAcceptor(acceptors.front());
 	}
 	
 	return true;
@@ -601,6 +912,7 @@ int main(int argc, char **argv)
 {
 	bool nodaemon = false;
 	bool kill_running = false;
+	bool reconfigure = false;
 	string confdir;
 	string chroot_dir;
 	string effective_confdir; // relative to chroot_dir
@@ -631,7 +943,13 @@ int main(int argc, char **argv)
 				"[with -k]    Kill the running instance")
 			("kill,k", po::bool_switch(&kill_running),
 				"Kill the running instance. To be used with -p")
+#if defined(ENABLE_RECONFIGURE)
+			// This one is used by the installer
+			("reconfigure", po::bool_switch(&reconfigure),
+				"Find a suitable listen port and update network locations")
+#endif
 		;
+		
 		po::variables_map vm;
 		po::store(po::parse_command_line(argc, argv, desc), vm);
 		po::notify(vm);
@@ -729,6 +1047,20 @@ int main(int argc, char **argv)
 		}
 	}
 	
+#if defined(ENABLE_RECONFIGURE)
+	// This must come before dropping priviledges or chroot()ing.
+	if (reconfigure) {
+		uid_t const bind_uid = user_r ? user_r->pw_uid : geteuid();
+		gid_t const bind_gid = group_r ? group_r->gr_gid : getegid();
+		Daemon::Reconfigurer reconfigurer(confdir, bind_uid, bind_gid);
+		if (reconfigurer.reconfigure()) {
+			return EXIT_SUCCESS;
+		} else {
+			return EXIT_FAILURE;
+		}
+	}
+#endif
+	
 	if (group_r) {
 		if (setgid(group_r->gr_gid)) {
 			std::cerr << "Could not drop priviliges: setgid() failed" << std::endl;
