@@ -377,6 +377,9 @@ AdvancedConfigWindow::~AdvancedConfigWindow()
 {
 	assert(m_spInstance);
 	m_spInstance = 0;
+	
+	// prevent slots being called during destruction
+	sigc::trackable::notify_callbacks();
 }
 
 void
@@ -583,6 +586,8 @@ AdvancedConfigWindow::Page::Page(
 
 AdvancedConfigWindow::Page::~Page()
 {
+	// prevent slots being called during destruction
+	sigc::trackable::notify_callbacks();
 }
 
 void
