@@ -45,7 +45,7 @@
 #include "IntrusivePtr.h"
 #include "RefCountableSAP.h"
 #include "AutoClosingSAP.h"
-#include "AcceptorThread.h"
+#include "WorkerThreadPool.h"
 #include "ScopedIncDec.h"
 #include "EffectiveFileTimestamps.h"
 #ifdef DEBUG
@@ -114,7 +114,7 @@ Application::Application(int argc, char* argv[])
 #ifdef DEBUG
 	m_ptrDebugWindow.reset(new DebugWindow);
 #endif
-	m_ptrAcceptorThread.reset(new AcceptorThread);
+	m_ptrWorkerPool.reset(new WorkerThreadPool);
 }
 
 Application::~Application()
@@ -134,8 +134,12 @@ Application::run()
 	if (errors) {
 		showLogDialog();
 	}
+	
+	m_ptrWorkerPool->start();
 	m_gtkMain.run();
-	m_ptrAcceptorThread.reset(0);
+	
+	// This will finish the worker threads.
+	m_ptrWorkerPool.reset(0);
 }
 
 void
@@ -424,7 +428,7 @@ Application::applyConfig(Config const& config)
 	// We apply the config independently of the result of the
 	// bind operation, to make the Basic Config dialog work as expected.
 	
-	m_ptrAcceptorThread->removeAll();
+	m_ptrWorkerPool->removeAllAcceptors();
 	setAcceptingConnections(false);
 	
 	bool ok = true;
@@ -462,7 +466,7 @@ Application::applyConfig(Config const& config)
 	}
 	
 	for (; !acceptors.empty(); acceptors.pop_front()) {
-		m_ptrAcceptorThread->add(acceptors.front());
+		m_ptrWorkerPool->addAcceptor(acceptors.front());
 	}
 	setAcceptingConnections(true);
 	
@@ -475,7 +479,7 @@ Application::applyBrokenConfig(Config const& broken_config)
 	GlobalState::WriteAccessor()->config() = broken_config;
 	// We apply it anyway, to make the Basic Config dialog work as expected.
 	
-	m_ptrAcceptorThread->removeAll();
+	m_ptrWorkerPool->removeAllAcceptors();
 	setAcceptingConnections(false);
 }
 
