@@ -50,7 +50,7 @@ class UrlPatterns;
 class ContentFilters;
 class TrayIcon;
 class TrayMenu;
-class AcceptorThread;
+class WorkerThreadPool;
 class DebugWindow;
 class ACE_FILE_IO;
 
@@ -252,7 +252,7 @@ private:
 	std::auto_ptr<DebugWindow> m_ptrDebugWindow;
 #endif
 	bool m_isAcceptingConnections;
-	std::auto_ptr<AcceptorThread> m_ptrAcceptorThread;
+	std::auto_ptr<WorkerThreadPool> m_ptrWorkerPool;
 	// Must be the last thing to construct and first to destroy
 	// because we don't want worker threads to act on destroyed objects.
 };
