@@ -33,10 +33,12 @@
 #include "SymbolicInetAddr.h"
 #include "CompiledImages.h"
 #include "EffectiveFileTimestamps.h"
+#include "ArraySize.h"
 #include "types.h"
 #include <boost/lexical_cast.hpp>
 #include <gtkmm/entry.h>
 #include <gtkmm/checkbutton.h>
+#include <gtkmm/comboboxtext.h>
 #include <gtkmm/box.h>
 #include <gtkmm/buttonbox.h>
 #include <gtkmm/button.h>
@@ -44,6 +46,7 @@
 #include <gtkmm/label.h>
 #include <gtkmm/frame.h>
 #include <gtkmm/stock.h>
+#include <gtkmm/table.h>
 #include <gtkmm/messagedialog.h>
 #include <list>
 #include <sstream>
@@ -52,9 +55,32 @@
 
 using namespace std;
 
+struct BasicConfigDialog::ProxyOption
+{
+	ProxyDescriptor::ProxyType type;
+	char const* label;
+};
+
+
 BasicConfigDialog* BasicConfigDialog::m_spInstance = 0;
 
+BasicConfigDialog::ProxyOption const BasicConfigDialog::m_sProxyOptions[] = {
+	{ ProxyDescriptor::INVALID, "No forwarding" },
+	{ ProxyDescriptor::HTTP, "HTTP proxy" },
+	{ ProxyDescriptor::SOCKS4, "SOCKS4 proxy" },
+	{ ProxyDescriptor::SOCKS4A, "SOCKS4A proxy" },
+	{ ProxyDescriptor::SOCKS5, "SOCKS5 proxy" }
+};
+
+
 BasicConfigDialog::BasicConfigDialog()
+:	m_pListenPort(0),
+	m_pTrayAnimation(0),
+	m_pProxyType(0),
+	m_pProxyHost(0),
+	m_pProxyPort(0),
+	m_pProxyUser(0),
+	m_pProxyPass(0)
 {
 	set_title("Basic Configuration");
 	set_icon(CompiledImages::window_icon_png.getPixbuf());
@@ -83,10 +109,17 @@ BasicConfigDialog::BasicConfigDialog()
 	fwd_frame->add(*vbox2);
 	
 	vbox2->set_border_width(5);
-	vbox2->set_spacing(2);
+	vbox2->set_spacing(3);
+	
+	m_pProxyType = manage(new Gtk::ComboBoxText);
+	vbox2->pack_start(*m_pProxyType, Gtk::PACK_SHRINK);
+	for (int i = 0; i < (int)ARRAY_SIZE(m_sProxyOptions); ++i) {
+		m_pProxyType->append_text(m_sProxyOptions[i].label);
+	}
+	m_pProxyType->signal_changed().connect(
+		sigc::mem_fun(*this, &BasicConfigDialog::onProxyTypeChanged)
+	);
 	
-	m_pForward = manage(new Gtk::CheckButton("Forward to another proxy"));
-	vbox2->pack_start(*m_pForward);
 	Gtk::HBox* hbox2 = manage(new Gtk::HBox);
 	vbox2->pack_start(*hbox2, Gtk::PACK_SHRINK);
 	hbox2->set_spacing(5);
@@ -99,6 +132,29 @@ BasicConfigDialog::BasicConfigDialog()
 	hbox2->pack_start(*m_pProxyPort);
 	m_pProxyPort->set_size_request(50);
 	
+	Gtk::Table* user_pass_table = manage(new Gtk::Table(2, 2));
+	vbox2->pack_start(*user_pass_table, Gtk::PACK_SHRINK);
+	user_pass_table->set_row_spacings(3);
+	user_pass_table->set_col_spacings(3);
+	
+	user_pass_table->attach(
+		*manage(new Gtk::Label("Username:", Gtk::ALIGN_RIGHT)),
+		0, 1, 0, 1, Gtk::SHRINK
+	);
+	
+	m_pProxyUser = manage(new Gtk::Entry);
+	user_pass_table->attach(*m_pProxyUser, 1, 2, 0, 1);
+	m_pProxyUser->set_size_request(100);
+	
+	user_pass_table->attach(
+		*manage(new Gtk::Label("Password:", Gtk::ALIGN_RIGHT)),
+		0, 1, 1, 2, Gtk::SHRINK
+	);
+	
+	m_pProxyPass = manage(new Gtk::Entry);
+	user_pass_table->attach(*m_pProxyPass, 1, 2, 1, 2);
+	m_pProxyPass->set_size_request(100);
+	
 	Gtk::Alignment* spacer1(new Gtk::Alignment);
 	vbox->pack_start(*spacer1);
 	spacer1->set_size_request(3, 3);
@@ -129,6 +185,7 @@ BasicConfigDialog::BasicConfigDialog()
 		sigc::mem_fun(*this, &BasicConfigDialog::onApply)
 	);
 	
+	setProxyType(ProxyDescriptor::INVALID);
 	loadConfig();
 	cancel_btn->grab_focus();
 	show_all_children();
@@ -160,10 +217,61 @@ BasicConfigDialog::on_hide()
 	delete this;
 }
 
+void
+BasicConfigDialog::onProxyTypeChanged()
+{
+	switch (getProxyType()) {
+		case ProxyDescriptor::INVALID:
+		m_pProxyHost->set_sensitive(false);
+		m_pProxyPort->set_sensitive(false);
+		m_pProxyUser->set_sensitive(false);
+		m_pProxyPass->set_sensitive(false);
+		break;
+		case ProxyDescriptor::HTTP:
+		m_pProxyHost->set_sensitive(true);
+		m_pProxyPort->set_sensitive(true);
+		m_pProxyUser->set_sensitive(false);
+		m_pProxyPass->set_sensitive(false);
+		break;
+		case ProxyDescriptor::SOCKS4:
+		case ProxyDescriptor::SOCKS4A:
+		m_pProxyHost->set_sensitive(true);
+		m_pProxyPort->set_sensitive(true);
+		m_pProxyUser->set_sensitive(true);
+		m_pProxyPass->set_sensitive(false);
+		break;
+		case ProxyDescriptor::SOCKS5:
+		m_pProxyHost->set_sensitive(true);
+		m_pProxyPort->set_sensitive(true);
+		m_pProxyUser->set_sensitive(true);
+		m_pProxyPass->set_sensitive(true);
+		break;
+	}
+}
+
+ProxyDescriptor::ProxyType
+BasicConfigDialog::getProxyType() const
+{
+	int const row = m_pProxyType->get_active_row_number();
+	assert(row >= 0);
+	return m_sProxyOptions[row].type;
+}
+
+void
+BasicConfigDialog::setProxyType(ProxyDescriptor::ProxyType type)
+{
+	for (int i = 0; i < (int)ARRAY_SIZE(m_sProxyOptions); ++i) {
+		if (m_sProxyOptions[i].type == type) {
+			m_pProxyType->set_active(i);
+			break;
+		}
+	}
+}
+
 void
 BasicConfigDialog::loadConfig()
 {
-	Config config = GlobalState::ReadAccessor()->config();
+	Config config(GlobalState::ReadAccessor()->config());
 	list<SymbolicInetAddr> listen_addrs = config.getListenAddresses();
 	if (listen_addrs.size() > 1) {
 		setCurrentListenPort(Glib::ustring());
@@ -179,16 +287,20 @@ BasicConfigDialog::loadConfig()
 		setCurrentListenPort(Glib::ustring());
 	}
 	m_pTrayAnimation->set_active(config.isTrayAnimationEnabled());
-	m_pForward->set_active(config.isNextHopProxyEnabled());
-	ProxyDescriptor pdesc = config.getNextHopProxy();
-	m_pProxyHost->set_text(pdesc.getProxyAddr().getHost());
-	if (pdesc.getProxyAddr().getPort() <= 0) {
+	
+	ProxyDescriptor pdesc(config.getNextHopProxy());
+	setProxyType(config.isNextHopProxyEnabled() ? pdesc.getType() : pdesc.INVALID);
+	
+	m_pProxyHost->set_text(Application::localeToUtf8(pdesc.getAddr().getHost()));
+	if (pdesc.getAddr().getPort() <= 0) {
 		m_pProxyPort->set_text(Glib::ustring());
 	} else {
 		m_pProxyPort->set_text(
-			StringUtils::fromNumber(pdesc.getProxyAddr().getPort())
+			StringUtils::fromNumber(pdesc.getAddr().getPort())
 		);
-	}	
+	}
+	m_pProxyUser->set_text(Application::localeToUtf8(pdesc.getUserName()));
+	m_pProxyUser->set_text(Application::localeToUtf8(pdesc.getPassword()));
 }
 
 void
@@ -215,8 +327,12 @@ BasicConfigDialog::onApply()
 		}
 	}
 	
-	bool const forwarding = m_pForward->get_active();
-	string proxy_host = StringUtils::trimStr(m_pProxyHost->get_text().c_str());	
+	ProxyDescriptor::ProxyType const proxy_type = getProxyType();
+	bool const forwarding = (proxy_type != ProxyDescriptor::INVALID);
+	
+	string proxy_host(StringUtils::trimStr(
+		Application::localeFromUtf8(m_pProxyHost->get_text())
+	));
 	if (forwarding && proxy_host.empty()) {
 		showError("Remote proxy host is not given");
 		m_pProxyHost->grab_focus();
@@ -234,6 +350,15 @@ BasicConfigDialog::onApply()
 		return;
 	}
 	
+	string proxy_user;
+	string proxy_pass;
+	if (proxy_type != ProxyDescriptor::HTTP || forwarding == false) {
+		proxy_user = Application::localeFromUtf8(m_pProxyUser->get_text());
+	}
+	if (proxy_type == ProxyDescriptor::SOCKS5 || forwarding == false) {
+		proxy_pass = Application::localeFromUtf8(m_pProxyPass->get_text());
+	}
+	
 	Config const old_config(GlobalState::ReadAccessor()->config());
 	Config new_config(old_config);
 	if (listen_port_changed) {
@@ -243,11 +368,17 @@ BasicConfigDialog::onApply()
 	}
 	new_config.setTrayAnimationEnabled(m_pTrayAnimation->get_active());
 	new_config.setNextHopProxyEnabled(forwarding);
+	
 	SymbolicInetAddr proxy_addr(proxy_host, -1);
 	if (proxy_port > 0) {
 		proxy_addr.setPort(proxy_port);
 	}
-	new_config.setNextHopProxy(ProxyDescriptor(proxy_addr));
+	ProxyDescriptor proxy(old_config.getNextHopProxy());
+	proxy.setType(proxy_type);
+	proxy.setAddr(proxy_addr);
+	proxy.setUserName(proxy_user);
+	proxy.setPassword(proxy_pass);
+	new_config.setNextHopProxy(proxy);
 	
 	Application& app = *Application::instance();
 	Log* log = OperationLog::instance();
