@@ -24,13 +24,16 @@
 #include <config.h>
 #endif
 
+#include "ProxyDescriptor.h"
 #include <gtkmm/window.h>
 #include <glibmm/ustring.h>
+#include <glibmm/refptr.h>
 
 class Config;
 namespace Gtk {
 	class Entry;
 	class CheckButton;
+	class ComboBoxText;
 }
 
 class BasicConfigDialog : public Gtk::Window
@@ -42,8 +45,16 @@ public:
 	
 	static void showWindow();
 private:
+	struct ProxyOption;
+	
 	virtual void on_hide();
 	
+	void onProxyTypeChanged();
+	
+	ProxyDescriptor::ProxyType getProxyType() const;
+	
+	void setProxyType(ProxyDescriptor::ProxyType type);
+	
 	void loadConfig();
 	
 	void setCurrentListenPort(Glib::ustring const& port);
@@ -57,12 +68,15 @@ private:
 	void showWarning(Glib::ustring const& msg);
 	
 	static BasicConfigDialog* m_spInstance;
+	static ProxyOption const m_sProxyOptions[];
 	Glib::ustring m_curListenPort;
 	Gtk::Entry* m_pListenPort;
 	Gtk::CheckButton* m_pTrayAnimation;
-	Gtk::CheckButton* m_pForward;
+	Gtk::ComboBoxText* m_pProxyType;
 	Gtk::Entry* m_pProxyHost;
 	Gtk::Entry* m_pProxyPort;
+	Gtk::Entry* m_pProxyUser;
+	Gtk::Entry* m_pProxyPass;
 };
 
 #endif
