@@ -35,7 +35,6 @@
 #include <sigc++/sigc++.h>
 #include <gtkmm/window.h>
 #include <gtkmm/fixed.h>
-#include <gtkmm/drawingarea.h>
 #include <gtkmm/notebook.h>
 #include <gtkmm/textbuffer.h>
 #include <gtkmm/textview.h>
@@ -209,7 +208,7 @@ DebugConnection::DebugConnection(DebugLayout& layout, int top, int left, int rig
 	
 	Gdk::Color border_color("#ac40ef");
 	for (int i = 0; i < 4; ++i) {
-		m_borders[i].modify_bg(Gtk::STATE_NORMAL, border_color);
+		m_borders[i].setColor(border_color);
 		if (i != 3) {
 			m_borders[i].show();
 		}
