@@ -457,6 +457,9 @@ FilterConfigWindow::~FilterConfigWindow()
 {
 	assert(m_spInstance);
 	m_spInstance = 0;
+	
+	// prevent slots being called during destruction
+	sigc::trackable::notify_callbacks();
 }
 
 bool
@@ -1170,6 +1173,8 @@ FilterConfigWindow::FilterTreeModel::FilterTreeModel()
 
 FilterConfigWindow::FilterTreeModel::~FilterTreeModel()
 {
+	// prevent slots being called during destruction
+	sigc::trackable::notify_callbacks();
 }
 
 Glib::RefPtr<FilterConfigWindow::FilterTreeModel>
@@ -1575,6 +1580,8 @@ FilterConfigWindow::FilterTree::FilterTree(
 
 FilterConfigWindow::FilterTree::~FilterTree()
 {
+	// prevent slots being called during destruction
+	sigc::trackable::notify_callbacks();
 }
 
 bool
