@@ -115,6 +115,9 @@ LogDialog::~LogDialog()
 {
 	assert(m_spInstance);
 	m_spInstance = 0;
+	
+	// prevent slots being called during destruction
+	sigc::trackable::notify_callbacks();
 }
 
 void
