@@ -30,9 +30,15 @@ class TrayMenu : public Gtk::Menu
 {
 public:
 	TrayMenu();
-protected:
+private:
+	void updateActiveItemText();
+	
 	void onActive();
 	
+	void onActiveSelected();
+	
+	void onActiveDeselected();
+	
 	void onBasicConfig();
 	
 	void onAdvancedConfig();
@@ -48,8 +54,9 @@ protected:
 	void onAbout();
 	
 	void onQuit();
-private:
+	
 	Gtk::CheckMenuItem* m_pActiveItem;
+	bool m_isActiveItemSelected;
 };
 
 #endif
