@@ -44,7 +44,7 @@
 #include "IntrusivePtr.h"
 #include "RefCountableSAP.h"
 #include "AutoClosingSAP.h"
-#include "AcceptorThread.h"
+#include "WorkerThreadPool.h"
 #include "ScopedIncDec.h"
 #include "EffectiveFileTimestamps.h"
 #include "AtomicOps.h"
@@ -120,7 +120,7 @@ Application::OnInit()
 	m_confDir.AppendDir(_T("conf"));
 	m_ptrTrayIcon.reset(new TrayIcon);
 	m_ptrTrayMenu.reset(new TrayMenu);
-	m_ptrAcceptorThread.reset(new AcceptorThread);
+	m_ptrWorkerPool.reset(new WorkerThreadPool);
 	
 	bool errors = false;
 	errors |= !loadConfig();
@@ -130,14 +130,15 @@ Application::OnInit()
 	if (errors) {
 		showLogDialog();
 	}
+	m_ptrWorkerPool->start();
 	return true;
 }
 
 int
 Application::OnExit()
 {
-	// This will also finish the worker threads.
-	m_ptrAcceptorThread.reset(0);
+	// This will finish the worker threads.
+	m_ptrWorkerPool.reset(0);
 	
 	/*
 	Initialyzing these things in the constructor
@@ -364,7 +365,7 @@ Application::applyConfig(Config const& config)
 	// We apply the config independently of the result of the
 	// bind operation, to make the Basic Config dialog work as expected.
 	
-	m_ptrAcceptorThread->removeAll();
+	m_ptrWorkerPool->removeAllAcceptors();
 	setAcceptingConnections(false);
 	
 	bool ok = true;
@@ -402,7 +403,7 @@ Application::applyConfig(Config const& config)
 	}
 	
 	for (; !acceptors.empty(); acceptors.pop_front()) {
-		m_ptrAcceptorThread->add(acceptors.front());
+		m_ptrWorkerPool->addAcceptor(acceptors.front());
 	}
 	setAcceptingConnections(true);
 	
@@ -415,7 +416,7 @@ Application::applyBrokenConfig(Config const& broken_config)
 	GlobalState::WriteAccessor()->config() = broken_config;
 	// We apply it anyway, to make the Basic Config dialog work as expected.
 	
-	m_ptrAcceptorThread->removeAll();
+	m_ptrWorkerPool->removeAllAcceptors();
 	setAcceptingConnections(false);
 }
 
