@@ -47,7 +47,7 @@ class Config;
 class UrlPatterns;
 class TrayIcon;
 class TrayMenu;
-class AcceptorThread;
+class WorkerThreadPool;
 class wxIdleEvent;
 class ACE_FILE_IO;
 
@@ -220,7 +220,7 @@ private:
 	std::auto_ptr<TrayIcon> m_ptrTrayIcon;
 	std::auto_ptr<TrayMenu> m_ptrTrayMenu;
 	bool m_isAcceptingConnections;
-	std::auto_ptr<AcceptorThread> m_ptrAcceptorThread;
+	std::auto_ptr<WorkerThreadPool> m_ptrWorkerPool;
 	// Must be the last thing to construct and first to destroy
 	// because we don't want worker threads to act on destroyed objects.
 	
