@@ -33,11 +33,13 @@
 #include "ProxyDescriptor.h"
 #include "SymbolicInetAddr.h"
 #include "EffectiveFileTimestamps.h"
+#include "ArraySize.h"
 #include "types.h"
 #include <ace/config-lite.h>
 #include <boost/lexical_cast.hpp>
 #include <wx/textctrl.h>
 #include <wx/checkbox.h>
+#include <wx/choice.h>
 #include <wx/sizer.h>
 #include <wx/msgdlg.h>
 #include <list>
@@ -46,19 +48,44 @@
 
 using namespace std;
 
+struct BasicConfigDialog::ProxyOption
+{
+	ProxyDescriptor::ProxyType type;
+	wxChar const* label;
+};
+
+
+enum {
+	ID_PROXY_TYPE
+};
+
+
 BEGIN_EVENT_TABLE(BasicConfigDialog, wxDialog)
 	EVT_CLOSE(onWindowClose)
 	EVT_BUTTON(wxID_OK, onApply)
+	EVT_CHOICE(ID_PROXY_TYPE, onProxyTypeChanged)
 END_EVENT_TABLE()
 
+
 BasicConfigDialog* BasicConfigDialog::m_spInstance = 0;
 
+BasicConfigDialog::ProxyOption const BasicConfigDialog::m_sProxyOptions[] = {
+	{ ProxyDescriptor::INVALID, _T("No forwarding") },
+	{ ProxyDescriptor::HTTP, _T("HTTP proxy") },
+	{ ProxyDescriptor::SOCKS4, _T("SOCKS4 proxy") },
+	{ ProxyDescriptor::SOCKS4A, _T("SOCKS4A proxy") },
+	{ ProxyDescriptor::SOCKS5, _T("SOCKS5 proxy") }
+};
+
+
 BasicConfigDialog::BasicConfigDialog()
 :	wxDialog(wxGetApp().GetTopWindow(), -1, _T("Basic Configuration"), wxDefaultPosition),
 	m_pListenPort(0),
-	m_pForwarding(0),
+	m_pProxyType(0),
 	m_pProxyHost(0),
-	m_pProxyPort(0)
+	m_pProxyPort(0),
+	m_pProxyUser(0),
+	m_pProxyPass(0)
 {
 	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
 	SetSizer(topsizer);
@@ -69,21 +96,47 @@ BasicConfigDialog::BasicConfigDialog()
 	m_pListenPort = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(45, -1));
 	hsz1->Add(m_pListenPort, 0, wxALIGN_CENTRE|wxALL, 5);
 	
-	wxStaticBoxSizer *vsz2 = new wxStaticBoxSizer(
+	wxStaticBoxSizer *fwd_sizer = new wxStaticBoxSizer(
 		new wxStaticBox(this, -1, _T("Forwarding")),
 		wxVERTICAL
 	);
-	topsizer->Add(vsz2, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5);
-	m_pForwarding = new wxCheckBox(this, -1, _T("Forward to another proxy"));
-	vsz2->Add(m_pForwarding, 0, wxALIGN_CENTER_VERTICAL|wxALL, 3);
+	topsizer->Add(fwd_sizer, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT, 5);
+	
+	m_pProxyType = new wxChoice(this, ID_PROXY_TYPE);
+	fwd_sizer->Add(m_pProxyType, 0, wxGROW|wxALIGN_CENTER_VERTICAL);
+	for (int i = 0; i < (int)ARRAY_SIZE(m_sProxyOptions); ++i) {
+		m_pProxyType->Append(m_sProxyOptions[i].label);
+	}
+	
+	fwd_sizer->AddSpacer(3);
 	
 	wxBoxSizer *hsz21 = new wxBoxSizer(wxHORIZONTAL);
-	vsz2->Add(hsz21, 0, wxALIGN_CENTRE|wxALL, 2);
+	fwd_sizer->Add(hsz21, 0, wxALIGN_CENTRE);
 	m_pProxyHost = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(120,-1));
-	hsz21->Add(m_pProxyHost, 0, wxALIGN_CENTRE|wxALL, 5);
+	hsz21->Add(m_pProxyHost, 0, wxALIGN_CENTRE|wxRIGHT, 5);
 	hsz21->Add(new wxStaticText(this, -1, _T(":")), 0, wxALIGN_CENTRE);
 	m_pProxyPort = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(45,-1));
-	hsz21->Add(m_pProxyPort, 0, wxALIGN_CENTRE|wxALL, 5 );
+	hsz21->Add(m_pProxyPort, 0, wxALIGN_CENTRE|wxLEFT, 5);
+	
+	fwd_sizer->AddSpacer(3);
+	
+	wxFlexGridSizer* user_pass_sizer = new wxFlexGridSizer(2, 3, 3);
+	fwd_sizer->Add(user_pass_sizer, 0, wxGROW);
+	user_pass_sizer->AddGrowableCol(1, 1);
+	
+	user_pass_sizer->Add(
+		new wxStaticText(this, -1, _T("Username:")),
+		0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL
+	);
+	m_pProxyUser = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(100,-1));
+	user_pass_sizer->Add(m_pProxyUser, 0, wxGROW);
+	
+	user_pass_sizer->Add(
+		new wxStaticText(this, -1, _T("Password:")),
+		0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL
+	);
+	m_pProxyPass = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(100,-1));
+	user_pass_sizer->Add(m_pProxyPass, 0, wxGROW);
 	
 	m_pTrayAnimation = new wxCheckBox(this, -1, _T("Tray icon animation"));
 	topsizer->Add(m_pTrayAnimation, 0, wxLEFT|wxRIGHT|wxTOP, 5);
@@ -97,6 +150,7 @@ BasicConfigDialog::BasicConfigDialog()
 	
 	topsizer->SetSizeHints(this);
 	
+	setProxyType(ProxyDescriptor::INVALID);
 	loadConfig();
 }
 
@@ -128,11 +182,67 @@ BasicConfigDialog::onWindowClose(wxCloseEvent& evt)
 	Destroy();
 }
 
+void
+BasicConfigDialog::onProxyTypeChanged(wxCommandEvent& evt)
+{
+	onProxyTypeChanged(m_sProxyOptions[evt.GetSelection()].type);
+}
+
+void
+BasicConfigDialog::onProxyTypeChanged(ProxyDescriptor::ProxyType type)
+{
+	switch (type) {
+		case ProxyDescriptor::INVALID:
+		m_pProxyHost->Enable(false);
+		m_pProxyPort->Enable(false);
+		m_pProxyUser->Enable(false);
+		m_pProxyPass->Enable(false);
+		break;
+		case ProxyDescriptor::HTTP:
+		m_pProxyHost->Enable(true);
+		m_pProxyPort->Enable(true);
+		m_pProxyUser->Enable(false);
+		m_pProxyPass->Enable(false);
+		break;
+		case ProxyDescriptor::SOCKS4:
+		case ProxyDescriptor::SOCKS4A:
+		m_pProxyHost->Enable(true);
+		m_pProxyPort->Enable(true);
+		m_pProxyUser->Enable(true);
+		m_pProxyPass->Enable(false);
+		break;
+		case ProxyDescriptor::SOCKS5:
+		m_pProxyHost->Enable(true);
+		m_pProxyPort->Enable(true);
+		m_pProxyUser->Enable(true);
+		m_pProxyPass->Enable(true);
+		break;
+	}
+}
+
+ProxyDescriptor::ProxyType
+BasicConfigDialog::getProxyType() const
+{
+	return m_sProxyOptions[m_pProxyType->GetSelection()].type;
+}
+	
+void
+BasicConfigDialog::setProxyType(ProxyDescriptor::ProxyType type)
+{
+	for (int i = 0; i < (int)ARRAY_SIZE(m_sProxyOptions); ++i) {
+		if (m_sProxyOptions[i].type == type) {
+			m_pProxyType->SetSelection(i);
+			onProxyTypeChanged(type);
+			break;
+		}
+	}
+}
+
 void
 BasicConfigDialog::loadConfig()
 {
-	Config config = GlobalState::ReadAccessor()->config();
-	list<SymbolicInetAddr> listen_addrs = config.getListenAddresses();
+	Config config(GlobalState::ReadAccessor()->config());
+	list<SymbolicInetAddr> listen_addrs(config.getListenAddresses());
 	if (listen_addrs.size() > 1) {
 		setCurrentListenPort(wxEmptyString),
 		showWarning(
@@ -147,16 +257,20 @@ BasicConfigDialog::loadConfig()
 		setCurrentListenPort(wxEmptyString);
 	}
 	m_pTrayAnimation->SetValue(config.isTrayAnimationEnabled());
-	m_pForwarding->SetValue(config.isNextHopProxyEnabled());
-	ProxyDescriptor pdesc = config.getNextHopProxy();
-	m_pProxyHost->SetValue(pdesc.getProxyAddr().getHost().c_str());
-	if (pdesc.getProxyAddr().getPort() <= 0) {
+	
+	ProxyDescriptor pdesc(config.getNextHopProxy());
+	setProxyType(config.isNextHopProxyEnabled() ? pdesc.getType() : pdesc.INVALID);
+	
+	m_pProxyHost->SetValue(pdesc.getAddr().getHost().c_str());
+	if (pdesc.getAddr().getPort() <= 0) {
 		m_pProxyPort->SetValue(wxEmptyString);
 	} else {
 		wxString val;
-		val << pdesc.getProxyAddr().getPort();
+		val << pdesc.getAddr().getPort();
 		m_pProxyPort->SetValue(val);
 	}
+	m_pProxyUser->SetValue(pdesc.getUserName().c_str());
+	m_pProxyPass->SetValue(pdesc.getPassword().c_str());
 }
 
 void
@@ -183,7 +297,9 @@ BasicConfigDialog::onApply(wxCommandEvent& evt)
 		}
 	}
 	
-	bool const forwarding = m_pForwarding->GetValue();
+	ProxyDescriptor::ProxyType const proxy_type = getProxyType();
+	bool const forwarding = (proxy_type != ProxyDescriptor::INVALID);
+	
 	string proxy_host = StringUtils::trimStr(m_pProxyHost->GetValue().mb_str());	
 	if (forwarding && proxy_host.empty()) {
 		showError(_T("Remote proxy host is not given"));
@@ -202,6 +318,15 @@ BasicConfigDialog::onApply(wxCommandEvent& evt)
 		return;
 	}
 	
+	string proxy_user;
+	string proxy_pass;
+	if (proxy_type != ProxyDescriptor::HTTP || forwarding == false) {
+		proxy_user = m_pProxyUser->GetValue().mb_str();
+	}
+	if (proxy_type == ProxyDescriptor::SOCKS5 || forwarding == false) {
+		proxy_pass = m_pProxyPass->GetValue().mb_str();
+	}
+	
 	Config const old_config(GlobalState::ReadAccessor()->config());
 	Config new_config(old_config);
 	if (listen_port_changed) {
@@ -211,11 +336,17 @@ BasicConfigDialog::onApply(wxCommandEvent& evt)
 	}
 	new_config.setTrayAnimationEnabled(m_pTrayAnimation->GetValue());
 	new_config.setNextHopProxyEnabled(forwarding);
+	
 	SymbolicInetAddr proxy_addr(proxy_host, -1);
 	if (proxy_port > 0) {
 		proxy_addr.setPort(proxy_port);
 	}
-	new_config.setNextHopProxy(ProxyDescriptor(proxy_addr));
+	ProxyDescriptor proxy(old_config.getNextHopProxy());
+	proxy.setType(proxy_type);
+	proxy.setAddr(proxy_addr);
+	proxy.setUserName(proxy_user);
+	proxy.setPassword(proxy_pass);
+	new_config.setNextHopProxy(proxy);
 	
 	Application& app = wxGetApp();
 	Log* log = OperationLog::instance();
