@@ -24,6 +24,7 @@
 #include <config.h>
 #endif
 
+#include "ProxyDescriptor.h"
 #include <wx/dialog.h>
 
 class wxWindow;
@@ -32,6 +33,7 @@ class wxCloseEvent;
 class wxString;
 class wxTextCtrl;
 class wxCheckBox;
+class wxChoice;
 class Config;
 
 class BasicConfigDialog : public wxDialog
@@ -43,8 +45,18 @@ public:
 	
 	static void show();
 private:
+	struct ProxyOption;
+	
 	void onWindowClose(wxCloseEvent& evt);
 	
+	void onProxyTypeChanged(wxCommandEvent& evt);
+	
+	void onProxyTypeChanged(ProxyDescriptor::ProxyType type);
+	
+	ProxyDescriptor::ProxyType getProxyType() const;
+	
+	void setProxyType(ProxyDescriptor::ProxyType type);
+	
 	void loadConfig();
 	
 	void setCurrentListenPort(wxString const& port);
@@ -58,12 +70,15 @@ private:
 	void showWarning(wxString const& msg);
 	
 	static BasicConfigDialog* m_spInstance;
+	static ProxyOption const m_sProxyOptions[];
 	wxString m_curListenPort;
 	wxTextCtrl* m_pListenPort;
 	wxCheckBox* m_pTrayAnimation;
-	wxCheckBox* m_pForwarding;
+	wxChoice* m_pProxyType;
 	wxTextCtrl* m_pProxyHost;
 	wxTextCtrl* m_pProxyPort;
+	wxTextCtrl* m_pProxyUser;
+	wxTextCtrl* m_pProxyPass;
 	DECLARE_EVENT_TABLE()
 };
 
