@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@
 #include "RefCountable.h"
 #include "RefCounter.h"
 #include "ScopedIncDec.h"
+#include <ace/config-lite.h>
 #include <ace/Synch_Traits.h>
 #include <boost/lexical_cast.hpp>
 #include <boost/regex.hpp>
@@ -105,6 +106,10 @@ public:
 	
 	virtual ~FilterTree();
 	
+	void clear();
+	
+	void reload();
+	
 	bool isFilterGroupBeingEdited(FilterGroupTag const& group_tag);
 	
 	bool isFileItem(wxTreeItemId const& item) const;
@@ -149,9 +154,9 @@ private:
 	void onSelChanging(wxTreeEvent& evt);
 	
 	void onItemMenu(wxTreeEvent& evt);
-	
+public:
 	void onNewFile(wxCommandEvent& evt);
-	
+private:
 	void onNewFilter(wxCommandEvent& evt);
 	
 	void onFilterUp(wxCommandEvent& evt);
@@ -294,7 +299,8 @@ enum {
 	ID_MENU_DELETE_FILE,
 	ID_MENU_DELETE_FILTER,
 	ID_MENU_MOVE_UP,
-	ID_MENU_MOVE_DOWN
+	ID_MENU_MOVE_DOWN,
+	ID_MENU_RELOAD
 };
 
 BEGIN_DECLARE_EVENT_TYPES()
@@ -311,6 +317,8 @@ BEGIN_EVENT_TABLE(FilterConfigWindow, wxFrame)
 	EVT_BUTTON(wxID_FORWARD, onSave)
 	EVT_MENU(ID_MENU_DELETE_FILE, onFileDelete)
 	EVT_MENU(ID_MENU_DELETE_FILTER, onFilterDelete)
+	EVT_MENU(ID_MENU_NEW_FILE, onNewFile)
+	EVT_MENU(ID_MENU_RELOAD, onReload)
 END_EVENT_TABLE()
 
 BEGIN_EVENT_TABLE(FilterConfigWindow::FilterTree, CheckTreeView)
@@ -347,6 +355,8 @@ FilterConfigWindow::FilterConfigWindow()
 	SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
 	SetIcon(wxIcon(_T("AppIcon"), wxBITMAP_TYPE_ICO_RESOURCE, 16, 16));
 	
+	SetMenuBar(createMenuBar().release());
+	
 	wxBoxSizer* topsizer = new wxBoxSizer(wxVERTICAL);
 	SetSizer(topsizer);
 	
@@ -413,6 +423,19 @@ FilterConfigWindow::isFilterGroupBeingEdited(FilterGroupTag const& group_tag)
 	return m_spInstance->m_pFilterTree->isFilterGroupBeingEdited(group_tag);
 }
 
+std::auto_ptr<wxMenuBar>
+FilterConfigWindow::createMenuBar()
+{
+	auto_ptr<wxMenu> file(new wxMenu);
+	file->Append(ID_MENU_NEW_FILE, "New\tCtrl+N");
+	file->Append(ID_MENU_RELOAD, "Reload\tCtrl+R");
+	
+	auto_ptr<wxMenuBar> mbar(new wxMenuBar);
+	mbar->Append(file.release(), "&File");
+	
+	return mbar;
+}
+
 void
 FilterConfigWindow::onWindowActivate(wxActivateEvent& evt)
 {
@@ -461,6 +484,31 @@ FilterConfigWindow::onClearLog(wxCommandEvent& evt)
 	OperationLog::instance()->clear();
 }
 
+void
+FilterConfigWindow::onNewFile(wxCommandEvent& evt)
+{
+	m_pFilterTree->onNewFile(evt);
+}
+
+void
+FilterConfigWindow::onReload(wxCommandEvent& evt)
+{
+	if (m_pFilterTree->haveModifiedItems()) {
+		int res = wxMessageBox(
+			_T("Reload filters and lose unsaved changes?"),
+			_T("Question"),
+			wxYES_NO, this
+		);
+		if (res != wxYES) {
+			return;
+		}
+	}
+	
+	wxGetApp().loadContentFilters();
+	m_pFilterTree->reload();
+	// FilterPanel will update itself
+}
+
 void
 FilterConfigWindow::onSelChanged(wxTreeEvent& evt)
 {
@@ -536,7 +584,7 @@ FilterConfigWindow::onSave(wxCommandEvent& evt)
 		new_filter.reset(new RegexFilterDescriptor(*node->filter));
 	} else {
 		new_filter.reset(new RegexFilterDescriptor(
-			FilterTag::create(),
+			FilterTag(),
 			file_node->filterGroup->getTag(),
 			m_pFilterTree->GetItemText(item).mb_str()
 		));
@@ -825,6 +873,20 @@ FilterConfigWindow::FilterTree::~FilterTree()
 	SetImageList(0);
 }
 
+void
+FilterConfigWindow::FilterTree::clear()
+{
+	UnselectAll();
+	DeleteChildren(GetRootItem());
+}
+
+void
+FilterConfigWindow::FilterTree::reload()
+{
+	clear();
+	loadFilterGroups();
+}
+
 bool
 FilterConfigWindow::FilterTree::isFilterGroupBeingEdited(
 	FilterGroupTag const& group_tag)
@@ -1409,7 +1471,7 @@ FilterConfigWindow::FilterTree::createFile(
 	}
 	
 	IntrusivePtr<ContentFilterGroup> group(
-		new ContentFilterGroup(FilterGroupTag::create(), name)
+		new ContentFilterGroup(FilterGroupTag(), name)
 	);
 	
 	if (!saveFilterGroup(*group)) {
