@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include <config.h>
 #endif
 
-#include <ace/config.h>
+#include <ace/config-lite.h>
 #include <wx/wx.h>
 #include <wx/menu.h>
 
@@ -41,7 +41,9 @@ private:
 
 	void onConfigFilters(wxCommandEvent& evt);
 	
-	void onLog(wxCommandEvent& evt);
+	void onOperationLog(wxCommandEvent& evt);
+	
+	void onRequestLog(wxCommandEvent& evt);
 	
 	void onQuit(wxCommandEvent& evt);
 	
