@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 #include <config.h>
 #endif
 
+#define ACE_DOESNT_INSTANTIATE_NONSTATIC_OBJECT_MANAGER
+
+#include <ace/config-lite.h>
 #include <ace/ACE.h>
 #include <wx/wx.h>
 #include <windows.h>
@@ -32,9 +35,18 @@
 int WINAPI
 WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
 {
+	if (__argc == 1 && !strcmp(__argv[0], "--have-win9x-support")) {
+		// used by mkinst.py to decide on the installer name
+#if ACE_HAS_WINNT4
+		return 0;
+#else
+		return 1;
+#endif
+	}
+	
 #ifdef DEBUG
-	freopen("out.txt", "a", stdout);
-	freopen("out.txt", "a", stderr);
+	freopen("out.txt", "ab", stdout);
+	freopen("out.txt", "ab", stderr);
 	setvbuf(stdout, 0, _IONBF, 0);
 	setvbuf(stderr, 0, _IONBF, 0);
 #endif
